
;---------------------------------------------
;FUNCTION - CLEAR SCREEN PIXELS, SETS ALL THE PIXELS OF THE SCREEN BLACK (WONT CLEAR THE SPRITES)
;---------------------------------------------
CLEANSCREENPIXELS:
	;CLEAR THE SCREEN 
	;DRAW BLACK PIXELS ON THE WHOLE 320X240 PIXELS
	SetVeraAddr $100000 		;Bitmap ram $-0 00 00 (Autoinc $01)
	
	ldx #$01+1					;320x240=$12C00
	ldy #$2C+1
	lda #$00+1
TestPatternLoop:	
	stz V_D1					;WRITE $00 TO SCREEN MEMORY
	sec
	sbc #1
	bne TestPatternLoop
	dey
	bne TestPatternLoop
	dex
	bne TestPatternLoop
	
	RTS							;RETURN TO MAIN GAME LOOP
	
;---------------------------------------------
;FUNCTION - INITVARS, INITIALIZES THE VARIABLES BEFORE THE GAME STARTS
;---------------------------------------------
INITVARS:	
	STZ Z_IXH
	STZ Z_IYH					;ZERO OUT THE HIGH BYTES OF Z_IX AND Z_IY, TO PREVENT ANY PROBLEMS WHEN DRAWING SPRITES TO THE SCREEN.
	STZ VAR_GAMETICK			;ZERO OUT THE CURRENT GAME TICK, STARTING THE GAME AT 0 TICKS
	
	JSR CLOCKAPI				;CALL KERNAL CLOCK_GET_DATE_TIME. IT WILL RETURN THE CURRENT NUMBER OF JIFFY'S IN $08, LABELED CLOCKJIFFY FOR EASE
	LDA CLOCKJIFFY				;LOAD THE CURRENT JIFFY
	STA VAR_JIFFYC				;STORE THE CURRENT JIFFY AS THE STARTING POINT OF THE GAME.
	
	STZ VAR_JIFFYS				;ZERO OUT THE SHORT PAUSE FLAG, USED TO KEEP TRACK OF SHORT (2 JIFFY) OR A LONG (3 JIFFY) PAUSE PERFORMED BY THE WAITGAMETICK SUB.
	
	;RESET SPRITEDATA ARRAY 1, 2 AND 3 ($0400-$0628) ALL TO 0.
	;FIRST RESET ARRAY 1($0400-$04FF)
	LDY #0						;RESET X AS A LOOP COUNTER FOR THE LOOP
	LDA #SPRITEARRAY1H			;LOAD THE HIGH BYTE OF SPRITE DATA ARRAY 1
	STA ZPADDR+1				;STORE HIGH BYTE IN ZPADDR+1
	STZ ZPADDR					;STORE 0 IN LOW BYTE
	LDA #0						;LOAD 0 IN A
	
INITVARS_L1:
	STA (ZPADDR),Y				;STORE 0 IN 16-BIT VALUE
	INY
	BNE INITVARS_L1				;JUMP BACK TO INITVARS_L1 IF THE ZERO FLAG IS NOT SET. INX UPDATES THE Z FLAG, SO WHEN X BECOMES 0 (THE NEXT VALUE AFTER 255), THE Z-FLAG WILL BE SET.

	;NOW RESET ARRAY 2 (X IS 0, SO WE DONT NEED TO RESET IT)
	LDA #SPRITEARRAY2H			;LOAD THE HIGH BYTE OF SPRITE DATA ARRAY 1
	STA ZPADDR+1				;STORE HIGH BYTE IN ZPADDR+1
	STZ ZPADDR					;STORE 0 IN LOW BYTE
	LDA #0						;LOAD 0 IN A
INITVARS_L2:
	STA (ZPADDR),Y				;STORE 0 IN 16-BIT VALUE
	INY
	BNE INITVARS_L2				;JUMP BACK TO INITVARS_L2 IF THE ZERO FLAG IS NOT SET. INX UPDATES THE Z FLAG, SO WHEN X BECOMES 0 (THE NEXT VALUE AFTER 255), THE Z-FLAG WILL BE SET.

	;NOW LETS RESET THE FIRST 40 BYTES OF SPRITE DATA ARRAY 3 (X IS 0, SO WE DONT NEED TO RESET IT)
	LDA #SPRITEARRAY3H			;LOAD THE HIGH BYTE OF SPRITE DATA ARRAY 1
	STA ZPADDR+1				;STORE HIGH BYTE IN ZPADDR+1
	STZ ZPADDR					;STORE 0 IN LOW BYTE
	LDA #0						;LOAD 0 IN A
INITVARS_L3:
	STA (ZPADDR),Y				;STORE 0 IN 16-BIT VALUE
	INY
	CPY #40						;CHECK IF 40 BITS HAVE ALREADY BEEN ZERO'ED OUT
	BNE INITVARS_L3				;IF 40 HAVE NOT BEEN ZERO'ED OUT, THEN JUMP BACK TO INITVARS_L3
	
	;HERE, WE NEED TO RESET VERA SPRITE DATA, ACTUALLY DISABLING SPRITE 0 THROUGH 61
	LDX #0						;LOAD 0 IN X, IT'LL BE THE SPRITE/LOOP COUNTER
INITVARS_L4:			
	LDA #4
	STA VAR_SPRITESTATUS		;SET 4 IN THE SPRITE STATUS VARIABLE, TO TELL THE SETSPRITE SUB TO DISABLE THE SPRITE
	TXA							;TRANSFER X (SPRITE NUMBER) TO A, TO CONTAIN THE SPRITE NUMBER TO DISABLE
	PHX							;BACKUP X WE'LL NEED IT BACK THE WAY THEY ARE NOW.
		JSR SETSPRITE			;NOW LOAD THE SPRITE, WHICH THANKS TO VAR_SPRITESTATUS BEING 4, IT WILL DISABLE THE SPRITE IN VERA.
	PLX							;RELOAD X
	CPX #66						;CHECK IF WE'VE ALREADY DONE 61 LOOPS
	BEQ INITVARS_L4_DONE		;IF ALL 61 SPRITES HAVE BEEN DISABLED, THEN 
	INX							;IF ALL 61 SPRITES HAVE NOT BEEN DISABLED YET, THEN INCREMENT X AND DO THE NEXT ONE
	BRA INITVARS_L4				;BRANCH TO INITVARS_L4 TO DISABLE THE NEXT SPRITE.
INITVARS_L4_DONE:

	;NOW LET'S CLEAR STATUS TEXT FROM THE SCREEN
	LDA #<TITLECLEAR				
	STA z_L
	LDA #>TITLECLEAR
	STA z_H
	JSR PrintString				;CLEAR THE TITLE SCREEN TEXT (PRINT A EMPTY CHAR TO CLEAR THE LINE)
	
	;LOAD THE DATA FOR THE PLAYER
	LDA #SPRITEARRAY3H
	STA ZPADDR+1
	STZ ZPADDR
	LDY #0
	LDA #120
	STA (ZPADDR),Y				;STORE X POSITION OF THE PLAYER
	INY
	LDA #210
	STA (ZPADDR),Y				;STORE Y POSITION OF THE PLAYER
	INY
	LDA #0
	STA (ZPADDR),Y				;SET 
	INY
	LDA #2
	STA (ZPADDR),Y				;2
	INY
	LDA #4
	STA (ZPADDR),Y				;4
	LDA #0
	INY
	STA (ZPADDR),Y				;0
	
	RTS	
	
;---------------------------------------------
;FUNCTION - DRAWSPRITES, USED TO HANDLE DRAWING ALL THE ACTIVE SPRITES 
;---------------------------------------------
DRAWSPRITES:
	;---------------------------------------------
	;RUN THROUGH SPRITE DATA ARRAY 1 TO CHECK IF THE SPRITES ARE ACTIVE (0-34)
	LDA #SPRITEARRAY1H
	STA ZPADDR+8				;STORE THE HIGHT BYTE IN ZPADDR+8
	STZ ZPADDR+7				;STORE 0 IN ZPADDR+7, IT WILL BE THE LOW BYTE
	LDY #3						;SET Y TO 3, IT'LL BE THE OFFSET, POINTING TO BYTE 4 (SPRITE STATE) OF THE SPRITE DATA ARRAY
	LDX #0						;SET X TO ZERO, IT'LL BE THE LOOP COUNTER/SPRITE NUMBER
	
DRAWSPRITES_L1:
	LDA (ZPADDR+7),Y			;LOAD BYTE 4 (SPRITE STATE), OF THE SPRITE DATA ARRAY
	CMP #0						;CHECK IF THE RESULT IS 0 (AKA THE SPRITE IS DEAD/INACTIVE)
	BEQ DRAWSPRITES_L1_NEXT		;GET READY TO LOAD THE NEXT SPRITE'S DATA...
	
	;IF THE SPRITE IS ACTIVE
	STX ZPADDR+9				;STORE THE SPRITE NUMBER IN ZPADDR+9
	JSR DRAWSPRITE				;DRAW THE SPRITE ON THE SCREEN
	
DRAWSPRITES_L1_NEXT:			;CHANGE VARIABLES TO LOAD THE NEXT SPRITE'S DATA.
	INX							;INCREMENT X, INCREASING THE LOOP COUNTER/SPRITE NUMBER
	
	CPX #35						;CHECK IF WE'VE ALREADY DONE 34 LOOPS (AND ARE ABOUT TO DO THE 35TH).
	BEQ DRAWSPRITES_L1_DONE		;IF WE HAVE, THEN WE'RE DONE CHECKING, EXIT THE LOOP
	
	TYA							;TRANSFER Y TO A
	CLC
	ADC #6						;ADD 6 TO A, WE'RE GOING TO OFFSET Y BY 6
	TAY							;TRANSFER A BACK TO Y
	BRA DRAWSPRITES_L1			;ALWAYS BRANCH TO DRAWSPRITES_L1, TO LOAD THE NEXT SPRITE'S DATA

DRAWSPRITES_L1_DONE:			;DONE DRAWING SPRITE ARRAY 1 (SPRITE 0-34)

	;---------------------------------------------
	;NOW WE RUN THROUGH SPRITE DATA ARRAY 2 TO CHECK IF THE SPRITES ARE ACTIVE (35-54)
	LDA #SPRITEARRAY2H
	STA ZPADDR+8				;STORE THE HIGHT BYTE IN ZPADDR+8
	STZ ZPADDR+7				;STORE 0 IN ZPADDR+7, IT WILL BE THE LOW BYTE
	LDY #3						;SET Y TO 3, IT'LL BE THE OFFSET (POINTING TO THE 4TH BYTE OF THE SPRITE DATA ARRAY)
	LDX #0						;SET X TO ZERO, IT'LL BE THE LOOP COUNTER
	
DRAWSPRITES_L2:
	LDA (ZPADDR+7),Y			;LOAD BYTE 4 (SPRITE STATE), OF THE SPRITE DATA ARRAY
	CMP #0						;CHECK IF THE RESULT IS 0 (AKA THE SPRITE IS DEAD/INACTIVE)
	BEQ DRAWSPRITES_L2_NEXT		;GET READY TO LOAD THE NEXT SPRITE'S DATA...
	
	;IF THE SPRITE IS ACTIVE
	TXA							;TRANSFER X (SPRITE NUMBER 0-19) TO A
	CLC					
	ADC #35						;ADD 35 TO THE SPRITE NUMBER, TO SHOW THE ACTUAL NUMBER (0-19 BECOMES 35-54)
	STA ZPADDR+9				;STORE THE UPDATED SPRITE NUMBER IN ZPADDR+9
	JSR DRAWSPRITE				;DRAW THE SPRITE ON THE SCREEN
	
DRAWSPRITES_L2_NEXT:			;CHANGE VARIABLES TO LOAD THE NEXT SPRITE'S DATA.
	INX							;INCREMENT X, INCREASING THE LOOP COUNTER/SPRITE NUMBER
	
	CPX #20						;CHECK IF WE'VE ALREADY DONE 19 LOOPS (AND ARE ABOUT TO DO THE 20TH).
	BEQ DRAWSPRITES_L2_DONE		;IF WE HAVE, THEN WE'RE DONE CHECKING, EXIT THE LOOP
	
	TYA							;TRANSFER Y TO A
	CLC
	ADC #6						;ADD 6 TO A, WE'RE GOING TO OFFSET Y BY 6
	TAY							;TRANSFER A BACK TO Y
	BRA DRAWSPRITES_L2			;ALWAYS BRANCH TO DRAWSPRITES_L2, TO LOAD THE NEXT SPRITE'S DATA TO CHECK

DRAWSPRITES_L2_DONE:			;DONE DRAWING SPRITE ARRAY 2 (SPRITE 35-54)

	;---------------------------------------------
	;NOW WE RUN THROUGH SPRITE DATA ARRAY 3 TO CHECK IF THE SPRITES ARE ACTIVE (55-61)
	;FIRST WE RUN THROUGH THE PLAYER SPRITE, WHICH DOESNT NEED A LOOPING
	LDA #SPRITEARRAY3H
	STA ZPADDR+8				;STORE THE HIGHT BYTE IN ZPADDR+8
	STZ ZPADDR+7				;STORE 0 IN ZPADDR+7, IT WILL BE THE LOW BYTE
	LDY #3						;SET Y TO 3, IT'LL BE THE OFFSET (POINTING TO THE 4TH BYTE OF THE SPRITE DATA ARRAY)
	
	LDA (ZPADDR+7),Y			;LOAD BYTE 4 (SPRITE STATE), OF THE SPRITE DATA ARRAY
	CMP #0						;CHECK IF THE RESULT IS 0 (AKA THE SPRITE IS DEAD/INACTIVE)
	BEQ DRAWSPRITES_PDONE		;IF IT IS DEAD, THEN WE DONT NEED TO DRAW THE SPRITE.
	;IF THE SPRITE IS ACTIVE
	LDA #55						;LOAD 55 TO THE SPRITE NUMBER, TO SHOW THE ACTUAL NUMBER (0 BECOMES 55)
	STA ZPADDR+9				;STORE THE UPDATED SPRITE NUMBER IN ZPADDR+9
	JSR DRAWSPRITE				;DRAW THE SPRITE ON THE SCREEN
DRAWSPRITES_PDONE:	

	;---------------------------------------------
	;WE'RE DONE WITH THE PLAYER SPRITE, NOW WE CHECK THE PLAYER MISSILE AND 5 ENEMY LASER SPRITES.
	LDA #SPRITEARRAY3H
	STA ZPADDR+8				;STORE THE HIGH BYTE IN ZPADDR+8
	LDA #6
	STA ZPADDR+7				;STORE 6 IN ZPADDR+7, IT WILL BE THE LOW BYTE (STARTING AT THE PLAYER MISSILE SPRITE DATA)
	LDY #3						;SET Y TO 3, IT'LL BE THE OFFSET (POINTING TO THE 4TH BYTE OF THE SPRITE DATA ARRAY)
	LDX #1						;SET X TO 1, IT'LL BE THE LOOP COUNTER (STARTING AT 1 AS 0 IS THE PLAYER, WE ALREADY CHECKED AND DREW THAT ONE)
	
DRAWSPRITES_L3:
	LDA (ZPADDR+7),Y			;LOAD BYTE 4 (SPRITE STATE), OF THE SPRITE DATA ARRAY
	CMP #0						;CHECK IF THE RESULT IS 0 (AKA THE SPRITE IS DEAD/INACTIVE)
	BEQ DRAWSPRITES_L3_NEXT		;GET READY TO LOAD THE NEXT SPRITE'S DATA...
	
	;IF THE SPRITE IS ACTIVE
	TXA							;TRANSFER X (SPRITE NUMBER 0-7) TO A
	CLC					
	ADC #55						;ADD 55 TO THE SPRITE NUMBER, TO SHOW THE ACTUAL NUMBER (1-7 BECOMES 56-61)
	STA ZPADDR+9				;STORE THE UPDATED SPRITE NUMBER IN ZPADDR+9
	STZ VAR_SPRITESTATUS		;AND SET VAR_SPRITESTATUS TO 0, SO IT WILL PRINT A 8X8 PIXEL SPRITE TO THE SCREEN INSTEAD OF A 16X16 SPRITE.
	JSR DRAWSPRITE				;DRAW THE SPRITE ON THE SCREEN
	
DRAWSPRITES_L3_NEXT:			;CHANGE VARIABLES TO LOAD THE NEXT SPRITE'S DATA.
	INX							;INCREMENT X, INCREASING THE LOOP COUNTER/SPRITE NUMBER
	
	CPX #6						;CHECK IF WE'VE ALREADY DONE 6 LOOPS (AND ARE ABOUT TO DO THE 7TH).
	BEQ DRAWSPRITES_L3_DONE		;IF WE HAVE, THEN WE'RE DONE CHECKING, EXIT THE LOOP
	
	TYA							;TRANSFER Y TO A
	CLC
	ADC #6						;ADD 6 TO A, WE'RE GOING TO OFFSET Y BY 6
	TAY							;TRANSFER A BACK TO Y
	BRA DRAWSPRITES_L3			;ALWAYS BRANCH TO DRAWSPRITES_L2, TO LOAD THE NEXT SPRITE'S DATA TO CHECK

DRAWSPRITES_L3_DONE:			;DONE DRAWING SPRITE ARRAY 3 (SPRITE 55-61)
	RTS
	
;---------------------------------------------
;FUNCTION - DRAWSPRITE, USED TO HANDLE DRAWING OF A SINGLE SPRITE.
; THE SPRITE NUMBER IS IN ZPADDR+9, THE HIGH AND LOW BYTE OF THE SPRITE DATA ARRAY ARE STILL IN ZPADDR+7 AND ZPADDR+8. 
; AND THE OFFSET IS IN Y (POINTING TO THE 4TH BYTE OF THE SPRITE'S DATA)
;---------------------------------------------	
DRAWSPRITE:
	DEY
	DEY
	DEY							;DECREMENT Y THREE TIMES SO IT POINTS TO THE 1ST BYTE OF THE SPRITE DATA ARRAY
	
	LDA (ZPADDR+7),Y			;LOAD THE FIRST BYTE OF THE SPRITE'S DATA (X POSITION)
	STA Z_IXL					;STORE THE X POSITION TO Z_IXL FOR THE SPRITE PRINTING.
	
	INY							;INCREMENT Y BY ONE
	LDA (ZPADDR+7),Y			;LOAD THE SECOND BYTE OF THE SPRITE'S DATA (Y POSITION)
	STA Z_IYL					;STORE THE X POSITION TO Z_IXL FOR THE SPRITE PRINTING.
	
	INY
	INY							;INCREMENT Y TWICE (SO IT POINTS TO THE 4TH BYTE, THE SPRITE STATE)
	
	;HERE WE LOAD THE VERA VRAM ADDRESS OF THE SPRITE, OFFSET BY THE SPRITE ANIMATION FRAME
	;---------------------------------------------
	LDA VAR_SPRITESTATUS		;LOAD THE SPRITE DRAW STATUS INTO A
	CMP #0
	BEQ DRAWSPRITE_8X8			;IF IT'S 0, THEN WE NEED TO PRINT A 8X8 SPRITE, SO THE VERA START ADDRESS HAS TO BE CALCULATED DIFFERENTLY
	;OTHERWISE WE PRINT A 16X16 SPRITE TO THE SCREEN
	
	;NOW WE NEED TO CHECK IF THE SPRITE STATE IS DIEING INSTEAD OF ALIVE, BECAUSE IT NEEDS A DIFFERENT ADDRESS IN ZPADDR+10, Y IS STILL POINTING TO THE SPRITE STATE
	LDA (ZPADDR+7),Y			;LOAD THE 4TH BYTE OF THE SPRITE'S DATA (SPRITE STATE)
	CMP #2						;CHECK IF THE STATE OF THE SPRITE IS 2 (ALIVE)
	BEQ DRAWSPRITE_NORMAL		;IF SO, THEN LOAD THE NORMAL VERA START ADDRESS
	;IF NOT 2, IT HAS TO BE THE STATE OF 1 (AS 0=DEAD WOULD NOT TRIGGER THE DRAWSPRITE SUBROUTINE)
	LDA #16
	STA ZPADDR+10				;STORE 16 IN ZPADDR+10, SO THE VRAM START ADDRESS POINTS TO THE EXPLOSION SPRITE.
	INY
	BRA DRAWSPRITE_CONTINUE		;CONTINUE MULTIPLYING THE ANIMATION FRAME NUMBERS NOW

DRAWSPRITE_NORMAL:				;LOAD THE VRAM START ADDRESS FROM THE SPRITE DATA ARRAY'S 5TH BYTE
	INY							;INCREMENT Y AGAIN, TO POINT TO THE 5TH BYTE OF THE SPRITE DATA ARRAY, THE VRAM START ADDRESS
	LDA (ZPADDR+7),Y			;LOAD THE 5TH BYTE OF THE SPRITE'S DATA (VERA VRAM START ADDRESS)
	STA ZPADDR+10				;STORE THE VERA ADDRESS IN ZPADDR+10 FOR NOW
	
DRAWSPRITE_CONTINUE:
	INY							;INCREMENT Y AGAIN, TO POINT TO THE 6TH BYTE, THE ANIMATION FRAME NUMBER
	
	LDA (ZPADDR+7),Y			;LOAD THE 6TH BYTE OF THE SPRITE'S DATA (ANIMATION FRAME NUMBER)
	ASL							
	ASL
	STA ZPADDR+11				;STORE THE ANIMATION FRAME * 4 IN ZPADDR+11 FOR NOW
	ASL
	ASL							;MULTIPLY BY 16 (4 BITSHIFTS).
	CLC
	ADC ZPADDR+11				;ADD THE TEMP STORED ANIMATION FRAME * 4 + ANIMATION FRAME * 16, MAKING IT TIMES 20.
	
	CLC
	ADC ZPADDR+10				;ADD THE VERA VRAM START ADDRESS TO THE ANIMATION FRAME NUMBER BY 4, THIS IS THE FRAME TO LOADED
	STA Z_L						;STORE THE VERA VRAM ADDRESS TO LOAD TO Z_L (LOW BYTE OF Z_HL)
	BRA DRAWSPRITE_DRAW			;BRANCH TO DRAWSPRITE_DRAW TO DRAW THE SPRITE
	
DRAWSPRITE_8X8:
	INY							;INCREMENT Y BY 1, NOW IT'S POINTINT TO THE 5TH BYTE, THE VRAM START ADDRESS
	LDA (ZPADDR+7),Y			;LOAD THE 5TH BYTE OF THE SPRITE'S DATA (VERA VRAM START ADDRESS)
	STA ZPADDR+10				;STORE THE VERA ADDRESS IN ZPADDR+10 FOR NOW
	INY							;INCREMENT Y AGAIN, TO POINT TO THE 6TH BYTES
	
	LDA (ZPADDR+7),Y			;LOAD THE 6TH BYTE OF THE SPRITE'S DATA (ANIMATION FRAME NUMBER)
	ASL							;MULTIPLY THE ANIMATION FRAME NUMBER BY 2.
	CLC
	ADC ZPADDR+10				;ADD THE VERA VRAM START ADDRESS TO THE ANIMATION FRAME NUMBER BY 4, THIS IS THE FRAME TO LOADED
	
	STA Z_L						;STORE THE VERA VRAM ADDRESS TO LOAD TO Z_L (LOW BYTE OF Z_HL)

DRAWSPRITE_DRAW:	
	LDA #$08
	STA Z_H						;LOAD $08 AND STORE IT TO THE HIGH BYTE OF Z_HL
	LDA ZPADDR+9				;LOAD THE SPRITE NUMBER IN A, NOW WE HAVE ALL THE VARIABLES FILLED TO DRAW THE SPRITE! SO LET'S DO IT!
	
	;BACKUP A, X AND Y, WE NEED THEM BACK AFTERWARDS!
	PHA
		PHX
			PHY
				JSR SETSPRITE	;ACTUALLY DRAW THE SPRITE ON SCREEN.
			PLY					;NOW RETURN THE Y, X AND A REGISTER BACKUPS FROM THE STACK
		PLX
	PLA
	
	DEY
	DEY							;DECREMENT Y TWICE, SO IT POINTS BACK TO THE 4TH BYTE OF THE SPRITE ARRAY, JUST LIKE IT WAS WHEN WE ENTERED THE DRAWSPRITE ROUTINE
	
	;NOW BEFORE WE EXIT, WE RESET THE VAR_SPRITESTATUS BACK TO 1, TO PRINT 16X16 SPRITES, IN CASE IT GOT CHANGED TO SOMETHING ELSE TO DRAW THE LAST SPRITE
	LDA #1
	STA VAR_SPRITESTATUS
	
	RTS							;RETURN BACK TO DRAWSPRITES ROUTINE

;---------------------------------------------
;FUNCTION - UPDATEANIMATION, USED TO UPDATE THE ANIMATION FRAMES OF THE SPRITES 
;---------------------------------------------
UPDATEANIMATIONS:
	;---------------------------------------------
	;RUN THROUGH SPRITE DATA ARRAY 1 TO CHECK IF THE SPRITES ARE ACTIVE (0-34)
	LDA #SPRITEARRAY1H
	STA ZPADDR+8				;STORE THE HIGHT BYTE IN ZPADDR+8
	STZ ZPADDR+7				;STORE 0 IN ZPADDR+7, IT WILL BE THE LOW BYTE
	LDY #3						;SET Y TO 3, IT'LL BE THE OFFSET, POINTING TO BYTE 4 (SPRITE STATE) OF THE SPRITE DATA ARRAY
	LDX #0						;SET X TO ZERO, IT'LL BE THE LOOP COUNTER/SPRITE NUMBER
	
UPDATEANIMATION_L1:
	LDA (ZPADDR+7),Y			;LOAD BYTE 4 (SPRITE STATE), OF THE SPRITE DATA ARRAY
	CMP #0						;CHECK IF THE RESULT IS 0 (AKA THE SPRITE IS DEAD/INACTIVE)
	BEQ UPDATEANIMATION_L1_NEXT		;GET READY TO LOAD THE NEXT SPRITE'S DATA...

	;NOW WE NEED TO UPDATE THE SPRITES ANIMATION FRAME BY 1 FOR THE NEXT LOOP
	INY
	INY							;INCREMENT Y TWICE, NOW WE'RE POINTING TO THE 6TH BYTE OF THE SPRITE DATA (ANIMATION FRAME NUMBER)
	LDA (ZPADDR+7),Y			;LOAD BYTE 6 (ANIMATON FRAME NUMBER)
	CMP #3						;CHECK IF THE ANIMATION FRAME IS ALREADY AT 3
	BEQ UPDATEANIMATION_L1_FRAME0	;IF SO, BRANCH TO UPDATEANIMATION_L1_FRAME0
	;IF ITS >3, THEN ADD 1 TO THE ANIMATION FRAME
	CLC
	ADC #1						;ADD ONE TO THE ANIMATION FRAME
	BRA UPDATEANIMATION_L1_FRAME0_DONE	;JUMP TO UPDATEANIMATION_L1_FRAME0, TO SAVE THE NEW ANIMATION FRAME

UPDATEANIMATION_L1_FRAME0:
	;BEFORE WE SET THE ANIMATION FRAME NUMBER BACK TO 0, CHECK IF THE SPRITE WAS DIEING, IF SO IT'S DONE ALL 4 FRAMES AND IS DEAD NOW! IT CAN BE REMOVED
	DEY
	DEY							;DECREMENT Y TWICE, SO IT POINTS BACK TO THE SPRITE STATE (4TH BYTE)
	LDA (ZPADDR+7),Y			;LOAD THE SPRITE STATE.
	CMP #1						;CHECK IF THE STATE IS 1 (DIEING)
	BEQ UPDATEANIMATION_L1_DEAD	;IF IT IS, LETS KILL THE SPRITE
	;OTHERWISE IT'S ALIVE, SO CONTINUE AS NORMAL:
	INY
	INY							;INCREMENT Y TWICE, SO IT POINTS BACK TO THE 6TH BYTE, THE ANIMATION FRAME NUMBER
	LDA #0						;SET 0 TO A, TO STORE AS ANIMATION FRAME NUMBER
UPDATEANIMATION_L1_FRAME0_DONE:	
	STA (ZPADDR+7),Y			;STORE THE ANIMATION FRAME NUMBER	
	DEY
	DEY							;DECREASE Y BY 2 AGAIN, SO IT POINTS BACK TO THE 4TH BYTE OF THE SPRITE DATA ARRAY
	
UPDATEANIMATION_L1_NEXT:		;CHANGE VARIABLES TO LOAD THE NEXT SPRITE'S DATA.
	INX							;INCREMENT X, INCREASING THE LOOP COUNTER/SPRITE NUMBER
	
	CPX #35						;CHECK IF WE'VE ALREADY DONE 34 LOOPS (AND ARE ABOUT TO DO THE 35TH).
	BEQ UPDATEANIMATION_L1_DONE		;IF WE HAVE, THEN WE'RE DONE CHECKING, EXIT THE LOOP
	
	TYA							;TRANSFER Y TO A
	CLC
	ADC #6						;ADD 6 TO A, WE'RE GOING TO OFFSET Y BY 6
	TAY							;TRANSFER A BACK TO Y
	BRA UPDATEANIMATION_L1		;ALWAYS BRANCH TO UPDATEANIMATION_L1, TO LOAD THE NEXT SPRITE'S DATA

UPDATEANIMATION_L1_DEAD:		;THE SPRITE DIED AND IT HAS RUN ALL 4 ANIMATION FRAMES, LETS REMOVE IT FROM THE SCREEN
	;THE INFORMATION IS STILL IN ZPADDR+7 AND ZPADDR+8, OFFSET BY Y, THAT POINTS TO THE ANIMATION FRAME NUMBER FOR NOW
	LDA #0						;SET 0 TO A, TO STORE AS ANIMATION FRAME NUMBER
	STA (ZPADDR+7),Y			;STORE THE ANIMATION FRAME NUMBER
	DEY
	DEY							;SUBTRACT Y BY TWO, SO IT POINTS TO THE 4TH BYTE NOW, THE SPRITE STATE
	STA (ZPADDR+7),Y			;STORE 0 (DEAD) TO THE SPRITE STATE
	;BUT THAT'S NOT ALL, WE NEED TO ACTUALLY DISABLE THE SPRITE ITSELF, AS CHANGING THE STATE TO DEAD IN THE DATA ARRAY JUST STOPS IT FROM UPDATING, BUT THE SPRITE WILL STILL BE ON SCREEN.
	LDA #4
	STA VAR_SPRITESTATUS		;SET 4 IN THE SPRITE STATUS VARIABLE, TO TELL THE SETSPRITE SUB TO DISABLE THE SPRITE
	TXA							;TRANSFER X (SPRITE NUMBER) TO A, TO CONTAIN THE SPRITE NUMBER TO DISABLE
	PHX							;BACKUP X WE'LL NEED IT BACK THE WAY THEY ARE NOW.
		JSR SETSPRITE			;NOW LOAD THE SPRITE, WHICH THANKS TO VAR_SPRITESTATUS BEING 4, IT WILL DISABLE THE SPRITE IN VERA.
	PLX							;RELOAD X
	LDY #3						;LOAD Y WITH 3 AGAIN, TO POINT TO THE 4TH BYTE (SPRITE STATE)	
	BRA UPDATEANIMATION_L1_NEXT	;ALL DONE, WE CAN CONTINUE TO CHECK THE NEXT SPRITE TO UPDATE IT'S ANIMATION FRAME
	
	
UPDATEANIMATION_L1_DONE:		;DONE UPDATING ANIMATION FRAMES FOR  SPRITE ARRAY 1 (SPRITE 0-34)
	;---------------------------------------------
	;RUN THROUGH SPRITE DATA ARRAY 2 TO CHECK IF THE SPRITES ARE ACTIVE (35-54)
	LDA #SPRITEARRAY2H
	STA ZPADDR+8				;STORE THE HIGHT BYTE IN ZPADDR+8
	STZ ZPADDR+7				;STORE 0 IN ZPADDR+7, IT WILL BE THE LOW BYTE
	LDY #3						;SET Y TO 3, IT'LL BE THE OFFSET, POINTING TO BYTE 4 (SPRITE STATE) OF THE SPRITE DATA ARRAY
	LDX #0						;SET X TO ZERO, IT'LL BE THE LOOP COUNTER/SPRITE NUMBER
	
UPDATEANIMATION_L2:
	LDA (ZPADDR+7),Y			;LOAD BYTE 4 (SPRITE STATE), OF THE SPRITE DATA ARRAY
	CMP #0						;CHECK IF THE RESULT IS 0 (AKA THE SPRITE IS DEAD/INACTIVE)
	BEQ UPDATEANIMATION_L2_NEXT		;GET READY TO LOAD THE NEXT SPRITE'S DATA...
	
	;NOW WE NEED TO UPDATE THE SPRITES ANIMATION FRAME BY 1 FOR THE NEXT LOOP
	INY
	INY							;INCREMENT Y TWICE, NOW WE'RE POINTING TO THE 6TH BYTE OF THE SPRITE DATA (ANIMATION FRAME NUMBER)
	LDA (ZPADDR+7),Y			;LOAD BYTE 6 (ANIMATON FRAME NUMBER)
	CMP #3						;CHECK IF THE ANIMATION FRAME IS ALREADY AT 3
	BEQ UPDATEANIMATION_L2_FRAME0	;IF SO, BRANCH TO UPDATEANIMATION_L2_FRAME0
	;IF ITS >3, THEN ADD 1 TO THE ANIMATION FRAME
	CLC
	ADC #1						;ADD ONE TO THE ANIMATION FRAME
	BRA UPDATEANIMATION_L2_FRAME0_DONE	;JUMP TO UPDATEANIMATION_L2_FRAME0, TO SAVE THE NEW ANIMATION FRAME
UPDATEANIMATION_L2_FRAME0:
	;BEFORE WE SET THE ANIMATION FRAME NUMBER BACK TO 0, CHECK IF THE SPRITE WAS DIEING, IF SO IT'S DONE ALL 4 FRAMES AND IS DEAD NOW! IT CAN BE REMOVED
	DEY
	DEY							;DECREMENT Y TWICE, SO IT POINTS BACK TO THE SPRITE STATE (4TH BYTE)
	LDA (ZPADDR+7),Y			;LOAD THE SPRITE STATE.
	CMP #1						;CHECK IF THE STATE IS 1 (DIEING)
	BEQ UPDATEANIMATION_L2_DEAD	;IF IT IS, LETS KILL THE SPRITE
	;OTHERWISE IT'S ALIVE, SO CONTINUE AS NORMAL:
	INY
	INY							;INCREMENT Y TWICE, SO IT POINTS BACK TO THE 6TH BYTE, THE ANIMATION FRAME NUMBER
	LDA #0						;SET 0 TO A, TO STORE AS ANIMATION FRAME NUMBER
UPDATEANIMATION_L2_FRAME0_DONE:	
	STA (ZPADDR+7),Y			;STORE THE ANIMATION FRAME NUMBER	
	DEY
	DEY							;DECREASE Y BY 2 AGAIN, SO IT POINTS BACK TO THE 4TH BYTE OF THE SPRITE DATA ARRAY
	
UPDATEANIMATION_L2_NEXT:		;CHANGE VARIABLES TO LOAD THE NEXT SPRITE'S DATA.
	INX							;INCREMENT X, INCREASING THE LOOP COUNTER/SPRITE NUMBER
	
	CPX #20						;CHECK IF WE'VE ALREADY DONE 19 LOOPS (AND ARE ABOUT TO DO THE 20TH).
	BEQ UPDATEANIMATION_L2_DONE		;IF WE HAVE, THEN WE'RE DONE CHECKING, EXIT THE LOOP
	
	TYA							;TRANSFER Y TO A
	CLC
	ADC #6						;ADD 6 TO A, WE'RE GOING TO OFFSET Y BY 6
	TAY							;TRANSFER A BACK TO Y
	BRA UPDATEANIMATION_L2		;ALWAYS BRANCH TO UPDATEANIMATION_L2, TO LOAD THE NEXT SPRITE'S DATA

UPDATEANIMATION_L2_DEAD:		;THE SPRITE DIED AND IT HAS RUN ALL 4 ANIMATION FRAMES, LETS REMOVE IT FROM THE SCREEN
	;THE INFORMATION IS STILL IN ZPADDR+7 AND ZPADDR+8, OFFSET BY Y, THAT POINTS TO THE ANIMATION FRAME NUMBER FOR NOW
	LDA #0						;SET 0 TO A, TO STORE AS ANIMATION FRAME NUMBER
	STA (ZPADDR+7),Y			;STORE THE ANIMATION FRAME NUMBER
	DEY
	DEY							;SUBTRACT Y BY TWO, SO IT POINTS TO THE 4TH BYTE NOW, THE SPRITE STATE
	STA (ZPADDR+7),Y			;STORE 0 (DEAD) TO THE SPRITE STATE
	;BUT THAT'S NOT ALL, WE NEED TO ACTUALLY DISABLE THE SPRITE ITSELF, AS CHANGING THE STATE TO DEAD IN THE DATA ARRAY JUST STOPS IT FROM UPDATING, BUT THE SPRITE WILL STILL BE ON SCREEN.
	LDA #4
	STA VAR_SPRITESTATUS		;SET 4 IN THE SPRITE STATUS VARIABLE, TO TELL THE SETSPRITE SUB TO DISABLE THE SPRITE
	TXA							;TRANSFER X (SPRITE NUMBER) TO A, TO CONTAIN THE SPRITE NUMBER TO DISABLE
	CLC
	ADC #35						;ADD 35 TO THE SPRITE NUMBER (AS WE ARE IN SPRITE DATA ARRAY 2 NOW!)
	PHX							;BACKUP X WE'LL NEED IT BACK THE WAY THEY ARE NOW.
		JSR SETSPRITE			;NOW LOAD THE SPRITE, WHICH THANKS TO VAR_SPRITESTATUS BEING 4, IT WILL DISABLE THE SPRITE IN VERA.
	PLX							;RELOAD X
	LDY #3						;LOAD Y WITH 3 AGAIN, TO POINT TO THE 4TH BYTE (SPRITE STATE)	
	BRA UPDATEANIMATION_L2_NEXT	;ALL DONE, WE CAN CONTINUE TO CHECK THE NEXT SPRITE TO UPDATE IT'S ANIMATION FRAME
	
	
UPDATEANIMATION_L2_DONE:		;DONE UPDATING ANIMATION FRAMES FOR SPRITE ARRAY 2 (SPRITE 25-54)
	;---------------------------------------------
	;RUN THROUGH SPRITE DATA ARRAY 3 TO CHECK IF THE SPRITES ARE ACTIVE (55-61)
	LDA #SPRITEARRAY3H
	STA ZPADDR+8				;STORE THE HIGHT BYTE IN ZPADDR+8
	STZ ZPADDR+7				;STORE 0 IN ZPADDR+7, IT WILL BE THE LOW BYTE
	LDY #3						;SET Y TO 3, IT'LL BE THE OFFSET, POINTING TO BYTE 4 (SPRITE STATE) OF THE SPRITE DATA ARRAY
	LDX #0						;SET X TO ZERO, IT'LL BE THE LOOP COUNTER/SPRITE NUMBER
	
UPDATEANIMATION_L3:
	LDA (ZPADDR+7),Y			;LOAD BYTE 4 (SPRITE STATE), OF THE SPRITE DATA ARRAY
	CMP #0						;CHECK IF THE RESULT IS 0 (AKA THE SPRITE IS DEAD/INACTIVE)
	BEQ UPDATEANIMATION_L3_NEXT		;GET READY TO LOAD THE NEXT SPRITE'S DATA...
	
	;NOW WE NEED TO UPDATE THE SPRITES ANIMATION FRAME BY 1 FOR THE NEXT LOOP
	INY
	INY							;INCREMENT Y TWICE, NOW WE'RE POINTING TO THE 6TH BYTE OF THE SPRITE DATA (ANIMATION FRAME NUMBER)
	LDA (ZPADDR+7),Y			;LOAD BYTE 6 (ANIMATON FRAME NUMBER)
	CMP #3						;CHECK IF THE ANIMATION FRAME IS ALREADY AT 3
	BEQ UPDATEANIMATION_L3_FRAME0	;IF SO, BRANCH TO UPDATEANIMATION_L3_FRAME0
	;IF ITS >3, THEN ADD 1 TO THE ANIMATION FRAME
	CLC
	ADC #1						;ADD ONE TO THE ANIMATION FRAME
	BRA UPDATEANIMATION_L3_FRAME0_DONE	;JUMP TO UPDATEANIMATION_L3_FRAME0, TO SAVE THE NEW ANIMATION FRAME
UPDATEANIMATION_L3_FRAME0:
	;BEFORE WE SET THE ANIMATION FRAME NUMBER BACK TO 0, CHECK IF THE SPRITE WAS DIEING, IF SO IT'S DONE ALL 4 FRAMES AND IS DEAD NOW! IT CAN BE REMOVED
	DEY
	DEY							;DECREMENT Y TWICE, SO IT POINTS BACK TO THE SPRITE STATE (4TH BYTE)
	LDA (ZPADDR+7),Y			;LOAD THE SPRITE STATE.
	CMP #1						;CHECK IF THE STATE IS 1 (DIEING)
	BEQ UPDATEANIMATION_L3_DEAD	;IF IT IS, LETS KILL THE SPRITE
	;OTHERWISE IT'S ALIVE, SO CONTINUE AS NORMAL:
	INY
	INY							;INCREMENT Y TWICE, SO IT POINTS BACK TO THE 6TH BYTE, THE ANIMATION FRAME NUMBER
	LDA #0						;SET 0 TO A, TO STORE AS ANIMATION FRAME NUMBER
UPDATEANIMATION_L3_FRAME0_DONE:	
	STA (ZPADDR+7),Y			;STORE THE ANIMATION FRAME NUMBER	
	DEY
	DEY							;DECREASE Y BY 2 AGAIN, SO IT POINTS BACK TO THE 4TH BYTE OF THE SPRITE DATA ARRAY
	
UPDATEANIMATION_L3_NEXT:		;CHANGE VARIABLES TO LOAD THE NEXT SPRITE'S DATA.
	INX							;INCREMENT X, INCREASING THE LOOP COUNTER/SPRITE NUMBER
	
	CPX #7						;CHECK IF WE'VE ALREADY DONE 6 LOOPS (AND ARE ABOUT TO DO THE 7TH).
	BEQ UPDATEANIMATION_L3_DONE		;IF WE HAVE, THEN WE'RE DONE CHECKING, EXIT THE LOOP
	
	TYA							;TRANSFER Y TO A
	CLC
	ADC #6						;ADD 6 TO A, WE'RE GOING TO OFFSET Y BY 6
	TAY							;TRANSFER A BACK TO Y
	BRA UPDATEANIMATION_L3		;ALWAYS BRANCH TO UPDATEANIMATION_L3, TO LOAD THE NEXT SPRITE'S DATA
	
UPDATEANIMATION_L3_DEAD:		;THE SPRITE DIED AND IT HAS RUN ALL 4 ANIMATION FRAMES, LETS REMOVE IT FROM THE SCREEN
	;THE INFORMATION IS STILL IN ZPADDR+7 AND ZPADDR+8, OFFSET BY Y, THAT POINTS TO THE ANIMATION FRAME NUMBER FOR NOW
	LDA #0						;SET 0 TO A, TO STORE AS ANIMATION FRAME NUMBER
	STA (ZPADDR+7),Y			;STORE THE ANIMATION FRAME NUMBER
	DEY
	DEY							;SUBTRACT Y BY TWO, SO IT POINTS TO THE 4TH BYTE NOW, THE SPRITE STATE
	STA (ZPADDR+7),Y			;STORE 0 (DEAD) TO THE SPRITE STATE
	;BUT THAT'S NOT ALL, WE NEED TO ACTUALLY DISABLE THE SPRITE ITSELF, AS CHANGING THE STATE TO DEAD IN THE DATA ARRAY JUST STOPS IT FROM UPDATING, BUT THE SPRITE WILL STILL BE ON SCREEN.
	LDA #4
	STA VAR_SPRITESTATUS		;SET 4 IN THE SPRITE STATUS VARIABLE, TO TELL THE SETSPRITE SUB TO DISABLE THE SPRITE
	TXA							;TRANSFER X (SPRITE NUMBER) TO A, TO CONTAIN THE SPRITE NUMBER TO DISABLE
	CLC
	ADC #55						;ADD 55 TO THE SPRITE NUMBER (AS WE ARE IN SPRITE DATA ARRAY 3 NOW!)
	PHX							;BACKUP X WE'LL NEED IT BACK THE WAY THEY ARE NOW.
		JSR SETSPRITE			;NOW LOAD THE SPRITE, WHICH THANKS TO VAR_SPRITESTATUS BEING 4, IT WILL DISABLE THE SPRITE IN VERA.
	PLX							;RELOAD X
	LDY #3						;LOAD Y WITH 3 AGAIN, TO POINT TO THE 4TH BYTE (SPRITE STATE)	
	BRA UPDATEANIMATION_L3_NEXT	;ALL DONE, WE CAN CONTINUE TO CHECK THE NEXT SPRITE TO UPDATE IT'S ANIMATION FRAME

UPDATEANIMATION_L3_DONE:		;DONE UPDATING ANIMATION FRAMES FOR SPRITE ARRAY 3 (SPRITE 55-61)

	RTS	
	
;---------------------------------------------
;FUNCTION - MOVEENEMIES, THIS FUNCTION WILL LOOP THROUGH ALL ACTIVE ENEMIES AND MOVE THEM
;---------------------------------------------	
MOVEENEMIES:
	;HERE WE NEED TO LOOP THOUGH THE ACTIVE ENEMY SPRITES 1) CHECK IF THE MOVEMENT NEEDS TO BE CHANGED AND 2) MOVE THE SPRITES
	;MOVEMENT IS KEPT TRACK OF THANKS TO VAR_MOVE1 (DIRECTION OF MOVEMENT) AND VAR_MOVE2 (DISTANCE TRAVELLED SINCE LAST MOVEMENT CHANGE)
	;LETS RUN THROUGH SPRITE DATA ARRAY 1 AND 2 TO CHECK THE ENEMY SPRITES.
	
	STZ ZPADDR+10				;WE'RE RESETTING ZPADDR+10 TO ZERO, WE'RE GOING TO USE IT TO KEEP TRACK IF THE DIRECTION NEEDS TO BE CHANGED
	
	;---------------------------------------------	
	;CHECK IF THE ENEMIES ARE MOVING DOWN (VAR_MOVE1=1 OR 3), IF THEY ARE, CHECK HOW MANY PIXELS WE'VE MOVED ALREADY (VAR_MOVE2),
	;ENEMIES NEED TO MOVE 10 PIXELS DOWN/UP BEFORE CHANGING DIRECTION AGAIN
	LDA VAR_MOVE1				;LOAD THE ENEMIES CURRENT DIRECTION
	CMP #1						;CHECK IF THEY'RE MOVING DOWN (PREVIOUS WAS MOVING RIGHT)
	BEQ MOVEENEMIES_HOZ			;IF THEY ARE, BRANCH TO MOVEENEMIES_HOZ TO CHECK THE DISTANCE ALREADY MOVED
	;IF NOT MOVING DOWN (PREV. RIGHT), WE NEED TO CHECK IF THEY ARE MOVING DOWN (PREVIOUS DIRECTION WAS MOVING LEFT)
	CMP #3						;CHECK IF THE ENEMIES ARE MOVING DOWN (PREVIOUS DIRECTION WAS LEFT)
	BEQ MOVEENEMIES_HOZ			;IF THEY ARE, BRANCH TO MOVEENEMIES_HOZ TO CHECK HOW FAR THEY HAVE MOVED DOWN ALREADY.
	BNE MOVEENEMIES_L			;IF THEY ARE NOT MOVING DOWN AGAIN, THEN THEY ARE MOVING SIDEWAYS. BRANCH TO MOVEENEMIES_L TO CHECK THE SPRITES!
	
MOVEENEMIES_HOZ:				;THE ENEMIES ARE MOVING UP/DOWN, CHECK HOW FAR THEY HAVE ALREADY TRAVELLED
	LDA VAR_MOVE2				;LOAD THE DISTANCE TRAVELLED SINCE THE LAST MOVE CHANGE
	CMP #10						;CHECK IF THE ENEMIES HAVE ALREADY MOVED 10 PIXELS DOWN, IF SO, SET ZPADDR+10 TO LET THE SUB KNOW WE NEED TO CHANGE DIRECTION!
	BNE MOVEENEMIES_L2_DONE		;IF IT'S NOT MOVED 10 PIXELS YET, THEN WE WON'T SET ANYTHING, AND JUST MOVE THE ENEMIES
	LDA #1
	STA ZPADDR+10				;OTHERWISE WE SET ZPADDR+10 TO LET IT KNOW TO CHANGE DIRECTION
	BNE MOVEENEMIES_L2_DONE

MOVEENEMIES_L:					;IF THEY ARE NOT MOVING UP OR DOWN, THEY ARE MOVING SIDEWAYS, WE NEED TO CHECK THE SPRITES...
	;---------------------------------------------	
	;RUN THROUGH SPRITE DATA ARRAY 1 TO CHECK IF THE SPRITES ARE ACTIVE (0-34)
	LDA #SPRITEARRAY1H
	STA ZPADDR+8				;STORE THE HIGHT BYTE IN ZPADDR+8
	STZ ZPADDR+7				;STORE 0 IN ZPADDR+7, IT WILL BE THE LOW BYTE
	LDY #3						;SET Y TO 3, IT'LL BE THE OFFSET (POINTING TO THE 4TH BYTE IN THE SPRITE DATA ARRAY)
	LDX #0						;SET X TO ZERO, IT'LL BE THE LOOP COUNTER/SPRITE NUMBER
	
MOVEENEMIES_L1:
	LDA (ZPADDR+7),Y			;LOAD BYTE 4 (SPRITE STATE), OF THE SPRITE DATA ARRAY
	CMP #2						;CHECK IF THE RESULT IS LESS THEN 2 (AKA THE SPRITE IS DEAD/INACTIVE=0 OR DIEING=1)
	BNE MOVEENEMIES_L1_NEXT		;IF 0 OR 1, THEN GET READY TO LOAD THE NEXT SPRITE'S DATA...	
	;IF THE SPRITE IS ACTIVE
	STX ZPADDR+9				;STORE THE SPRITE NUMBER IN ZPADDR+9
	JSR CHECKENEMYPOS			;GO TO CHECKENEMYPOS SUB, TO CHECK IF THE DIRECTION NEEDS TO CHANGE IF THE ENEMY IS AT THE LEFT OR RIGHT MOST EDGE OF THE SCREEN
	LDA ZPADDR+10				;LOAD ZPADDR+10, CHECK IF THE MOVEMENT NEEDS TO BE CHANGED BASED ON THE LAST SPRITE CHECKED (0=NO,1=YES)
	CMP #1
	BEQ MOVEENEMIES_L2_DONE		;IF THE VALUE IS 1, THEN WE NEED TO CHANGE DIRECTION, SKIP OVER CHECKING THE REST OF THE SPRITES IN SPRITE DATA ARRAY 1 AND 2
	;IF WE DON'T NEED TO MOVE DIRECTION YET, CHECK THE NEXT SPRITE
MOVEENEMIES_L1_NEXT:			;CHANGE VARIABLES TO LOAD THE NEXT SPRITE'S DATA.
	INX							;INCREMENT X, INCREASING THE LOOP COUNTER/SPRITE NUMBER
	
	CPX #35						;CHECK IF WE'VE ALREADY DONE 34 LOOPS (AND ARE ABOUT TO DO THE 35TH).
	BEQ MOVEENEMIES_L1_DONE		;IF WE HAVE, THEN WE'RE DONE CHECKING, EXIT THE LOOP
	
	TYA							;TRANSFER Y TO A
	CLC
	ADC #6						;ADD 6 TO A, WE'RE GOING TO OFFSET Y BY 6
	TAY							;TRANSFER A BACK TO Y
	BRA MOVEENEMIES_L1			;ALWAYS BRANCH TO MOVEENEMIES_L1, TO LOAD THE NEXT SPRITE'S DATA

MOVEENEMIES_L1_DONE:			;DONE CHECKING SPRITE ARRAY 1 (SPRITE 0-34)
	
	;---------------------------------------------	
	;RUN THROUGH SPRITE DATA ARRAY 2 TO CHECK IF THE SPRITES ARE ACTIVE (35-54)
	LDA #SPRITEARRAY2H			;LOAD THE HIGH SPRITE OF SPRITE DATA ARRAY 2
	STA ZPADDR+8				;STORE THE HIGHT BYTE IN ZPADDR+8
	STZ ZPADDR+7				;STORE 0 IN ZPADDR+7, IT WILL BE THE LOW BYTE
	LDY #3						;SET Y TO 3, IT'LL BE THE OFFSET (POINTING TO THE 4TH BYTE IN THE SPRITE DATA ARRAY)
	LDX #0						;SET X TO ZERO, IT'LL BE THE LOOP COUNTER/SPRITE NUMBER
	
MOVEENEMIES_L2:
	LDA (ZPADDR+7),Y			;LOAD BYTE 4 (SPRITE STATE), OF THE SPRITE DATA ARRAY
	CMP #2						;CHECK IF THE RESULT IS LESS THEN 2 (AKA THE SPRITE IS DEAD/INACTIVE=0 OR DIEING=1)
	BNE MOVEENEMIES_L2_NEXT		;IF 0 OR 1, THEN GET READY TO LOAD THE NEXT SPRITE'S DATA...	
	
	;IF THE SPRITE IS ACTIVE
	TXA							;TRANSFER THE SPRITE NUMBER FROM X TO A
	CLC
	ADC #35						;ADD 35 TO IT, SO IT POINTS TO THE CORRECT SPRITE NUMBER
	STA ZPADDR+9				;STORE THE SPRITE NUMBER IN ZPADDR+9
	
	JSR CHECKENEMYPOS			;GO TO CHECKENEMYPOS SUB, TO CHECK IF THE DIRECTION NEEDS TO CHANGE IF THE ENEMY IS AT THE LEFT OR RIGHT MOST EDGE OF THE SCREEN
	LDA ZPADDR+10				;LOAD ZPADDR+10, CHECK IF THE MOVEMENT NEEDS TO BE CHANGED BASED ON THE LAST SPRITE CHECKED (0=NO,1=YES)
	CMP #1
	BEQ MOVEENEMIES_L2_DONE		;IF THE VALUE IS 1, THEN WE NEED TO CHANGE DIRECTION, SKIP OVER CHECKING THE REST OF THE SPRITES IN SPRITE DATA ARRAY 1 AND 2
	;IF WE DON'T NEED TO MOVE DIRECTION YET, CHECK THE NEXT SPRITE
MOVEENEMIES_L2_NEXT:			;CHANGE VARIABLES TO LOAD THE NEXT SPRITE'S DATA.
	INX							;INCREMENT X, INCREASING THE LOOP COUNTER/SPRITE NUMBER
	
	CPX #20						;CHECK IF WE'VE ALREADY DONE 19 LOOPS (AND ARE ABOUT TO DO THE 20TH).
	BEQ MOVEENEMIES_L2_DONE		;IF WE HAVE, THEN WE'RE DONE CHECKING, EXIT THE LOOP
	
	TYA							;TRANSFER Y TO A
	CLC
	ADC #6						;ADD 6 TO A, WE'RE GOING TO OFFSET Y BY 6
	TAY							;TRANSFER A BACK TO Y
	BRA MOVEENEMIES_L2			;ALWAYS BRANCH TO MOVEENEMIES_L2, TO LOAD THE NEXT SPRITE'S DATA
	
	
MOVEENEMIES_L2_DONE:			;ALSO DONE CHECKING SPRITE DATA ARRAY 2
	;---------------------------------------------	
	;NOW WE NEED TO CHECK IF THE DIRECTION NEEDS TO BE CHANGED BEFORE WE MOVE ANYTHING!
	LDA ZPADDR+10				;LOAD ZPADDR+10, IT WILL CONTAIN THE FLAG TO CHANGE DIRECTION (0=NO,1=YES)
	CMP #0						;CHECK IF WE DON'T NEED TO CHANGE DIRECTION (ZPADDR+10 = 0)
	BEQ MOVEENEMIES_MOVE		;IF THE DIRECTION DOES NOT NEED TO CHANGE, THEN BRANCH TO MOVEENEMIES_MOVE TO ACTUALLY MOVE THE ENEMIES
	;IF THE DIRECTION DOES NEED TO CHANGE, WE NEED TO UPDATE IT
	LDA VAR_MOVE1				;LOAD THE CURRENT DIRECTION IN A
	CLC
	ADC #1						;ADD ONE TO THE DIRECTION
	AND #%00000011				;APPLY A BITMASK, LEAVING ONLY THE BUTTOM 2 BITS, SO THE VALUE CAN ONLY BE BETWEEN 0 AND 3.
	STA VAR_MOVE1				;AND STORE THE NEW DIRECTION IN VAR_MOVE1
	STZ VAR_MOVE2				;RESET VAR_MOVE2 (THE NUMBER OF PIXELS TRAVELLED SINCE LAST DIRECTION CHANGE) TO 0
	
MOVEENEMIES_MOVE:				;HERE WE'RE GOING TO GO THROUGH ALL ACTIVE ENEMIES AND MOVE THEM 1 PIXEL
	;---------------------------------------------	
	; SO LETS START WITH SPRITE DATA ARRAY 1, CHECK IF ANY ENEMIES ARE ACTIVE, AND IF SO, MOVE THEM 1 PIXEL ACCORDINGLY!
	LDA #SPRITEARRAY1H
	STA ZPADDR+8				;STORE THE HIGHT BYTE IN ZPADDR+8
	STZ ZPADDR+7				;STORE 0 IN ZPADDR+7, IT WILL BE THE LOW BYTE
	LDY #3						;SET Y TO 3, IT'LL BE THE OFFSET (POINTING TO THE 4TH BYTE OF THE SPRITE'S DATA)
	LDX #0						;SET X TO ZERO, IT'LL BE THE LOOP COUNTER/SPRITE NUMBER
	
MOVEENEMIES_MOVE_L1:
	LDA (ZPADDR+7),Y			;LOAD BYTE 4 (SPRITE STATE), OF THE SPRITE DATA ARRAY	
	CMP #2						;CHECK IF THE RESULT IS ANYTHING ELSE THEN 2, SO 0 (AKA THE SPRITE IS DEAD/INACTIVE) OR 1 (DIEING)
	BNE MOVEENEMIES_MOVE_L1_NEXT	;IF IT IS DEAD, DIEING OR INACTIVE, THEN NO NEED TO UPDATE IT, JUST MOVE ON TO THE NEXT SPRITE.
	;IF IT'S NOT DEAD/INACTIVE, THEN WE CAN UPDATE THE ENEMIES POSITION.
	JSR MOVEENEMY				;GO AND MOVE THE ENEMY'S POSITION.
	
MOVEENEMIES_MOVE_L1_NEXT:		;CHANGE VARIABLES TO LOAD THE NEXT SPRITE'S DATA.
	INX							;INCREMENT X, INCREASING THE LOOP COUNTER/SPRITE NUMBER
	
	CPX #35						;CHECK IF WE'VE ALREADY DONE 34 LOOPS (AND ARE ABOUT TO DO THE 35TH).
	BEQ MOVEENEMIES_MOVE_L1_DONE	;IF WE HAVE, THEN WE'RE DONE CHECKING, EXIT THE LOOP
	
	TYA							;TRANSFER Y TO A
	CLC
	ADC #6						;ADD 6 TO A, WE'RE GOING TO OFFSET Y BY 6
	TAY							;TRANSFER A BACK TO Y
	BRA MOVEENEMIES_MOVE_L1		;ALWAYS BRANCH TO MOVEENEMIES_L1, TO LOAD THE NEXT SPRITE'S DATA

MOVEENEMIES_MOVE_L1_DONE:		;DONE MOVING THE SPRITES IN SPRITE DATA ARRAY 1

	;---------------------------------------------	
	; NOW MOVE SPRITE DATA ARRAY 2, CHECK IF ANY ENEMIES ARE ACTIVE, AND IF SO, MOVE THEM 1 PIXEL ACCORDINGLY!
	LDA #SPRITEARRAY2H
	STA ZPADDR+8				;STORE THE HIGHT BYTE IN ZPADDR+8
	STZ ZPADDR+7				;STORE 0 IN ZPADDR+7, IT WILL BE THE LOW BYTE
	LDY #3						;SET Y TO 3, IT'LL BE THE OFFSET (POINTING TO THE 4TH BYTE OF THE SPRITE'S DATA)
	LDX #0						;SET X TO ZERO, IT'LL BE THE LOOP COUNTER/SPRITE NUMBER
	
MOVEENEMIES_MOVE_L2:
	LDA (ZPADDR+7),Y			;LOAD BYTE 4 (SPRITE STATE), OF THE SPRITE DATA ARRAY	
	CMP #2						;CHECK IF THE RESULT IS ANYTHING ELSE THEN 2, SO 0 (AKA THE SPRITE IS DEAD/INACTIVE) OR 1 (DIEING)
	BNE MOVEENEMIES_MOVE_L2_NEXT	;IF IT IS DEAD, DIEING OR INACTIVE, THEN NO NEED TO UPDATE IT, JUST MOVE ON TO THE NEXT SPRITE.
	;IF IT'S NOT DEAD/INACTIVE, THEN WE CAN UPDATE THE ENEMIES POSITION.
	JSR MOVEENEMY				;GO AND MOVE THE ENEMY'S POSITION.
	
MOVEENEMIES_MOVE_L2_NEXT:		;CHANGE VARIABLES TO LOAD THE NEXT SPRITE'S DATA.
	INX							;INCREMENT X, INCREASING THE LOOP COUNTER/SPRITE NUMBER
	
	CPX #20						;CHECK IF WE'VE ALREADY DONE 19 LOOPS (AND ARE ABOUT TO DO THE 20TH).
	BEQ MOVEENEMIES_MOVE_L2_DONE	;IF WE HAVE, THEN WE'RE DONE CHECKING, EXIT THE LOOP
	
	TYA							;TRANSFER Y TO A
	CLC
	ADC #6						;ADD 6 TO A, WE'RE GOING TO OFFSET Y BY 6
	TAY							;TRANSFER A BACK TO Y
	BRA MOVEENEMIES_MOVE_L2		;ALWAYS BRANCH TO MOVEENEMIES_L1, TO LOAD THE NEXT SPRITE'S DATA

MOVEENEMIES_MOVE_L2_DONE:		;DONE MOVING THE SPRITES IN SPRITE DATA ARRAY 1


	LDA VAR_MOVE2				;LOAD VAR_MOVE2 (DISTANCE TRAVELLED SINCE LAST DIRECTION CHANGE)
	CLC
	ADC #1						;ADD 1 TO THE DISTANCE
	STA VAR_MOVE2				;AND STORE AGAIN IN VAR_MOVE2
	RTS
	
;---------------------------------------------
;FUNCTION - MOVEENEMY, HANDLES THE ACTUAL MOVING OF A SINGLE ENEMY
;  ZPADDR+7 AND ZPADDR+8 POINT TO THE SPRITE DATA ARRAY'S POSITION (LOW AND HIGH BYTE), Y IS THE OFFSET TO POINT TO THE 4TH BYTE (SPRITE STATE)
;---------------------------------------------		
MOVEENEMY:
	;LETS CHECK WHICH DIRECTION TO MOVE THE ENEMY TO.
	LDA VAR_MOVE1				;LOAD THE DIRECTION OF THE ENEMIES	
	CMP #0						;CHECK IF THE ENEMY NEEDS TO MOVE TO THE RIGHT
	BEQ MOVEENEMY_R				;BRANCH TO MOVEENEMY_R.
	CMP #1						;CHECK IF THE ENEMY NEEDS TO MOVE DOWN (PREVIOUS DIRECTION WAS RIGHT)
	BEQ MOVEENEMY_D				;BRANCH TO MOVEENEMY_D
	CMP #2						;CHECK IF THE ENEMY NEEDS TO MOVE TO THE LEFT
	BEQ MOVEENEMY_L				;MOVE THE ENEMY TO THE LEFT
	;IF NOT, THEN THE DIRECTION IS 3 (DOWN, PREVIOUS WAS LEFT), SO WE GO TO MOVE DOWN AS WELL...
MOVEENEMY_D:					;MOVE THE ENEMY DOWN.
	DEY
	DEY							;DECREMENT Y TWICE, SO IT POINTS TO THE 2ND BYTE IN THE SPRITE DATA ARRAY (Y POSITION)
	LDA (ZPADDR+7),Y			;LOAD BYTE 2 (Y POSITION), OF THE SPRITE DATA ARRAY
	CLC
	ADC #1						;ADD 1 TO THE Y POSITION
	STA (ZPADDR+7),Y			;AND STORE THE Y POSITION TO THE DATA ARRAY AGAIN!
	INY
	INY							;INCREMENT Y TWICE, SO IT POINTS BACK TO THE 4TH BYTE OF THE SPRITE DATA ARRAY
	BRA MOVEENEMY_DONE			;BRANCH TO THE END OF THE SUBROUTINE, WE'RE DONE MOVING!
	
MOVEENEMY_R:					;MOVE THE ENEMY TO THE RIGHT
	DEY
	DEY
	DEY							;DECREMENT Y THREE TIMES, SO IT POINTS TO THE 1ST BYTE IN THE SPRITE DATA ARRAY (X POSITION)
	LDA (ZPADDR+7),Y			;LOAD BYTE 1 (X POSITION), OF THE SPRITE DATA ARRAY
	CLC
	ADC #1						;ADD 1 TO THE X POSITION
	STA (ZPADDR+7),Y			;AND STORE THE X POSITION TO THE DATA ARRAY AGAIN!	
	INY
	INY
	INY							;INCREMENT Y THREE TIMES, SO IT POINTS BACK TO THE 4TH BYTE OF THE SPRITE DATA ARRAY
	BRA MOVEENEMY_DONE			;BRANCH TO THE END OF THE SUBROUTINE, WE'RE DONE MOVING!

MOVEENEMY_L:					;MOVE THE ENEMY TO THE LEFT
	DEY
	DEY
	DEY							;DECREMENT Y THREE TIMES, SO IT POINTS TO THE 1ST BYTE IN THE SPRITE DATA ARRAY (X POSITION)
	LDA (ZPADDR+7),Y			;LOAD BYTE 1 (X POSITION), OF THE SPRITE DATA ARRAY
	SEC
	SBC #1						;SUBTRACT 1 FROM THE X POSITION
	STA (ZPADDR+7),Y			;AND STORE THE X POSITION TO THE DATA ARRAY AGAIN!
	INY
	INY
	INY							;INCREMENT Y THREE TIMES, SO IT POINTS BACK TO THE 4TH BYTE OF THE SPRITE DATA ARRAY
MOVEENEMY_DONE:					;HERE WE'RE DONE MOVING 
	RTS
	
;---------------------------------------------
;FUNCTION - CHECKENEMYPOS, HERE WE'RE GOING TO CHECK IF THE ENEMY NEEDS TO CHANGE DIRECTION BASED ON THE X POSITION AT THE LEFT MOST OR RIGHT MOST POSITION.
;  SPRITE NUMBER IS IN ZPADDR+9, ZPADDR+7 AND ZPADDR+8 POINT TO THE SPRITE DATA ARRAY'S POSITION (LOW AND HIGH BYTE), Y IS THE OFFSET TO POINT TO THE 4TH BYTE (SPRITE STATE)
;---------------------------------------------		
CHECKENEMYPOS:
	;WE NEED TO CHECK THE SPRITE'S POSITION BASED ON THE CURRENT DIRECTION THE ENEMIES ARE MOVING IN.
	;VAR_MOVE1 = 0 MEANS THE ENEMIES ARE MOVING TO THE RIGHT
	;VAR_MOVE1 = 1 MEANS THE ENEMIES ARE MOVING DOWN (WAS RIGHT BEFORE)
	;VAR_MOVE1 = 2 MEANS THE ENEMIES ARE MOVING TO THE LEFT
	;VAR_MOVE1 = 3 MEANS THE ENEMIES ARE MOVING DOWN (WAS LEFT BEFORE)
	;WE DON'T HAVE TO CHECK IF VAR_MOVE1 = 1 OR 3, AS DIRECTION CHANGE WHEN MOVING UP OR DOWN IS CONTROLLED BY THE AMOUNT MOVED, NOT THE POSITION OF THE SPRITES.
	
	DEY
	DEY
	DEY							;DECREMENT Y THREE TIMES, SO IT POINTS TO THE 1ST BYTE OF THE SPRITE DATA ARRAY (X POSITION)
	;FIRST LETS CHECK VAR_MOVE1 = 0 (MOVING TO THE RIGHT)
	LDA VAR_MOVE1				;LOAD THE DIRECTION THE ENEMIES ARE MOVING IN
	CMP #0						;COMPARE TO 0 (MOVING TO THE RIGHT)
	BEQ CHECKENEMYPOS_R			;IF IT IS, BRANCH TO CHECKENEMYPOS_R TO HANDLE CHECKING ENEMIES MOVING TO THE RIGHT
	
	;IF IT ISN'T, THE ENEMIES ARE MOVING TO THE LEFT (AS IF WE GET TO THE CHECKENEMYPOS SUBROUTINE, IT CAN ONLY BE RIGHT OR LEFT, NOT MOVING DOWN)
	LDA (ZPADDR+7),Y			;LOAD THE SPRITE'S X POSITION FROM THE SPRITE DATA ARRAY
	CMP #0						;CHECK IF THE SPRITE IS ALREADY AT THE LEFT MOST EDGE OF THE SCREEN (0)
	BNE CHECKENEMYPOS_DONE		;IF IT ISN'T THEN WE DON'T NEED TO KEEP CHECKING THIS SPRITE, MOVE TO CHECK THE NEXT SPRITE IN THE LIST.
	;IF WE ARE AT THE LEFT MOST EDGE, WE NEED TO LET THE MOVEENEMIES SUB KNOW WE ARE.
	LDA #1
	STA ZPADDR+10				;SET ZPADDR+10 TO 1, TO SIGNAL THAT THE DIRECTION NEEDS TO CHANGE!
	BRA CHECKENEMYPOS_DONE		;NOW BRANCH OVER TO CHECKENEMYPOS_DONE, WE'RE DONE CHECKING THE SPRITE.
	
CHECKENEMYPOS_R:				;HERE WE NEED TO DO CHECKS IF THE ENEMIES ARE MOVING TO THE RIGHT
	LDA (ZPADDR+7),Y			;LOAD THE SPRITE'S X POSITION FROM THE SPRITE DATA ARRAY	
	CMP #247					;CHECK IF THE SPRITE'S POSITION IS ALREADY AT THE RIGHT MOST EDGE (X=247), AS WE NEED TO RESERVE 8 PIXELS FOR THE LAUNCHING OF ENEMY LASERS
	BNE CHECKENEMYPOS_DONE		;IF IT ISNT AT THE EDGE, WE'RE DONE CHECKING THE SPRITE, RETURN BACK TO CHECK THE NEXT SPRITE
	
	;IF WE ARE AT THE RIGHT MOST EDGE, WE NEED TO LET THE MOVEENEMIES SUB KNOW WE ARE...
	LDA #1
	STA ZPADDR+10				;SET ZPADDR+10 TO 1, TO SIGNAL THAT THE DIRECTION NEEDS TO CHANGE!
	
CHECKENEMYPOS_DONE:				;DONE CHECKING THE X POSITION OF THE SPRITE	
	INY
	INY
	INY							;INCREMENT Y BY 3 AGAIN, SO IT POINTS BACK TO THE 4TH BYTE OF THE SPRITE DATA ARRAY
	RTS							;RETURN BACK TO MOVEENEMIES SUBROUTINE

;---------------------------------------------
;FUNCTION - HANDLEUSERINPUT, CHECKS THE MOVEMENT CURSORS USED BY THE PLAYER 
;---------------------------------------------		
HANDLEUSERINPUT:
	;BASICALLY, WE NEED TO CHECK HERE IF THE PLAYER WANTS TO MOVE TO THE RIGHT, LEFT, OR FIRE A MISSILE.
	;JSR GETIN					;READ KEYBOARD USING A KERNAL CALL; BE AWARE! THIS WILL RESET X
	LDA #0						;LOAD 0 IN A
	JSR JOYSTICK_GET			;COMMANDER X16 OS CALL, GET THE STATE OF JOYSTICK 1
	STA ZPADDR					;STORE THE PRESSED BUTTONS IN ZPADDR FOR NOW
	AND #1						;APPLY BITMASK, LEAVING ONLY BIT 0, OR IF RIGHT BUTTON IS PRESSED
	BEQ HANDLEUSERINPUT_CHECKR	;IF THE RIGHT BUTTON HAS BEEN PRESSED, GO TO HANDLE IT
	
	LDA ZPADDR					;RELOAD THE BUTTONS PRESSED IN JOYSTICK 1
	AND #2						;APPLY BITMASK, LEAVING ONLY BIT 1, OR IF THE LEFT BUTTON IS PRESSED
	BEQ HANDLEUSERINPUT_CHECKL	;IF THE LEFT BUTTON HAS BEEN PRESSED, GO TO HANDLE IT
	BRA HANDLEUSERINPUT_NOKEY	;IF NOTHING IS PRESSED, THEN MOVE THE ACCELERATION BACK TO NEUTRAL/NOTHING
	
HANDLEUSERINPUT_NOKEY:			;IF NO KEY HAS BEEN PRESSED
	LDA VAR_PLAYERACCEL			;LOAD THE PLAYERS ACCELERATION
	CMP #2						;CHECK IF IT'S 2 ALREADY (X ACCEL = 0, NOTHING)
	BEQ HANDLEUSERINPUT_MOVEPLAYER	;IF IT IS ALREADY AT MAXIMUM LEFT ACCELERATION, THEN DO NOTHING
	BCC HANDLEUSERINPUT_NOKEYMORE	;IF ITS LESS THEN 2 (SO 0 OR 1), THEN WE NEED TO ADD ONE.
	;OTHERWISE ITS 3 OR 4 (+1/+2), SO NEED TO SUBTRACT ONE
	SEC
	SBC #1						;SUBTRACT THE ACCELATION BY ONE.
	STA VAR_PLAYERACCEL			;AND STORE THE ACCELERATION
	BRA HANDLEUSERINPUT_MOVEPLAYER	;NOW MOVE THE PLAYER'S X POSITION
	
HANDLEUSERINPUT_NOKEYMORE:
	CLC
	ADC #1						;ADD ONE TO THE PLAYER ACCELERATION
	STA VAR_PLAYERACCEL			;AND STORE THE ACCELERATION
	BRA HANDLEUSERINPUT_MOVEPLAYER	;NOW MOVE THE PLAYER'S X POSITION
	
HANDLEUSERINPUT_CHECKL
	;---------------------------------------------		
	LDA VAR_PLAYERACCEL			;LOAD THE PLAYERS ACCELERATION
	CMP #0						;CHECK IF IT'S 0 ALREADY (X ACCEL = -2)
	BEQ HANDLEUSERINPUT_MOVEPLAYER	;IF IT IS ALREADY AT MAXIMUM LEFT ACCELERATION, THEN DO NOTHING
	SEC
	SBC #1						;IF IT'S NOT AT MAX, THEN SUBTRACT THE ACCELATION BY ONE.
	STA VAR_PLAYERACCEL			;AND STORE THE ACCELERATION
	BRA HANDLEUSERINPUT_MOVEPLAYER	;NOW MOVE THE PLAYER'S X POSITION
	
HANDLEUSERINPUT_CHECKR
	;---------------------------------------------		
	CMP #$1D					;IF THE CURSOR RIGHT BUTTON IS PRESSED ($1D OR DEC 29)
	LDA VAR_PLAYERACCEL			;LOAD THE PLAYERS ACCELERATION
	CMP #4						;CHECK IF IT'S 0 ALREADY (X ACCEL = +2)
	BEQ HANDLEUSERINPUT_MOVEPLAYER	;IF IT IS ALREADY AT MAXIMUM LEFT ACCELERATION, THEN DO NOTHING
	CLC
	ADC #1						;IF IT'S NOT AT MAX, THEN ADD ONE TO THE ACCELATION.
	STA VAR_PLAYERACCEL			;AND STORE THE ACCELERATION
	
HANDLEUSERINPUT_MOVEPLAYER:		;ACTUALLY MOVE THE PLAYER HERE	
	;---------------------------------------------		
	LDA VAR_PLAYERACCEL			;LOAD THE PLAYERS ACCELERATION
	CMP #2						;CHECK IF IT'S 2 (NEUTRAL), THEN WE DO NOTHING
	BEQ HANDLEUSERINPUT_CHECKFIRE
	BCC HANDLEUSERINPUT_MOVEPLAYERL	;IF ITS LESS THEN 2, SO 0 OR 1, THEN WE NEED TO MOVE LEFT
	;OTHERWISE, THE PLAYER NEEDS TO MOVE TO THE RIGHT
	CMP #3						;CHECK IF THE MOVEMENT IS 3, OR +1
	BEQ HANDLEUSERINPUT_MOVEPLAYERR1
	;OTHERWISE MOVE THE PLAYER X TWO TO THE RIGHT
	LDA #SPRITEARRAY3H			;LOAD THE HIGH BYTE OF SPRITE DATA ARRAY 3
	STA ZPADDR+1				;STORE IN ZPADDR+1, IT WILL BE THE HIGH BYTE
	STZ ZPADDR					;STORE 0 IN THE LOW BYTE (ZPADDR)
	LDA (ZPADDR)				;LOAD THE FIRST BYTE OF SPRITE 55, THE PLAYER'S X POSITION
	CMP #246					;CHECK IF X IS 246 OR MORE. IF SO WE CANT MOVE, SO DO NOTHING (246 +2 PIXELS WOULD PUT IT OVER THE 248 LIMIT (WE NEED TO KEEP IT 8 LESS THEN 256 AS THE MISSILE NEEDS TO BE LAUNCHED)
	BCS HANDLEUSERINPUT_CHECKFIRE	;BRANCH TO HANDLEUSERINPUT_CHECKFIRE IF WE CANT MOVE
	;OTHERWISE, LETS MOVE IT TO THE RIGHT!
	CLC
	ADC #2						;ADD 2 TO THE X POSITION
	STA (ZPADDR)				;STORE THE X POSITION AGAIN
	BRA HANDLEUSERINPUT_CHECKFIRE	;NOW CHECK IF THE FIRE BUTTON IS (ALSO) PRESSED
	
HANDLEUSERINPUT_MOVEPLAYERR1:	;MOVE THE PLAYER X ONE TO THE RIGHT
	LDA #SPRITEARRAY3H			;LOAD THE HIGH BYTE OF SPRITE DATA ARRAY 3
	STA ZPADDR+1				;STORE IN ZPADDR+1, IT WILL BE THE HIGH BYTE
	STZ ZPADDR					;STORE 0 IN THE LOW BYTE (ZPADDR)
	LDA (ZPADDR)				;LOAD THE FIRST BYTE OF SPRITE 55, THE PLAYER'S X POSITION
	CMP #247						;CHECK IF X IS 247 ALREADY, IF SO WE CANT MOVE, SO DO NOTHING (247 +1 PIXELS WOULD PUT IT OVER THE 248 LIMIT (WE NEED TO KEEP IT 8 LESS THEN 255 AS THE MISSILE NEEDS TO BE LAUNCHED)
	BEQ HANDLEUSERINPUT_CHECKFIRE	;BRANCH TO HANDLEUSERINPUT_CHECKFIRE IF WE CANT MOVE
	CLC
	ADC #1						;ADD 1 TO THE X POSITION
	STA (ZPADDR)				;STORE THE X POSITION AGAIN
	BRA HANDLEUSERINPUT_CHECKFIRE	;NOW CHECK IF THE FIRE BUTTON IS (ALSO) PRESSED

HANDLEUSERINPUT_MOVEPLAYERL		;MOVE THE PLAYER'S SPRITE TO THE LEFT
	;---------------------------------------------
	CMP #1						;CHECK IF THE MOVEMENT IS -1
	BEQ HANDLEUSERINPUT_MOVEPLAYERL1
	;OTHERWISE MOVE THE PLAYER X TWO TO THE LEFT
	LDA #SPRITEARRAY3H			;LOAD THE HIGH BYTE OF SPRITE DATA ARRAY 3
	STA ZPADDR+1				;STORE IN ZPADDR+1, IT WILL BE THE HIGH BYTE
	STZ ZPADDR					;STORE 0 IN THE LOW BYTE (ZPADDR)
	LDA (ZPADDR)				;LOAD THE FIRST BYTE OF SPRITE 55, THE PLAYER'S X POSITION
	CMP #2						;CHECK IF X IS 0 OR 1 ALREADY, IF SO WE CANT MOVE, SO DO NOTHING
	BCC HANDLEUSERINPUT_CHECKFIRE	;BRANCH IF LESS THEN 2, TO HANDLEUSERINPUT_CHECKFIRE IF WE CANT MOVE
	SEC
	SBC #2						;SUBTRACT 2 FROM THE X POSITION
	STA (ZPADDR)				;STORE THE X POSITION AGAIN
	BRA HANDLEUSERINPUT_CHECKFIRE	;NOW CHECK IF THE FIRE BUTTON IS (ALSO) PRESSED
	
HANDLEUSERINPUT_MOVEPLAYERL1:	;MOVE THE PLAYER X ONE TO THE LEFT
	LDA #SPRITEARRAY3H			;LOAD THE HIGH BYTE OF SPRITE DATA ARRAY 3
	STA ZPADDR+1				;STORE IN ZPADDR+1, IT WILL BE THE HIGH BYTE
	STZ ZPADDR					;STORE 0 IN THE LOW BYTE (ZPADDR)
	LDA (ZPADDR)				;LOAD THE FIRST BYTE OF SPRITE 55, THE PLAYER'S X POSITION
	CMP #0						;CHECK IF X IS 0 ALREADY, IF SO WE CANT MOVE, SO DO NOTHING
	BEQ HANDLEUSERINPUT_CHECKFIRE	;BRANCH TO HANDLEUSERINPUT_CHECKFIRE IF WE CANT MOVE
	SEC
	SBC #1						;SUBTRACT 1 FROM THE X POSITION
	STA (ZPADDR)				;STORE THE X POSITION AGAIN
	BRA HANDLEUSERINPUT_CHECKFIRE	;NOW CHECK IF THE FIRE BUTTON IS (ALSO) PRESSED
	
HANDLEUSERINPUT_CHECKFIRE:		;CHECK IF THE FIRE BUTTON (SPACE) HAS (ALSO) BEEN PRESSED
	;---------------------------------------------		
	;TODO, ADD FIRE BUTTON (SPACEBAR) CHECK
	LDA #0						;LOAD 0 IN A
	JSR JOYSTICK_GET			;COMMANDER X16 OS CALL, GET THE STATE OF JOYSTICK 1
	STA ZPADDR					;STORE THE PRESSED BUTTONS IN ZPADDR FOR NOW
	AND #128					;APPLY BITMASK, LEAVING ONLY BIT 7, OR IF THE A BUTTON IS PRESSED
	BNE HANDLEUSERINPUT_DONE	;IF THE A BUTTON IS NOT PRESSED, THEN EXIT THE SUBROUTINE
	;OTHERWISE THE A BUTTON (OR CTRL KEY) IS PRESSED
	;LET'S SEE IF THE PLAYER MISSILE IS ALREADY ACTIVE.
	LDA #SPRITEARRAY3H
	STA ZPADDR+1				;STORE THE HIGH BYTE IN ZPADDR+1
	LDA #9
	STA ZPADDR					;STORE 9 AS THE LOW BYTES
	LDA (ZPADDR)				;LOAD THE SPRITE STATE OF THE MISSILE ($0609)
	CMP #2						;CHECK IF THE SPRITE STATE IS ALIVE (2).
	BEQ HANDLEUSERINPUT_DONE	;IF IT IS ALREADY ON THE SCREEN, THEN WE'RE NOT GOING TO DO ANYTHING.
	;IF IT'S NOT ALIVE, THEN WE CAN PUT IT ON THE SCREEN
	JSR LAUNCHMISSILE			;LAUNCH THE PLAYER'S MISSILE

HANDLEUSERINPUT_DONE:	
	RTS		
	
;---------------------------------------------
;FUNCTION - LAUNCHMISSILE, HERE WE'RE GOING TO LAUNCH A NEW PLAYER MISSILE
;---------------------------------------------		
LAUNCHMISSILE:
	;LOAD THE PLAYER'S X POSITION
	LDA #SPRITEARRAY3H
	STA ZPADDR+1				;STORE SPRITE DATA ARRAY 3'S HIGH BYTE TO ZPADDR+1 (FIRST 16-BIT ARRAY, SOURCES)
	STA ZPADDR+3				;AND STORE THE SPRITE DATA ARRAY 3'S HIGH BYTE TO ZPADDR+3 (SECOND 16-BIT ARRAY, DESTINATION)
	
	LDA #6						;STORE 06 AS THE LOW BYTE (SECOND 16-BIT ARRAY)
	STA ZPADDR+2				;STORE THE LOW BYTE IN THE SECOND 16-BIT ARRAY, DESTINATION
	
	;MISSILE X POSITION!
	LDA #0
	STA ZPADDR					;STORE THE LOW BYTE (POINTING TO THE PLAYERS X POSITION) IN THE FIRST 16-BIT ARRAY, SOURCE.
	LDA (ZPADDR)				;LOAD THE PLAYER'S X POSITION
	
	CLC
	ADC #4						;ADD 4 TO THE X POSITION OF THE PLAYER (SO THAT IF THE 8X8 MISSILE IS SHOWN, IT'LL BE IN THE MIDDLE OF THE PLAYER'S X POSITION)
	STA (ZPADDR+2)				;STORE THE NEW X POSITION IN THE PLAYER MISSILE IN THE PLAYER MISSILE'S X POSITION ($0606)

	;MISSILE Y POSITION!
	LDA #1
	STA ZPADDR					;STORE 1 IN THE FIRST 16-BIT ARRAY, SOURCE. POINTING THE FIRST 16-BIT ARRAY TO THE Y POSITION OF THE PLAYER
	LDA #7						
	STA ZPADDR+2				;STORE 7 IN THE SECOND 16-BIT ARRAY, DESTINATION. POINTING TO THE Y POSITION OF THE PLAYER MISSILE.
	
	LDA (ZPADDR)				;LOAD THE Y POSITION OF THE PLAYER SPRITE
	SEC
	SBC #8						;SUBTRACT 8 PIXELS FROM THE Y POSITION
	STA (ZPADDR+2)				;STORE THE NEW Y POSITION OF THE PLAYER MISSILE SPRITE
	
	;MISSILE SPRITE TYPE
	LDA #8
	STA ZPADDR+2				;STORE 08 AS THE NEW LOW BYTE IN THE 2ND 16-BIT ARRAY, DESTINATION
	LDA #1
	STA (ZPADDR+2)				;STORE TYPE 1 (PLAYER MISSILE) IN THE SPRITE TYPE BYTE OF THE PLAYER MISSILE SPRITE
	
	;MISSILE SPRITE TYPE
	LDA #9
	STA ZPADDR+2				;STORE 09 AS THE NEW LOW BYTE IN THE 2ND 16-BIT ARRAY, DESTINATION
	LDA #2
	STA (ZPADDR+2)				;STORE STATE 2 (ALIVE) IN THE SPRITE STATE BYTE OF THE PLAYER MISSILE SPRITE
	
	;MISSILE SPRITE VRAM START ADDRESS
	LDA #10
	STA ZPADDR+2				;STORE 0A AS THE NEW LOW BYTE IN THE 2ND 16-BIT ARRAY, DESTINATION
	LDA #80
	STA (ZPADDR+2)				;STORE 80 AS THE VERA VRAM START ADDRESS OF THE MISSILE SPRITE
	
	;MISSILE SPRITE VRAM START ADDRESS
	LDA #11
	STA ZPADDR+2				;STORE 0B AS THE NEW LOW BYTE IN THE 2ND 16-BIT ARRAY, DESTINATION
	LDA #0
	STA (ZPADDR+2)				;STORE 0 AS THE SPRITE ANIMATION FRAME FOR THE PLAYER MISSILE
	
	RTS
	
;---------------------------------------------
;FUNCTION - WAITGAMETICK, LIMITS THE GAME LOOP TO 24 FPS 
;---------------------------------------------		
WAITGAMETICK:
	;WHAT WE NEED TO DO, IS WAIT AT THE END OF EACH GAME LOOP UNTIL 1/24TH OF A SECOND HAS PASSED SINCE THE LAST WAIT
	;1/24TH OF A SECOND = 24 FPS = 1 GAME TICK = 0.04167 SECONDS (41.67 MS)
	;THE COMMANDER X16 CAN COUNT JIFFY'S (1/60TH OF A SECOND, 16.67 MS PER JIFFY)
	;THIS POSES A PROBLEM, AS YOU CANT MAKE 1/24 WITH 1/60TH. WHAT WE'RE GOING TO DO IS AS FOLLOWED, IT'S A MULTI STEP PROCESS
	;1) LOAD THE LAST JIFFY COUNT (VAR_JIFFYC). WE'RE GOING TO WAIT UNTIL 2 JIFFY'S PASS SINCE THE LAST TIME THE PAUSE ROUTINE RAN (33.34 MS), AND STORE THAT JIFFY NUMBER (JIFFY+2 TO VAR_JIFFYC). 
	;	ALSO SET THE VAR_JIFFYS TO 1 TO LET THE GAME KNOW WE'VE DONE THE SHORT JIFFY PAUSE THIS TIME (2X JIFFY)
	;2) WE'RE GOING TO RUN A DELAYING LOOP THAT RUNS FOR 69300 CPU CLOCK CYCLES, WHICH SHOULD BE 8.32 MS (WITH A 6502 RUNNING AT 8.32 MHZ), BRINGING IT IN TOTAL TO 41.66 MS. INCREMENT THE GAMETICK TIMER (0-23)
	;3) WE CAN THEN RUN A GAMETICK AGAIN AS NORMAL. WHEN WE GET BACK TO THE WAITGAMETICK SUB, WE LOAD THE JIFFY COUNT (VAR_JIFFYC) AND IF THE PREV WAIT WAS A SHORT PAUSE (VAR_JIFFYS=1) THEN,
	;	WE WAIT 3 JIFFY THIS TIME, UPDATE THE JIFFY COUNTER, SET VAR_JIFFYS TO 0 (LONG PAUSE), AND RUN THE GAMELOOP AS USUAL. AND THE PROCESS THEN REPEATS AT STEP 1

	;ALL THIS MEANS THAT THE GAME SHOULD RUN AT 24 FPS, WHERE 1 GAMETICK IS 1/24TH OF A SECOND.
	LDA VAR_JIFFYS				;LOAD THE FLAG FOR IF THE LAST WAITGAMETICK IS A SHORT ONE 
	BEQ WAITGAMETICK_LONG		;BRANCH IF VAR_JIFFYS IS 0 (THIS TIME IS NOT A SHORT WAIT, SO WE'RE DOING A LONG ONE NOW)
	
	;IF WE GET HERE, VAR_JIFFYS = 1, SO WE NEED TO DO A SHORT JIFFY CHECK
	LDX VAR_JIFFYC				;LOAD THE LAST JIFFY NUMBER AT THE END OF THE LAST WAITGAMETICK FUNCTION
	INX							;ADD 1 TO THE JIFFYC COUNTER
	CPX #60						;CHECK IF IT'S NOW AT 60
	BEQ JC_SHORT_OVERFL1		;IF IT IS, PROBLEM! JUMP TO JC_SHORT_OVERFL1, WE NEED TO SET IT TO 1 (60 SHOULD OVERFLOW 0, BUT WE NEED TO ADD 2, SO ADD ONE MORE = 1)
	INX							;ADD ONE MORE
	CPX #60						;CHECK IF IT'S NOW AT 60
	BEQ JC_SHORT_OVERFL0		;IF IT IS, PROBLEM! JUMP TO JC_SHORT_OVERFL0, WE NEED TO SET IT TO 0 (60 SHOULD OVERFLOW 0)
	BRA WAITGAMETICK_SHORT		;OTHERWISE JUST STORE THE NEW VAR_JIFFYC VALUE AND START WAITING!
	
JC_SHORT_OVERFL1:				;OVERFLOW THE VAR_JIFFYC TO 1
	LDX #1						;LOAD 1 IN X
	BRA WAITGAMETICK_SHORT		;STORE AND START WAITING!

JC_SHORT_OVERFL0:
	LDX #0						;LOAD 0 IN X
	
WAITGAMETICK_SHORT:
	STX VAR_JIFFYC				;STORE THE NEW VALUE AS VAR_JIFFYC, SO WE KNOW AT WHICH JIFFY TO START CHECKING NEXT WAITGAMETICK.
WAITGAMETICK_SHORT_L1:			;BEGIN OF WAIT LOOP UNTIL THE SHORT TIMER IS UP
	JSR CLOCKAPI				;CALL KERNAL CLOCK_GET_DATE_TIME. IT WILL RETURN THE CURRENT NUMBER OF JIFFY'S IN $08, LABELED CLOCKJIFFY FOR EASE

	LDA CLOCKJIFFY				;LOAD THE REAL TIME JIFFY COUNTER
	CMP VAR_JIFFYC				;COMPARE THE CURRENT JIFFY NUMBER IN THE REAL TIME CLOCK TO LAST JIFFY NUMER +2
	BNE WAITGAMETICK_SHORT_L1	;IF THE CURRENT CLOCK IS NOT AT LAST JIFFY NUMBER +2 YET, THEN LOOP BACK AND TRY AGAIN, KEEP DOING SO UNTIL IT HAS BEEN 2 JIFFY.
	
	;IF WE GET HERE, THEN THE 2 JIFFY HAVE PASSED. NOW WE NEED TO WAIT ANOTHER 0.5 JIFFY. SINCE WE CAN'T COUNT PARTIAL JIFFYS IN THE REAL TIME CLOCK,;
	;WE'RE GOING TO DO KEEP THE CPU BUSY FOR ABOUT 69300 CYCLES, WHICH AT 8.33 MHZ, WILL TAKE 8.32 MS (OR HALF A JIFFY)
	LDX #$00
	LDY #$00
DLOOP:
	NOP							;DO NOTHING (2 CYCLES)
	NOP							;DO NOTHING (2 CYCLES)
	INX							;INCREMENT X, FIRST LOOP COUNTER (2 CYCLES)
	BNE DLOOP					;BRANCH BACK TO DLOOP IF X HAS NOT OVERFLOWED TO 0 (OR 256) YET. 3 CYCLES IF TAKEN, TOTAL PER LOOP IS 9 CYCLES. AFTER 256 TIMES = 2304 CYCLES.
	INY							;INCREMENT Y, SECOND LOOP COUNTER (2 CYCLES)
	CPY #$1E					;CHECK IF Y IS $1E, OR DEC 30. AKA, IF THE SECOND LOOP HAS RUN 30 TIMES. (2 CYCLES)
	BNE DLOOP					;BRANCH BACK TO DLOOP IF SECOND LOOP HASNT DONE 30 LOOPS YET. (3 CYCLES). SO ADD 7 CYCLES TO 2304 = 2311. 
	;IF IT GET'S HERE THE SECOND LOOP HAS DONE 30 LOOPS. FOR A TOTAL OF 2311 * 30 = 69330 CYCLES	
	STZ VAR_JIFFYS				;STORE 0 IN VAR_JIFFYS TO LET THE WAITGAMETICK SUB KNOW THAT NEXT TIME WE NEED TO DO A SHORT 2 PAUSE.
	
WAITGAMETICK_UPDATETICK:		;NOW LETS UPDATE THE GAME TICK COUNTER
	LDX VAR_GAMETICK			;LOAD THE CURRENT GAMETICK IN 1
	INX							;INCREMENT THE CURRENT GAMETICK BY ONE
	STX VAR_GAMETICK			;STORE THE UPDATED GAMETICK IN VAR_GAMETICK
	CPX #24						;CHECK IF VAR_GAMETICK IS 24 NOW, SINCE IT SHOULD BE BETWEEN 0-23, SET IT TO 0.
	BNE WAITGAMETICK_DONE		;IF IT'S NOT 24 YET, THEN WE'RE DONE!
	
	STZ VAR_GAMETICK			;RESET GAMETICK TO 0
	BRA WAITGAMETICK_DONE		;AND NOW WE'RE DONE!

WAITGAMETICK_LONG:				;DO A LONG WAIT HERE (3 JIFFY)
	LDX VAR_JIFFYC				;LOAD THE LAST JIFFY NUMBER AT THE END OF THE LAST WAITGAMETICK FUNCTION
	INX							;ADD 1 TO THE JIFFYC COUNTER
	CPX #60						;CHECK IF IT'S NOW AT 60
	BEQ JC_LONG_OVERFL2			;IF IT IS, PROBLEM! JUMP TO JC_LONG_OVERFL2, WE NEED TO SET IT TO 2 (60 SHOULD OVERFLOW 0, BUT WE NEED TO ADD 3, SO ADD 2 MORE = 2)
	INX							;ADD ONE MORE
	CPX #60						;CHECK IF IT'S NOW AT 60
	BEQ JC_LONG_OVERFL1			;IF IT IS, PROBLEM! JUMP TO JC_LONG_OVERFL1, WE NEED TO SET IT TO 1 (60 SHOULD OVERFLOW 0, BUT WE NEED TO ADD 2, SO ADD 1 MORE = 1)
	INX							;ADD ONE MORE
	CPX #60						;CHECK IF IT'S NOW AT 60
	BEQ JC_LONG_OVERFL0			;IF IT IS, PROBLEM! JUMP TO JC_LONG_OVERFL0, WE NEED TO SET IT TO 0 (60 SHOULD OVERFLOW 0)	
	BRA WAITGAMETICK_LONG_S		;OTHERWISE JUST STORE THE NEW VAR_JIFFYC VALUE AND START WAITING!
	
JC_LONG_OVERFL2:				;OVERFLOW THE VAR_JIFFYC TO 2
	LDX #2						;LOAD 2 IN X
	BRA WAITGAMETICK_LONG_S		;STORE AND START WAITING!
	
JC_LONG_OVERFL1:				;OVERFLOW THE VAR_JIFFYC TO 1
	LDX #1						;LOAD 1 IN X
	BRA WAITGAMETICK_LONG_S		;STORE AND START WAITING!

JC_LONG_OVERFL0:				;OVERFLOW THE VAR_JIFFYC TO 0
	LDX #0						;LOAD 0 IN X
WAITGAMETICK_LONG_S:
	STX VAR_JIFFYC				;STORE THE NEW VALUE AS VAR_JIFFYC, SO WE KNOW AT WHICH JIFFY TO START CHECKING NEXT WAITGAMETICK
WAITGAMETICK_LONG_L1:			;BEGIN OF WAIT LOOP UNTIL THE SHORT TIMER IS UP
	JSR CLOCKAPI				;CALL KERNAL CLOCK_GET_DATE_TIME. IT WILL RETURN THE CURRENT NUMBER OF JIFFY'S IN $08, LABELED CLOCKJIFFY FOR EASE
	
	LDA CLOCKJIFFY				;LOAD THE REAL TIME JIFFY COUNTER
	CMP VAR_JIFFYC				;COMPARE THE CURRENT JIFFY NUMBER IN THE REAL TIME CLOCK TO LAST JIFFY NUMER +3
	BNE WAITGAMETICK_LONG_L1	;IF THE CURRENT CLOCK IS NOT AT LAST JIFFY NUMBER +3 YET, THEN LOOP BACK AND TRY AGAIN, KEEP DOING SO UNTIL IT HAS BEEN 3 JIFFY.
	
	;IF WE GET HERE, THEN THE 3 JIFFY HAVE PASSED.
	LDA #1
	STA VAR_JIFFYS				;STORE 1 IN VAR_JIFFYS TO LET THE WAITGAMETICK SUB KNOW THAT NEXT TIME WE NEED TO DO A SHORT 2 PAUSE.
	BRA WAITGAMETICK_UPDATETICK	;BRANCH TO WAITGAMETICK_UPDATETICK, TO UPDATE THE GAMETICK BY 1 (OR RESET 0)
	
WAITGAMETICK_DONE:				;IF THE WAITGAMETICK FUNCTION HAS WAITED ENOUGH FOR 1 GAME TICK TO HAVE TAKEN 1/24TH OF A SECOND
	RTS
	
	
;---------------------------------------------
;FUNCTION - LOADENEMYPATTERN, INITIAL LOAD OF THE ENEMIES FROM A LOOKUP TABLE
;---------------------------------------------		
;IN THIS FUNCTION WE'RE GOING TO RUN THROUGH ALL 55 POSSIBLE ENEMY POSITIONS AND SET THEM TO A ENEMY SPRITE OR LEAVE THE EMPTY
;THIS BUILDS UP SPRITE DATA ARRAY 1 AND 2 BEFORE THE FIRST LEVEL STARTS.
LOADENEMYPATTERN:
	;WE SIMPLY NEED TO RUN THROUGH THE LOOKUP TABLE CALLED LEVEL1. IT CONSISTS OF 55 VALUES.
	;0 IS A EMPTY FIELD, 1 IS ENEMY NUMBER 1.....
	STZ ZPADDR+2				;EMPTY ZPADDR+2, WE'LL USE IT TO KEEP TRACK OF THE NEXT X POSITION TO PUT THE SPRITE
	STZ ZPADDR+3				;EMPTY ZPADDR+3, WE'LL USE IT TO KEEP TRACK OF THE NEXT Y POSITION TO PUT THE SPRITE
	LDY #0						;LOAD Y WITH 0, IT WILL BE OUR LOOP COUNTER
	
	JSR LOADLEVELS				;LOAD THE LEVEL DATA POINTER IN ZPADDR AND ZPADDR+1 BASED ON THE LEVEL TO LOAD IN VAR_LEVEL
	
LOADENEMYPATTERN_L1:	
	LDA (ZPADDR), Y				;LOAD A BYTE FROM THE LEVEL1 LOOKUP TABLE
	
	CMP #0						;CHECK IF THE BYTE IS 0 (DONT PLACE ANYTHING)
	BEQ LOADENEMYPATTERN_NEXT	;IF IT IS 0, THEN GO TO THE NEXT LOOP
	;CHECK IF WE NEED TO LOAD ENEMY 1
	CMP #1
	BEQ LOADENEMYPATTERN_E1		;LOAD ENEMY 1.
	CMP #2
	BEQ LOADENEMYPATTERN_E2		;LOAD ENEMY 2.
	CMP #3
	BEQ LOADENEMYPATTERN_E3		;LOAD ENEMY 3.
	
LOADENEMYPATTERN_E1:			;LOAD ENEMY NUMBER 1
	STA ZPADDR+7				;STORE THE TYPE OF ENEMY TO ZPADDR+7, WE'LL NEED IT LATER
	LDA #2
	STA ZPADDR+8				;STORE THE SPRITE STATE (2=ALIVE) IN ZPADDR+8
	LDA #0
	STA ZPADDR+9				;STORE THE VERA START ADDRESS OF ENEMY 0 IN ZPADDR+9.	
	JSR LOADENEMY				;SET THE SPRITE DATA INTO THE SPRITE DATA ARRAY
	BRA LOADENEMYPATTERN_NEXT	;BRANCH TO LOAD THE NEXT ENEMY IN THE LOOP.
	
LOADENEMYPATTERN_E2:			;LOAD ENEMY NUMBER 2
	STA ZPADDR+7				;STORE THE TYPE OF ENEMY TO ZPADDR+7, WE'LL NEED IT LATER
	LDA #2
	STA ZPADDR+8				;STORE THE SPRITE STATE (2=ALIVE) IN ZPADDR+8
	LDA #8
	STA ZPADDR+9				;STORE THE VERA START ADDRESS OF ENEMY 0 IN ZPADDR+9.	
	JSR LOADENEMY				;SET THE SPRITE DATA INTO THE SPRITE DATA ARRAY
	BRA LOADENEMYPATTERN_NEXT	;BRANCH TO LOAD THE NEXT ENEMY IN THE LOOP.
	
LOADENEMYPATTERN_E3:			;LOAD ENEMY NUMBER 3
	STA ZPADDR+7				;STORE THE TYPE OF ENEMY TO ZPADDR+7, WE'LL NEED IT LATER
	LDA #2
	STA ZPADDR+8				;STORE THE SPRITE STATE (2=ALIVE) IN ZPADDR+8
	LDA #12
	STA ZPADDR+9				;STORE THE VERA START ADDRESS OF ENEMY 0 IN ZPADDR+9.	
	JSR LOADENEMY				;SET THE SPRITE DATA INTO THE SPRITE DATA ARRAY
	
LOADENEMYPATTERN_NEXT:			;PREPARE FOR THE NEXT LOOP
	LDA ZPADDR+2				;LOAD THE X POSITION
	CLC
	ADC #20						;ADD 20 PIXELS TO IT.
	STA ZPADDR+2				;STORE THE NEW X POSITION
	CMP #220					;CHECK IF X IS 220 NOW, IF IT IS IT NEEDS TO BE SET TO 0, AND Y INCREASED
	BEQ LOADENEMYPATTERN_NEXTL	;BRANCH TO LOADENEMYPATTERN_NEXTL TO LOAD THE NEXT LINE. (RESET X TO 0 AND INCREASE Y)
LOADENEMYPATTERN_NEXT2:
	INY							;INCREASE Y (LOOP COUNTER)
	CPY #55						;CHECK IF WE HAVE ALREADY DONE 55 LOOPS
	BEQ LOADENEMYPATTERN_DONE	;IF WE HAVE, THEN END LOOPING, WE'RE DONE!	
	BRA LOADENEMYPATTERN_L1		;IF WE HAVEN'T RUN THROUGH ALL 55 ENEMY SPRITE POSITIONS YET, THEN CONTINUE WITH THE NEXT ONE

LOADENEMYPATTERN_NEXTL			;RESET X BACK TO 0, INCREASE Y TO THE NEXT LINE
	STZ ZPADDR+2				;RESET X TO 0 (AND STORE IT)
	LDA ZPADDR+3				;LOAD THE Y POSITION
	CLC
	ADC #20						;ADD 20 PIXELS TO THE Y POSITION
	STA ZPADDR+3				;STORE THE Y POSITION AGAIN.
	BRA LOADENEMYPATTERN_NEXT2	;BRANCH TO LOADENEMYPATTERN_NEXT2, TO CONTINUE TO THE NEXT ENTRY IN THE LOOKUP TABLE
	
LOADENEMYPATTERN_DONE:
	RTS
	

;---------------------------------------------
;FUNCTION - LOADLEVELS, LOADS THE LEVEL DATA IN ZPADDR AND ZPADDR+1
LOADLEVELS:
	
	LDA VAR_LEVEL				;LOAD THE GAME LEVEL TO LOADED
	CMP #0						;CHECK TO LOAD LEVEL 1
	BNE LOADLEVELS_2			;IF NOT THIS LEVEL, GO ON TO CHECK THE NEXT LEVEL
	LDA #<LEVEL1				;LOAD THE LOW BYTE OF THE LEVEL 1 LOOKUP TABLE
	STA ZPADDR					;STORE IN ZPADDR (AS THE LOW BYTE)
	LDA #>LEVEL1				;LOAD RHW HIGH BYTE OF THE LEVEL 1 LOOKUP TABLE
	STA ZPADDR+1				;STORE IN ZPADDR+1 (AS THE HIGH BYTE)
	JMP LOADLEVELS_END			
	
LOADLEVELS_2:
	CMP #1						;CHECK TO LOAD LEVEL 2
	BNE LOADLEVELS_3			;IF NOT THIS LEVEL, GO ON TO CHECK THE NEXT LEVEL
	LDA #<LEVEL2				;LOAD THE LOW BYTE OF THE LEVEL 2 LOOKUP TABLE
	STA ZPADDR					;STORE IN ZPADDR (AS THE LOW BYTE)
	LDA #>LEVEL2				;LOAD RHW HIGH BYTE OF THE LEVEL 2 LOOKUP TABLE
	STA ZPADDR+1				;STORE IN ZPADDR+1 (AS THE HIGH BYTE)
	BRA LOADLEVELS_END			
	
LOADLEVELS_3:
	CMP #2						;CHECK TO LOAD LEVEL 3
	BNE LOADLEVELS_4			;IF NOT THIS LEVEL, GO ON TO CHECK THE NEXT LEVEL
	LDA #<LEVEL3				;LOAD THE LOW BYTE OF THE LEVEL 3 LOOKUP TABLE
	STA ZPADDR					;STORE IN ZPADDR (AS THE LOW BYTE)
	LDA #>LEVEL3				;LOAD RHW HIGH BYTE OF THE LEVEL 3 LOOKUP TABLE
	STA ZPADDR+1				;STORE IN ZPADDR+1 (AS THE HIGH BYTE)
	BRA LOADLEVELS_END	
	
LOADLEVELS_4:
	CMP #3						;CHECK TO LOAD LEVEL 4
	BNE LOADLEVELS_5			;IF NOT THIS LEVEL, GO ON TO CHECK THE NEXT LEVEL
	LDA #<LEVEL4				;LOAD THE LOW BYTE OF THE LEVEL 4 LOOKUP TABLE
	STA ZPADDR					;STORE IN ZPADDR (AS THE LOW BYTE)
	LDA #>LEVEL4				;LOAD RHW HIGH BYTE OF THE LEVEL 4 LOOKUP TABLE
	STA ZPADDR+1				;STORE IN ZPADDR+1 (AS THE HIGH BYTE)
	BRA LOADLEVELS_END	
	
LOADLEVELS_5:
	CMP #4						;CHECK TO LOAD LEVEL 5
	BNE LOADLEVELS_6			;IF NOT THIS LEVEL, GO ON TO CHECK THE NEXT LEVEL
	LDA #<LEVEL5				;LOAD THE LOW BYTE OF THE LEVEL 5 LOOKUP TABLE
	STA ZPADDR					;STORE IN ZPADDR (AS THE LOW BYTE)
	LDA #>LEVEL5				;LOAD RHW HIGH BYTE OF THE LEVEL 5 LOOKUP TABLE
	STA ZPADDR+1				;STORE IN ZPADDR+1 (AS THE HIGH BYTE)
	BRA LOADLEVELS_END
	
LOADLEVELS_6:
	CMP #5						;CHECK TO LOAD LEVEL 6
	BNE LOADLEVELS_7			;IF NOT THIS LEVEL, GO ON TO CHECK THE NEXT LEVEL
	LDA #<LEVEL6				;LOAD THE LOW BYTE OF THE LEVEL 6 LOOKUP TABLE
	STA ZPADDR					;STORE IN ZPADDR (AS THE LOW BYTE)
	LDA #>LEVEL6				;LOAD RHW HIGH BYTE OF THE LEVEL 6 LOOKUP TABLE
	STA ZPADDR+1				;STORE IN ZPADDR+1 (AS THE HIGH BYTE)
	BRA LOADLEVELS_END
	
LOADLEVELS_7:
	CMP #6						;CHECK TO LOAD LEVEL 7
	BNE LOADLEVELS_8			;IF NOT THIS LEVEL, GO ON TO CHECK THE NEXT LEVEL
	LDA #<LEVEL7				;LOAD THE LOW BYTE OF THE LEVEL 7 LOOKUP TABLE
	STA ZPADDR					;STORE IN ZPADDR (AS THE LOW BYTE)
	LDA #>LEVEL7				;LOAD RHW HIGH BYTE OF THE LEVEL 7 LOOKUP TABLE
	STA ZPADDR+1				;STORE IN ZPADDR+1 (AS THE HIGH BYTE)
	BRA LOADLEVELS_END
	
LOADLEVELS_8:
	CMP #7						;CHECK TO LOAD LEVEL 8
	BNE LOADLEVELS_9			;IF NOT THIS LEVEL, GO ON TO CHECK THE NEXT LEVEL
	LDA #<LEVEL8				;LOAD THE LOW BYTE OF THE LEVEL 8 LOOKUP TABLE
	STA ZPADDR					;STORE IN ZPADDR (AS THE LOW BYTE)
	LDA #>LEVEL8				;LOAD RHW HIGH BYTE OF THE LEVEL 8 LOOKUP TABLE
	STA ZPADDR+1				;STORE IN ZPADDR+1 (AS THE HIGH BYTE)
	BRA LOADLEVELS_END

LOADLEVELS_9:
	CMP #8						;CHECK TO LOAD LEVEL 9
	BNE LOADLEVELS_10			;IF NOT THIS LEVEL, GO ON TO CHECK THE NEXT LEVEL
	LDA #<LEVEL9				;LOAD THE LOW BYTE OF THE LEVEL 9 LOOKUP TABLE
	STA ZPADDR					;STORE IN ZPADDR (AS THE LOW BYTE)
	LDA #>LEVEL9				;LOAD RHW HIGH BYTE OF THE LEVEL 9 LOOKUP TABLE
	STA ZPADDR+1				;STORE IN ZPADDR+1 (AS THE HIGH BYTE)
	BRA LOADLEVELS_END

LOADLEVELS_10:
	CMP #9						;CHECK TO LOAD LEVEL 9
	BNE LOADLEVELS_11			;IF NOT THIS LEVEL, GO ON TO CHECK THE NEXT LEVEL
	LDA #<LEVEL10				;LOAD THE LOW BYTE OF THE LEVEL 9 LOOKUP TABLE
	STA ZPADDR					;STORE IN ZPADDR (AS THE LOW BYTE)
	LDA #>LEVEL10				;LOAD RHW HIGH BYTE OF THE LEVEL 9 LOOKUP TABLE
	STA ZPADDR+1				;STORE IN ZPADDR+1 (AS THE HIGH BYTE)
	BRA LOADLEVELS_END

LOADLEVELS_11:
	;THERE IS NOTHING BEYOND LEVEL 10, RESET BACK TO KEEP LOOPING LEVEL 10
	LDA #<LEVEL10				;LOAD THE LOW BYTE OF THE LEVEL 9 LOOKUP TABLE
	STA ZPADDR					;STORE IN ZPADDR (AS THE LOW BYTE)
	LDA #>LEVEL10				;LOAD RHW HIGH BYTE OF THE LEVEL 9 LOOKUP TABLE
	STA ZPADDR+1				;STORE IN ZPADDR+1 (AS THE HIGH BYTE)
	
LOADLEVELS_END:
	RTS
	
;---------------------------------------------
;FUNCTION - LOADENEMY, LOADS A SINGLE ENEMY IN THE X AND Y POSITION
; X POSITION IS IN ZPADDR+2, Y POSITION IS IN ZPADDR+3, SPRITE NUMBER IS IN Y
; SPRITE TYPE IS IN ZPADDR+7, SPRITE STATE IS IN ZPADDR+8 AND VERA START ADDRESS IN ZPADDR+9
;---------------------------------------------		
LOADENEMY:
	STY ZPADDR+4				;TEMP STORE THE SPRITE NUMBER IN ZPADDR+4
	CPY #35						;CHECK IF THE SPRITE NUMBER IS 35
	BCC	LOADENEMY_1				;LOAD THE SPRITE FROM SPRITE DATA ARRAY 1
	;OTHERWISE WE LOAD SPRITE DATA ARRAY 2
	LDA #SPRITEARRAY2H			;LOAD SPRITE DATA ARRAY 2
	STA ZPADDR+6				;LOAD THE HIGH BYTE IN ZPADDR+6 (IT'LL BE THE HIGH BYTE)
	
	LDA ZPADDR+4				;LOAD THE SPRITE NUMBER
	SEC
	SBC #35						;SUBTRACT 35 FROM THE SPRITE NUMBER TO GET IT READY FOR SPRITE DATA ARRAY 2
	STA ZPADDR+4				;STORE THE NEW SPRITE NUMBER IN ZPADDR+4	
	BRA LOADENEMY_C				;NOW WE BRANCH TO LOAD SPRITES 35-55

LOADENEMY_1:					;SET THE SPRITE DATA IN SPRITE DATA ARRAY 1
	LDA #SPRITEARRAY1H			;LOAD SPRITE DATA ARRAY 1
	STA ZPADDR+6				;LOAD THE HIGH BYTE IN ZPADDR+6 (IT'LL BE THE HIGH BYTE)

LOADENEMY_C:					;CONTINUE AFTER LOADING THE HIGH BYTE
	LDA ZPADDR+4				;LOAD THE SPRITE NUMBER
	ASL
	ASL							;BITSHIFT TWICE, MULTIPLY BY 4
	STA ZPADDR+5				;STORE IN ZPADDR+5 FOR NOW, IT'LL BE THE LOW BYTE.
	LDA ZPADDR+4				;RELOAD THE SPRITE NUMBER
	ASL							;BITSHIFT TO THE LEFT ONCE, MULIPLY IT BY 2
	CLC
	ADC ZPADDR+5				;ADD SPRITE NUMBER*4 TO IT, MAKING THE TOTAL SPRITE NUMBER*6
	STA ZPADDR+5				;STORE THE LOW BYTE, IT'LL POINT TO THE 1ST BYTE IN THE SPRITE ARRAY (X POSITION)
	
	;STORE X POSITION (1ST BYTE OF DATA ARRAY)
	LDA ZPADDR+2				;LOAD THE X POSITION TO PUT THE SPRITE IN.
	STA (ZPADDR+5)				;STORE THE X POSITION OF THE SPRITE IN THE SPRITE DATA ARRAY
	;STORE Y POSITION (2ND BYTE OF DATA ARRAY)
	LDX ZPADDR+5				;RELOAD THE LOW BYTE
	INX							;ADD ONE TO IT, SO IT NOW POINTS TO THE 2ND BYTE (Y POSITION)
	STX ZPADDR+5				;STORE THE NEW LOW BYTE BACK TO ZPADDR+5
	LDA ZPADDR+3				;LOAD THE Y POSITION TO PUT THE SPRITE IN.
	STA (ZPADDR+5)				;STORE THE Y POSITION OF THE SPRITE IN THE SPRITE DATA ARRAY
	;STORE SPRITE TYPE (3RD BYTE OF DATA ARRAY)
	INX							;ADD 1 TO X, UPDATING THE LOW BYTE
	STX ZPADDR+5				;STORE THE NEW LOW BYTE BACK TO ZPADDR+5, IT'LL NOW POINT TO THE 3RD BYTES
	
	LDA ZPADDR+7				;LOAD THE ENEMY TYPE THAT WAS TEMP STORED IN ZPADDR+7
	CLC
	ADC #2						;ADD 2 TO IT, SO IT WILL HAVE THE CORRECT SPRITE TYPE (ENEMY1...)
	STA (ZPADDR+5)				;STORE THE SPRITE TYPE OF THE SPRITE IN THE SPRITE DATA ARRAY (3RD BYTE)
	
	;STORE SPRITE STATE (4TH BYTE OF DATA ARRAY)
	INX							;ADD 1 TO X, UPDATING THE LOW BYTE
	STX ZPADDR+5				;STORE THE NEW LOW BYTE IN ZPADDR+5. IT WILL NOW POINT TO THE 4TH BYTE.
	
	LDA ZPADDR+8				;LOAD THE SPRITE STATE FROM ZPADDR+8
	STA (ZPADDR+5)				;STORE THE SPRITE STATE OF THE SPRITE IN THE SPRITE DATA ARRAY (4TH BYTE)
	
	;STORE THE VERA START ADDRESS (5TH BYTE OF DATA ARRAY)
	INX							;ADD 1 TO X, UPDATING THE LOW BYTE
	STX ZPADDR+5				;STORE THE NEW LOW BYTE IN ZPADDR+5. IT WILL NOW POINT TO THE 5TH BYTE.
	
	LDA ZPADDR+9				;LOAD THE VRAM START ADDRESS FROM ZPADDR+9
	STA (ZPADDR+5)				;STORE THE SPRITE STATE OF THE SPRITE IN THE SPRITE DATA ARRAY (5TH BYTE)
	
	;STORE THE ANIMATION FRAME NUMBER (6TH BYTE OF DATA ARRAY)
	INX							;ADD 1 TO X, UPDATING THE LOW BYTE
	STX ZPADDR+5				;STORE THE NEW LOW BYTE IN ZPADDR+5. IT WILL NOW POINT TO THE 5TH BYTE.
	
	LDA #0						;LOAD 0 IN A, SO IT LOADS THE ANIMATION FRAME NUMBER OF 0 (THE FIRST ANIMATION FRAME)
	STA (ZPADDR+5)				;STORE THE SPRITE STATE OF THE SPRITE IN THE SPRITE DATA ARRAY (6TH BYTE)
	
	RTS
	
;---------------------------------------------
;FUNCTION - MOVEWEAPONS, THIS FUNCTION HANDLES THE MOVING OF THE PLAYER MISSILE AND THE ENEMY LASERS (SPRITE 56 - 61)
;---------------------------------------------
MOVEWEAPONS:
	;---------------------------------------------
	;FIRST WE CHECK THE PLAYER MISSILE, IF IT'S ENABLED, WE MOVE IT UPWARDS EVERY GAME TICKS
	LDA #SPRITEARRAY3H
	STA ZPADDR+1				;STORE SPRITE DATA ARRAY 3'S HIGH BYTE TO ZPADDR+1
	LDA #9
	STA ZPADDR					;STORE 9 AS THE LOW BYTE, POINTING TO THE PLAYER MISSILE'S SPRITE STATE
	LDA (ZPADDR)				;LOAD THE PLAYER MISSLE'S SPRITE STATE
	CMP #0						;CHECK IF THE STATUS OF THE PLAYER MISSILE IS DEAD (0).
	BEQ MOVEWEAPONS_DONEPM		;IF IT IS DEAD, THEN WE DON'T NEED TO DO ANYTHING WITH THE PLAYER MISSILE, BRANCH
	;OTHERWISE WE'LL UPDATE THE Y POSITION OF THE PLAYER MISSILE, MOVING IT UPWARDS
	LDA #7
	STA ZPADDR					;STORE 7 IN ZPADDR, CHANGING THE LOW BYTE TO POINT TO THE Y POSITION OF THE PLAYER MISSILE
	LDA (ZPADDR)				;LOAD THE PLAYER MISSILE'S Y POSITION INTO A
	SEC
	SBC #6						;SUBTRACT 6 PIXELS FROM THE PLAYER'S MISSILE
	STA (ZPADDR)				;STORE THE NEW Y POSITION OF THE PLAYER'S MISSILE
	
	;---------------------------------------------
	;NOW WE NEED TO CHECK IF THE PLAYER MISSILES MOVE OFF SCREEN, PAST THE Y BOUNDRY'S. AND KILL IT IF IT DID.
	LDA #7
	STA ZPADDR					;STORE 7 IN ZPADDR, CHANGING THE LOW BYTE TO POINT TO THE Y POSITION OF THE PLAYER MISSILE
	LDA (ZPADDR)				;LOAD THE PLAYER MISSILE'S Y POSITION
	CMP #250					;CHECK IF THE PLAYER MISSILE'S Y POSITITION IS 251 (MEANING IT ROLLED OVER PAST THE TOP (Y=0)
	BCC	MOVEWEAPONS_DONEPM		;IF THE Y POSITION IS LESS THEN 253, THEN IT'S STILL ON THE SCREEN PROPERLY, SO WE DON'T KILL IT.
	;IF IT IS 250 OR MORE, (250-255), THEN IT'S OVERFLOWED AROUND PAST THE TOP (0), SO WE KILL IT AND TAKE IT OFF SCREEN
	LDA #9
	STA ZPADDR					;STORE 9 IN ZPADDR, CHANGING THE LOW BYTE TO POINT TO THE SPRITE STATE OF THE PLAYER MISSILE
	LDA #0
	STA (ZPADDR)				;STORE 0 IN THE SPRITE STATE, CHANGING TO DEAD AND KILLING IT!
	;BUT THAT'S NOT ALL, WE NEED TO ACTUALLY DISABLE THE SPRITE ITSELF, AS CHANGING THE STATE TO DEAD IN THE DATA ARRAY JUST STOPS IT FROM UPDATING, BUT THE SPRITE WILL STILL BE ON SCREEN.
	LDA #4
	STA VAR_SPRITESTATUS		;SET 4 IN THE SPRITE STATUS VARIABLE, TO TELL THE SETSPRITE SUB TO DISABLE THE SPRITE
	LDA #56						;LOAD 56 INTO A, AS A IS USED AS THE SPRITE NUMBER. POINTING IT TO THE SPRITE NUMBER FOR THE PLAYER MISSILE FOR THE SETSPRITE.
	JSR SETSPRITE				;NOW LOAD THE SPRITE, WHICH THANKS TO VAR_SPRITESTATUS BEING 4, IT WILL DISABLE THE SPRITE IN VERA.
	
MOVEWEAPONS_DONEPM:				;DONE CHECKING THE PLAYER MISSILE MOVEMENT

	;---------------------------------------------
	;NOW WE CHECK THE 5 POSSIBLE ENEMY LASERS, AND IF NOT DEAD, THEN MOVE THEM 3 PIXELS DOWNWARDS..
	LDA #SPRITEARRAY3H
	STA ZPADDR+1				;LOAD THE HIGH BYTE WITH THE HIGH BYTE OF SPRITE DATA ARRAY 3
	STZ ZPADDR					;STORE 0 IN THE LOW BYTE
	LDX #0						;LOAD 0 IN X, IT WILL BE THE SPRITE COUNTER
	LDY #15						;LOAD 15 IN Y, IT WILL BE THE OFFSET (POINTING TO THE SPRITE'S STATE OF THE FIRST ENEMY LASER)
MOVEWEAPONS_L_L:
	LDA (ZPADDR),Y				;LOAD THE SPRITE'S STATE.
	CMP #0						;CHECK IF THE LASER IS DEAD/INACTIVE
	BEQ MOVEWEAPONS_L_L_NEXT	;IF IT IS DEAD/INACTIVE, THEN CHECK THE NEXT ONE.
	;IF IT'S NOT DEAD, WE NEED TO MOVE IT DOWN 3 PIXELS
	DEY
	DEY							;DECREMENT Y TWICE, SO IT POINTS BACK TO THE Y POSITION OF THE LASER
	LDA (ZPADDR),Y				;LOAD THE LASER SPRITE'S Y POSITION
	CLC
	ADC #3						;ADD THREE PIXELS TO THE Y POSITION
	STA (ZPADDR),Y				;STORE THE NEW Y POSITION BACK TO THE SPRITE'S DATA.
	INY
	INY							;INCREMENT Y TWICE, SO IT POINTS BACK TO THE SPRITE STATE SO WE CAN CHECK THE NEXT SPRITE.
	;NOW WE JUST NEED TO CHECK IF IT'S AT OR BELOW THE LOWER THRESHOLD OF THE SCREEN (Y>=220)
	CMP #220
	BCC	MOVEWEAPONS_L_NOKILL	;JUMP TO MOVEWEAPONS_L_NOKILL, WE'RE NOT GOING TO KILL THE SPRITE.
	;OTHERWISE IT'S AT OR ABOVE 220, SO WE NEED TO KILL IT, Y IS POINTING BACK TO THE SPRITE STATE
	LDA #0
	STA (ZPADDR),Y				;STORE 0 IN THE SPRITE STATE, INDICATING THE SPRITE IS DEAD!
	;BUT THAT'S NOT ALL, WE NEED TO ACTUALLY DISABLE THE SPRITE ITSELF, AS CHANGING THE STATE TO DEAD IN THE DATA ARRAY JUST STOPS IT FROM UPDATING, BUT THE SPRITE WILL STILL BE ON SCREEN.
	LDA #4
	STA VAR_SPRITESTATUS		;SET 4 IN THE SPRITE STATUS VARIABLE, TO TELL THE SETSPRITE SUB TO DISABLE THE SPRITE
	TXA							;TRANSFER THE SPRITE COUNT TO A
	CLC
	ADC #57						;ADD 57 TO IT SO IT'S AT ENEMY LASER+X
	PHX
		PHY						;STORE X AND Y TO THE STACK AS BACKUPS
			JSR SETSPRITE				;NOW LOAD THE SPRITE, WHICH THANKS TO VAR_SPRITESTATUS BEING 4, IT WILL DISABLE THE SPRITE IN VERA.
		PLY
	PLX							;RESTORE X AND Y FROM THE STACKS (BACKUPS)
	;AND NOW WE CHECK THE NEXT SPRITE!
MOVEWEAPONS_L_NOKILL:

MOVEWEAPONS_L_L_NEXT:			;CHECK THE NEXT SPRITE...
	INX							;INCREMENT THE SPRITE COUNTER
	CPX #5						;CHECK IF WE HAVE ALREADY CHECKED 5 SPRITES, AND ARE ABOUT TO CHECK THE 6TH...
	BEQ MOVEWEAPONS_L_L_DONE	;THEN WE'RE DONE CHECKING THE LASER SPRITES IN SPRITE ARRAY 3
	TYA							;TRANSFER Y (OFFSET) TO A
	CLC
	ADC #6						;ADD 6 TO 'Y'
	TAY							;TRANSFER A BACK TO Y, THE OFFSET IS NOW INCREASED BY 6, POINTING TO THE NEXT SPRITE'S STATE...
	BRA MOVEWEAPONS_L_L			;BRANCH BACK TO MOVEWEAPONS_L_L, TO CHECK THE NEXT SPRITE.

MOVEWEAPONS_L_L_DONE:			;DONE CHECKING ENEMY LASERS
	
	
MOVEWEAPONS_DONELASERS:			;DONE CHECKING THE ENEMY LASERS POSITION AND UPDATING IT.		
	RTS
	
;---------------------------------------------
;FUNCTION - HANDLEENEMYFIRE, THIS FUNCTION CHECKS IF THERE ARE ANY ENEMIES THAT NEED TO FIRE LASERS AT THE PLAYER.
;---------------------------------------------
HANDLEENEMYFIRE:
	;WE'RE GOING TO RELY ON THE NUMBER OF ENEMIES LEFT, ALREADY STORED IN VAR_ENEMIES.
	;AND DO A RANDOM NUMBER GENERATION BETWEEN 0 AND VAR_ENEMIES, THEN RUN THROUGH ALL ACTIVE SPRITES IN THE TWO SPRITE DATA ARRAYS
	;AND IF WE GET TO THE RANDOM NUMBER, WE CHECK IF IT'S A RED ENEMY (TYPE=3). IF IT IS, FIRE A MISSILE!
	LDA VAR_ENEMIES				;LOAD THE NUMBER OF ENEMIES LEFT ON SCREEN
	JSR RAND					;NOW GET A RANDOM NUMBER BETWEEN 0 AND THE NUMBER OF ACTIVE ENEMIES ON SCREEN
	CLC
	ADC #1						;ADD ONE TO THE RANDOM NUMBER, SO WE GET A VALUE BETWEEN 1 TO VAR_ENEMIES+1
	STA ZPADDR+10				;STORE THE RANDOM NUMBER IN ZPADDR+10 FOR NOW
	STZ ZPADDR+11				;EMPTY ZPADDR+11, IT WILL CONTAIN THE COUNTER FOR THE NUMBER OF ACTIVE SPRITES FOUND
	
	;---------------------------------------------
	;NOW WE HAVE TO RUN THROUG ALL THE ACTIVE ENEMY SPRITES AND KEEP COUNTING UNTIL WE FIND THE NUMBER, EQUAL TO THE RANDOM NUMBER, NOW IN ZPADDR+10
	;FIRST SPRITE DATA ARRAY 1 (ENEMY SPRITES 0-34)
	LDA #SPRITEARRAY1H
	STA ZPADDR+1				;LOAD THE HIGH BYTE WITH THE HIGH BYTE OF SPRITE DATA ARRAY 1
	STZ ZPADDR					;STORE 0 IN THE LOW BYTE
	LDX #0						;LOAD 0 IN X, IT WILL BE THE SPRITE COUNTER
	LDY #3						;LOAD 3 IN Y, IT WILL BE THE OFFSET (POINTING TO THE SPRITE'S STATE)
HANDLEENEMYFIRE_L1:
	LDA (ZPADDR),Y				;LOAD THE SPRITE'S STATE.
	CMP #2						;CHECK IF THE SPRITE IS LESS THEN 2 (0=DEAD,1=DIEING)
	BCC HANDLEENEMYFIRE_L1_NEXT	;IF IT IS LESS THEN 2, THUS NOT ALIVE, THEN CHECK THE NEXT SPRITE
	LDA ZPADDR+11				;LOAD THE COUNT OF ACTIVE SPRITES
	CLC
	ADC #1						;ADD ONE TO THE ACTIVE SPRITE COUNTER
	STA ZPADDR+11				;STORE THE COUNT OF ACTIVE SPRITES IN ZPADDR+11 AGAIN.
	CMP ZPADDR+10				;CHECK IF THE ACTIVE SPRITE COUNTER IS EQUAL TO THE RANDOM NUMBER WE NEED TO FIND.
	BEQ HANDLEENEMYFIRE_SPRITEFOUND	;IF IT'S EQUAL THEN WE FOUND THE SPRITE, BRANCH TO HANDLEENEMYFIRE_SPRITEFOUND!
	;OTHERWISE WE JUST CHECK THE NEXT SPRITE.	
HANDLEENEMYFIRE_L1_NEXT:		;CHECK THE NEXT SPRITE...
	INX							;INCREMENT THE SPRITE COUNTER
	CPX #35						;CHECK IF WE HAVE ALREADY CHECKED 35 SPRITES, AND ARE ABOUT TO CHECK THE 36TH...
	BEQ HANDLEENEMYFIRE_L1_DONE	;THEN WE'RE DONE CHECKING THE ENEMY SPRITES IN SPRITE ARRAY 1
	TYA							;TRANSFER Y (OFFSET) TO A
	CLC
	ADC #6						;ADD 6 TO 'Y'
	TAY							;TRANSFER A BACK TO Y, THE OFFSET IS NOW INCREASED BY 6, POINTING TO THE NEXT SPRITE'S STATE...
	BRA HANDLEENEMYFIRE_L1		;BRANCH BACK TO HANDLEENEMYFIRE_L1, TO CHECK THE NEXT SPRITE.

HANDLEENEMYFIRE_L1_DONE:		;DONE CHECKING SPRITE ARRAY 1
	;---------------------------------------------
	;NOW SPRITE DATA ARRAY 2 (ENEMY SPRITES 35-54)
	LDA #SPRITEARRAY2H
	STA ZPADDR+1				;LOAD THE HIGH BYTE WITH THE HIGH BYTE OF SPRITE DATA ARRAY 2
	STZ ZPADDR					;STORE 0 IN THE LOW BYTE
	LDX #0						;LOAD 0 IN X, IT WILL BE THE SPRITE COUNTER
	LDY #3						;LOAD 3 IN Y, IT WILL BE THE OFFSET (POINTING TO THE SPRITE'S STATE)
HANDLEENEMYFIRE_L2:
	LDA (ZPADDR),Y				;LOAD THE SPRITE'S STATE.
	CMP #2						;CHECK IF THE SPRITE IS LESS THEN 2 (0=DEAD,1=DIEING)
	BCC HANDLEENEMYFIRE_L2_NEXT	;IF IT IS LESS THEN 2, THUS NOT ALIVE, THEN CHECK THE NEXT SPRITE
	LDA ZPADDR+11				;LOAD THE COUNT OF ACTIVE SPRITES
	CLC
	ADC #1						;ADD ONE TO THE ACTIVE SPRITE COUNTER
	STA ZPADDR+11				;STORE THE COUNT OF ACTIVE SPRITES IN ZPADDR+11 AGAIN.
	CMP ZPADDR+10				;CHECK IF THE ACTIVE SPRITE COUNTER IS EQUAL TO THE RANDOM NUMBER WE NEED TO FIND.
	BEQ HANDLEENEMYFIRE_SPRITEFOUND	;IF IT'S EQUAL THEN WE FOUND THE SPRITE, BRANCH TO HANDLEENEMYFIRE_SPRITEFOUND!
	;OTHERWISE WE JUST CHECK THE NEXT SPRITE.	
HANDLEENEMYFIRE_L2_NEXT:		;CHECK THE NEXT SPRITE...
	INX							;INCREMENT THE SPRITE COUNTER
	CPX #20						;CHECK IF WE HAVE ALREADY CHECKED 20 SPRITES, AND ARE ABOUT TO CHECK THE 21TH...
	BEQ HANDLEENEMYFIRE_SPRITEFOUND_DONE	;THEN WE'RE DONE CHECKING THE ENEMY SPRITES IN SPRITE ARRAY 2
	TYA							;TRANSFER Y (OFFSET) TO A
	CLC
	ADC #6						;ADD 6 TO 'Y'
	TAY							;TRANSFER A BACK TO Y, THE OFFSET IS NOW INCREASED BY 6, POINTING TO THE NEXT SPRITE'S STATE...
	BRA HANDLEENEMYFIRE_L2		;BRANCH BACK TO HANDLEENEMYFIRE_L2, TO CHECK THE NEXT SPRITE.
	
HANDLEENEMYFIRE_SPRITEFOUND:	;HERE WE FOUND THE SPRITE EQUAL TO THE RANDOM NUMBER.
	;NOW WE NEED TO CHECK IF THE ENEMY'S TYPE IS TYPE 3 (RED DUDE), AND IF SO, FIRE A LASER
	DEY							;DECREASE Y BY ONE, SO IT NOW POINTS TO THE SPRITE TYPE
	LDA (ZPADDR),Y				;LOAD THE SPRITE TYPE IN A
	CMP #3						;CHECK IF IT'S EQUAL TO TYPE 3, RED DUDE. 
	BNE HANDLEENEMYFIRE_SPRITEFOUND_DONE	;IF IT'S NOT, THEN WE DO NOTHING, END OF SUB!
	;OTHERWISE WE NEED TO LAUNCH A MISSILE.
	JSR LAUNCHLASER				;LAUNCH A ENEMY LASER!
	;AND END THE SUBROUTINE	
HANDLEENEMYFIRE_SPRITEFOUND_DONE:
	RTS

;---------------------------------------------
;FUNCTION - LAUNCHLASER, HERE WE'RE GOING TO LAUNCH A ENEMY LASER!
;---------------------------------------------		
LAUNCHLASER:
	;LOAD THE X POSITION OF THE ENEMY THAT HAS TO FIRE THE LASER, ZPADDR AND ZPADDR+1 STILL POINT TO THE SPRITE DATA IN QUESTION, OFFSET BY Y (TO THE SPRITE TYPE, BYTE 3)
	DEY
	DEY							;DECREMENT Y TWICE SO IT POINTS TO THE X POSITION OF THE ENEMY SPRITE'S DATA ARRAY
	LDA (ZPADDR),Y				;LOAD THE X POSITION OF THE ENEMY SPRITE
	CLC
	ADC #4						;ADD 4 PIXELS TO THE X POSITION, SO IT STARTS AT THE CENTER OF THE ENEMY SPRITE
	STA ZPADDR+5				;STORE THE NEW X POSITION TO ZPADDR+5
	;NOW WE GET AND STORE THE Y POSITION
	INY							;INCREMENT Y ONCE, SO IT POINTS TO THE 2ND BYTE, THE Y POSITION OF THE ENEMY SPRITE.
	LDA (ZPADDR),Y				;LOAD THE Y POSITION OF THE ENEMY SPRITE
	CLC
	ADC #12						;ADD 12 TO THE Y POSITION, SO IT POINTS JUST BELOW THE BOTTOM OF THE ENEMY SPRITE.
	STA ZPADDR+6				;STORE THE NEW Y POSITION TO ZPADDR+6
	
	;NOW TO LOOP THROUGH THE 5 POSSIBLE LASERS, FIND A DEAD ONE, AND USE THAT TO FILL THE DATA FOR THE NEW LASER BLAST
	;NOW SPRITE DATA ARRAY 2 (ENEMY SPRITES 35-54)
	LDA #SPRITEARRAY3H
	STA ZPADDR+1				;LOAD THE HIGH BYTE WITH THE HIGH BYTE OF SPRITE DATA ARRAY 2
	STZ ZPADDR					;STORE 0 IN THE LOW BYTE
	LDX #0						;LOAD 0 IN X, IT WILL BE THE SPRITE COUNTER
	LDY #15						;LOAD 15 IN Y, IT WILL BE THE OFFSET (POINTING TO THE SPRITE'S STATE), STARTING AT THE ENEMY LASERS
LAUNCHLASER_L:
	LDA (ZPADDR),Y				;LOAD THE LASERS SPRITE'S STATE.
	CMP #0						;CHECK IF THE LASER IS DEAD/UNUSED.
	BNE LAUNCHLASER_L_NEXT	;IF IT'S NOT DEAD/UNUSED, CHECK THE NEXT ONE.
	;IF THE LASER IS DEAD, WE CAN FILL THE DATA IN THIS ONE
	LDA #2
	STA (ZPADDR),Y				;STORE 2 (ALIVE) IN THE SPRITE STATE
	DEY
	DEY
	DEY							;DECREMENT Y THREE TIMES, IT POINTS TO THE X POSITION NOW
	LDA ZPADDR+5				;LOAD THE MODIFIED X POSITION
	STA (ZPADDR),Y				;STORE IT IN THE MISSILE'S X POSITION
	INY							;INCREMENT Y ONCE, NOW IT POINTS TO THE 
	LDA ZPADDR+6				;LOAD THE MODIFIED Y POSITION
	STA (ZPADDR),Y				;STORE IT IN THE MISSILE'S Y POSITION
	INY							;INCREMENT Y AGAIN, NOT IT POINTS TO THE SPRITE TYPE.
	LDA #2
	STA (ZPADDR),Y				;STORE SPRITE TYPE 2 (ENEMY LASER) IN THE SPRITE TYPE
	INY
	INY							;INCREMENT Y TWICE, NOW IT POINTS TO THE VERA START ADDRESS.
	LDA #81						;LOAD 82 AS THE VERA START ADDRESS
	STA (ZPADDR),Y				;STORE THE VERA START ADDRESS
	INY							;INCREMENT Y ONE MORE TIME TO POINT TO THE ANIMATION FRAME NUMBER
	LDA #0
	STA (ZPADDR),Y				;STORE 0 AS THE ANIMATION FRAME NUMBER IN THE LASER
	BRA	LAUNCHLASER_DONE		;AND WE'RE DONE, END LOOP!!
	
LAUNCHLASER_L_NEXT:				;CHECK THE NEXT LASER SPRITE...
	INX							;INCREMENT THE SPRITE COUNTER
	CPX #5						;CHECK IF WE HAVE ALREADY CHECKED ALL 5 LASER'S AND ARE ABOUT TO CHECK THE 6TH...
	BEQ LAUNCHLASER_DONE		;THEN WE'RE DONE CHECKING, NO FREE LASER LEFT, SO WE CAN'T FIRE ANOTHER LASER, BETTER LUCK NEXT TIME!
	TYA							;TRANSFER Y (OFFSET) TO A
	CLC
	ADC #6						;ADD 6 TO 'Y'
	TAY							;TRANSFER A BACK TO Y, THE OFFSET IS NOW INCREASED BY 6, POINTING TO THE NEXT SPRITE'S STATE...
	BRA LAUNCHLASER_L			;BRANCH BACK TO LAUNCHLASER_L, TO CHECK THE NEXT ENEMY LASER

LAUNCHLASER_DONE:	
	RTS

;---------------------------------------------
;FUNCTION - RANGECHECKING_PL_LA, THIS FUNCTION WILL DO RANGE CHECKING ON THE PLAYER SPRITE AGAINST THE ENEMY LASERS
;---------------------------------------------
RANGECHECKING_PL_LA:
	;---------------------------------------------
	;FIRST WE'RE GOING TO DO RANGE CHECKING ON THE PLAYER SPRITE VS THE ACTIVE ENEMY LASERS
	LDA #SPRITEARRAY3H			
	STA ZPADDR+1				;STORE THE HIGH BYTE OF THE 3RD SPRITE DATA ARRAY IN ZPADDR+1
	LDA #3
	STA ZPADDR					;STORE 3 IN ZPADDR, TO POINT THE 16-BIT VALUE TO THE PLAYER SPRITE'S SPRITE STATE.
	LDA (ZPADDR)				;LOAD THE PLAYER'S SPRITE STATE.
	CMP #2						;CHECK THE SPRITE DATA VS 2
	BCC RANGECHECKING_PL_LA_L_DONE	;IF THE PLAYER SPRITE STATE IS LESS THEN 2 (0=DEAD OR 1=DIEING), THEN WE DON'T DON ANY COLLISION CHECKS ON THEM!!!
	;OTHERWISE THE STATE IS 2=ALIVE, LETS LOAD THE X AND Y POSITION OF THE PLAYER, X POSITION FIRST
	LDA #0
	STA ZPADDR					;LOAD 00 IN THE LOW BYTE, POINTING TO THE PLAYER SPRITE'S X POSITION
	LDA (ZPADDR)				;LOAD THE PLAYER SPRITE'S X POSITION
	CLC
	ADC #8						;ADD 8 TO THE X POSITION, SO IT'S IN THE MIDDLE OF THE VISIBLE SPRITE (ON THE X AXIS)
	STA ZPADDR+8				;STORE THE MODIFIED PLAYER MISSILE'S X POSITION IN ZPADDR+8 FOR NOW
	;AND NOW THE Y POSITION
	LDA #1
	STA ZPADDR					;STORE 01 IN THE LOW BYTE, POINTING TO THE PLAYER SPRITE'S Y POSITION
	LDA (ZPADDR)				;LOAD THE PLAYER MISSILE'S Y POSITION
	CLC
	ADC #8						;ADD 8 TO TO THE Y POSITION.
	STA ZPADDR+9				;AND STORE IT IN ZPADDR+9.
	
	;---------------------------------------------
	;NOW WE HAVE TO RUN THROUG ALL THE ENEMY LASER SPRITES, TO SEE IF ANY ARE WITHIN RANGE OF THE PLAYER SPRITE.
	;THE HIGH BYTE (ZPADDR+1) IS STILL LOADED WITH THE HIGH BYTE OF SPRITE DATA ARRAY 3
	STZ ZPADDR					;STORE 0 IN THE LOW BYTE
	LDX #0						;LOAD 0 IN X, IT WILL BE THE SPRITE COUNTER
	LDY #15						;LOAD 15 IN Y, IT WILL BE THE OFFSET (POINTING TO THE SPRITE'S STATE) OF THE FIRST ENEMY LASER
RANGECHECKING_PL_LA_L:
	LDA (ZPADDR),Y				;LOAD THE ENEMY LASER SPRITE'S STATE.
	CMP #0						;CHECK IF THE SPRITE IS DEAD (STATE=0)
	BEQ RANGECHECKING_PL_LA_L_NEXT	;IF SO, THEN CHECK THE NEXT SPRITE
	;IF NOT, THEN IT'S ACTIVE! LOAD THE X AND Y POSITION, Y POSITION FIRST.
	DEY
	DEY							;DECREMENT Y TWICE, NOW IT POINTS TO THE ENEMY LASERS SPRITE'S Y POSITION
	LDA (ZPADDR),Y				;LOAD THE ENEMY SPRITE'S Y POSITION
	CLC
	ADC #8						;ADD 8 TO THE Y POSITION OF THE SPRITE. SO IT TO THE BOTTOM OF THE LASER SPRITE (SPRITE IS 8X8 PIXELS)
	STA ZPADDR+11				;STORE THE MODIFIED Y POSITION TO ZPADDR+11
	;AND NOW THE X POSITION
	DEY							;DECREMENT Y ONCE MORE, NOW IT POINTS TO THE ENEMY SPRITE'S X POSITION
	LDA (ZPADDR),Y				;LOAD THE ENEMY SPRITE'S X POSITION
	CLC
	ADC #4						;ADD 4 TO THE X POSITION, SO IT'S IN THE MIDDLE OF THE VISIBLE SPRITE.
	STA ZPADDR+10				;STORE THE MODIFIED X POSITION TO ZPADDR+10
	INY
	INY
	INY							;INCREMENT Y THREE TIMES, SO IT POINTS BACK TO THE SPRITE STATE
	;NOW WE JUST NEED TO TELL THE RANGECHECK FUNCTION WHAT THE TWO SPRITES ARE, TO DO PROPER RANGE TESTING
	LDA #2
	STA ZPADDR+12				;STORE 2 IN ZPADDR+12, SO IT KNOWS THAT SPRITE 1 IS THE PLAYER  (1=ENEMY, 2=PLAYER, 4=PLAYER MISSILE, 8=ENEMY LASER)
	LDA #8
	STA ZPADDR+13				;STORE 8 IN ZPADDR+13, SO IT KNOWS THAT SPRITE 2 IS A ENEMY LASER (1=ENEMY, 2=PLAYER, 4=PLAYER MISSILE, 8=ENEMY LASER)
	PHX							;PUSH X TO THE STACK, THE COLLISIONCHECK SUB AND SUB-SUBS WILL USE IT TOO...
		JSR COLLISIONCHECK		;DO COLLISION CHECKING BASED ON THE VALUES STORED 
	PLX							;AND PULL X FROM THE STACK AGAIN, RESTORING THE VALUE.
	;THE RETURNED VALUE IS NOW IN A
	CMP #0						;CHECK IF WE HAVE NOT COLLIDED
	BEQ RANGECHECKING_PL_LA_L_NEXT	;IF NOT, THEN CHECK THE NEXT SPRITE...
	BNE RANGECHECKING_PL_LA_COLLISION	;THE PLAYER HAS HIT A ENEMY LASER, DEAL WITH THE CONSEQUENCES
	
RANGECHECKING_PL_LA_L_NEXT:		;CHECK THE NEXT SPRITE...
	INX							;INCREMENT THE SPRITE COUNTER
	CPX #5						;CHECK IF WE HAVE ALREADY CHECKED 5 SPRITES, AND ARE ABOUT TO CHECK THE 6TH...
	BEQ RANGECHECKING_PL_LA_L_DONE	;THEN WE'RE DONE CHECKING THE PLAYER MISSILE VS THE FIRST SPRITE DATA ARRAY
	TYA							;TRANSFER Y (OFFSET) TO A
	CLC
	ADC #6						;ADD 6
	TAY							;TRANSFER A BACK TO Y, THE OFFSET IS NOW INCREASED BY 6, POINTING TO THE NEXT SPRITE'S STATE...
	BRA RANGECHECKING_PL_LA_L	;BRANCH BACK TO RANGECHECKING_PL_LA_L, TO CHECK THE NEXT ENEMY LASER SPRITE.

RANGECHECKING_PL_LA_COLLISION:	
	;---------------------------------------------
	;FIRST WE UPDATE THE ENEMY SPRITE, AS ZPADDR AND ZPADDR+1 OFFSET BY Y IS STILL POINTING TO THE SPRITE STATE IN THE SPRITE DATA ARRAY (ARRAY 1 OR 2)
	LDA #0
	STA (ZPADDR),Y				;STORE THE SPRITE STATE OF 0 (DEAD) IN THE ENEMY LASER SPRITE'S DATA ARRAY
	;BUT THAT'S NOT ALL, WE NEED TO ACTUALLY DISABLE THE SPRITE ITSELF, AS CHANGING THE STATE TO DEAD IN THE DATA ARRAY JUST STOPS IT FROM UPDATING, BUT THE SPRITE WILL STILL BE ON SCREEN.
	LDA #4
	STA VAR_SPRITESTATUS		;SET 4 IN THE SPRITE STATUS VARIABLE, TO TELL THE SETSPRITE SUB TO DISABLE THE SPRITE
	TXA							;TRANSFER THE COUNTER/SPRITE NUMBER TO A
	CLC
	ADC #57						;ADD 57 TO IT, SO A POINTS TO THE RIGHT ENEMY LASER SPRITE (57-61)
	JSR SETSPRITE				;NOW LOAD THE SPRITE, WHICH THANKS TO VAR_SPRITESTATUS BEING 4, IT WILL DISABLE THE SPRITE IN VERA.
	;NOW UPDATE THE PLAYER'S SPRITE STATE
	LDA #SPRITEARRAY3H			;LOAD THE HIGH BYTE FOR SPRITE DATA ARRAY 3
	STA ZPADDR+1				;STORE IN THE HIGH BYTE OF THE 16-BIT PAIR
	LDA #3
	STA ZPADDR					;STORE 3 IN THE LOW BYTE OF THE 16-BIT PAIR, POINTING TO THE SPRITE STATE (OF THE PLAYER SPRITE)
	LDA #1
	STA (ZPADDR)				;STORE THE STATE 1 (DIEING) IN THE PLAYER DATA ARRAY.	
	LDA #5
	STA ZPADDR					;STORE 5 IN THE LOW BYTE, POINTING TO THE PLAYER SPRITE'S ANIMATION FRAME
	LDA #0
	STA (ZPADDR)				;STORE 0 IN THE PLAYER SPRITE'S ANIMATION FRAME, SO IT CAN DO A FULL 4 FRAME EXPLOSION ANIMATION.
	;AND WE'RE DONE, WE CAN EXIT THE LOOP NOW...
RANGECHECKING_PL_LA_L_DONE:		;DONE CHECKING ALL MISSILES!
	RTS
	
;---------------------------------------------
;FUNCTION - RANGECHECKING_PL_EN, THIS FUNCTION WILL DO RANGE CHECKING ON THE PLAYER SPRITE (AGAINST ENEMIES) AND ENEMY LASERS ON THE PLAYER'S POSITION
;---------------------------------------------
RANGECHECKING_PL_EN:
	;---------------------------------------------
	;FIRST WE'RE GOING TO DO RANGE CHECKING ON THE PLAYER SPRITE VS THE ACTIVE ENEMY SPRITES
	LDA #SPRITEARRAY3H			
	STA ZPADDR+1				;STORE THE HIGH BYTE OF THE 3RD SPRITE DATA ARRAY IN ZPADDR+1
	LDA #3
	STA ZPADDR					;STORE 3 IN ZPADDR, TO POINT THE 16-BIT VALUE TO THE PLAYER SPRITE'S SPRITE STATE.
	LDA (ZPADDR)				;LOAD THE PLAYER'S SPRITE STATE.
	CMP #2						;CHECK THE SPRITE DATA VS 2
	BCC RANGECHECKING_PL_EN_DONEP_J	;IF THE PLAYER SPRITE STATE IS LESS THEN 2 (0=DEAD OR 1=DIEING), THEN WE DON'T DON ANY COLLISION CHECKS ON THEM!!!
	;OTHERWISE THE STATE IS 2=ALIVE, LETS LOAD THE X AND Y POSITION OF THE PLAYER, X POSITION FIRST
	LDA #0
	STA ZPADDR					;LOAD 00 IN THE LOW BYTE, POINTING TO THE PLAYER SPRITE'S X POSITION
	LDA (ZPADDR)				;LOAD THE PLAYER SPRITE'S X POSITION
	CLC
	ADC #8						;ADD 8 TO THE X POSITION, SO IT'S IN THE MIDDLE OF THE VISIBLE SPRITE (ON THE X AXIS)
	STA ZPADDR+8				;STORE THE MODIFIED PLAYER MISSILE'S X POSITION IN ZPADDR+8 FOR NOW
	;AND NOW THE Y POSITION
	LDA #1
	STA ZPADDR					;STORE 01 IN THE LOW BYTE, POINTING TO THE PLAYER SPRITE'S Y POSITION
	LDA (ZPADDR)				;LOAD THE PLAYER MISSILE'S Y POSITION
	CLC
	ADC #8						;ADD 8 TO TO THE Y POSITION.
	STA ZPADDR+9				;AND STORE IT IN ZPADDR+9.
	
	;---------------------------------------------
	;NOW WE HAVE TO RUN THROUG ALL THE ENEMY SPRITES, TO SEE IF ANY ARE WITHIN RANGE OF THE PLAYER SPRITE.
	;FIRST SPRITE DATA ARRAY 1 (ENEMY SPRITES 0-34)
	LDA #SPRITEARRAY1H
	STA ZPADDR+1				;LOAD THE HIGH BYTE WITH THE HIGH BYTE OF SPRITE DATA ARRAY 1
	STZ ZPADDR					;STORE 0 IN THE LOW BYTE
	LDX #0						;LOAD 0 IN X, IT WILL BE THE SPRITE COUNTER
	LDY #3						;LOAD 3 IN Y, IT WILL BE THE OFFSET (POINTING TO THE SPRITE'S STATE)
RANGECHECKING_PL_EN_PL1:
	LDA (ZPADDR),Y				;LOAD THE SPRITE'S STATE.
	CMP #0						;CHECK IF THE SPRITE IS DEAD (STATE=0)
	BEQ RANGECHECKING_PL_EN_PL1_NEXT	;IF SO, THEN CHECK THE NEXT SPRITE
	;IF THE ENEMY SPRITE IS NOT DEAD, WE NEED TO CHECK IF IT'S DIEING...
	CMP #1						;CHECK IF THE ENEMY SPRITE IS DIEING (STATE=1)
	BEQ RANGECHECKING_PL_EN_PL1_NEXT	;IF SO, THEN CHECK THE NEXT SPRITE, WE DONT NEED TO DO RANGE CHECKING ON A DIEING/EXPLODING ENEMY.
	;IF NOT, THEN IT'S ALIVE! LOAD THE X AND Y POSITION, Y POSITION FIRST.
	DEY
	DEY							;DECREMENT Y TWICE, NOW IT POINTS TO THE ENEMY SPRITE'S Y POSITION
	LDA (ZPADDR),Y				;LOAD THE ENEMY SPRITE'S Y POSITION
	CLC
	ADC #8						;ADD 8 TO THE Y POSITION OF THE SPRITE. SO IT TO THE CENTER OF THE VISISIBLE SPRITE. (SPRITE IS 16X16 IN SIZE)
	STA ZPADDR+11				;STORE THE MODIFIED Y POSITION TO ZPADDR+11
	;AND NOW THE X POSITION
	DEY							;DECREMENT Y ONCE MORE, NOW IT POINTS TO THE ENEMY SPRITE'S X POSITION
	LDA (ZPADDR),Y				;LOAD THE ENEMY SPRITE'S X POSITION
	CLC
	ADC #8						;ADD 8 TO THE X POSITION, SO IT'S IN THE MIDDLE OF THE VISIBLE SPRITE.
	STA ZPADDR+10				;STORE THE MODIFIED X POSITION TO ZPADDR+10
	INY
	INY
	INY							;INCREMENT Y THREE TIMES, SO IT POINTS BACK TO THE SPRITE STATE
	;NOW WE JUST NEED TO TELL THE RANGECHECK FUNCTION WHAT THE TWO SPRITES ARE, TO DO PROPER RANGE TESTING
	LDA #2
	STA ZPADDR+12				;STORE 2 IN ZPADDR+12, SO IT KNOWS THAT SPRITE 1 IS A PLAYER MISSILE (1=ENEMY, 2=PLAYER, 4=PLAYER MISSILE, 8=ENEMY LASER)
	LDA #1
	STA ZPADDR+13				;STORE 1 IN ZPADDR+13, SO IT KNOWS THAT SPRITE 2 IS A ENEMY (1=ENEMY, 2=PLAYER, 4=PLAYER MISSILE, 8=ENEMY LASER)
	PHX							;PUSH X TO THE STACK, THE COLLISIONCHECK SUB AND SUB-SUBS WILL USE IT TOO...
		JSR COLLISIONCHECK		;DO COLLISION CHECKING BASED ON THE VALUES STORED 
	PLX							;AND PULL X FROM THE STACK AGAIN, RESTORING THE VALUE.
	;THE RETURNED VALUE IS NOW IN A
	CMP #0						;CHECK IF WE HAVE NOT COLLIDED
	BEQ RANGECHECKING_PL_EN_PL1_NEXT	;IF NOT, THEN CHECK THE NEXT SPRITE...
	BNE RANGECHECKING_PL_EN_COLLISION	;WE HAVE HIT THE ENEMY SPRITE, HANDLE THE CHANGING OF THE SPRITE STATES!

RANGECHECKING_PL_EN_DONEP_J:	;INTERMEDIARY JUMP POINT, BRANCH DESTINATION OUT OF RANGE
	JMP RANGECHECKING_PL_EN_DONEP
	
RANGECHECKING_PL_EN_PL1_NEXT:	;CHECK THE NEXT SPRITE...
	INX							;INCREMENT THE SPRITE COUNTER
	CPX #35						;CHECK IF WE HAVE ALREADY CHECKED 35 SPRITES, AND ARE ABOUT TO CHECK THE 36TH...
	BEQ RANGECHECKING_PL_EN_PL1_DONE	;THEN WE'RE DONE CHECKING THE PLAYER MISSILE VS THE FIRST SPRITE DATA ARRAY
	TYA							;TRANSFER Y (OFFSET) TO A
	CLC
	ADC #6						;ADD 6
	TAY							;TRANSFER A BACK TO Y, THE OFFSET IS NOW INCREASED BY 6, POINTING TO THE NEXT SPRITE'S STATE...
	BRA RANGECHECKING_PL_EN_PL1	;BRANCH BACK TO RANGECHECKING_PL1, TO CHECK THE NEXT SPRITE.

	
RANGECHECKING_PL_EN_PL1_DONE:	;DONE CHECKING THE PLAYER MISSILE'S RANGE VS THE SPRITES IN SPRITE DATA ARRAY 1
	;---------------------------------------------
	;NOW SPRITE DATA ARRAY 2 (ENEMY SPRITES 35-54)
	LDA #SPRITEARRAY2H
	STA ZPADDR+1				;LOAD THE HIGH BYTE WITH THE HIGH BYTE OF SPRITE DATA ARRAY 1
	STZ ZPADDR					;STORE 0 IN THE LOW BYTE
	LDX #0						;LOAD 0 IN X, IT WILL BE THE SPRITE COUNTER
	LDY #3						;LOAD 3 IN Y, IT WILL BE THE OFFSET (POINTING TO THE SPRITE'S STATE)
RANGECHECKING_PL_EN_PL2:
	LDA (ZPADDR),Y				;LOAD THE SPRITE'S STATE.
	CMP #0						;CHECK IF THE SPRITE IS DEAD (STATE=0)
	BEQ RANGECHECKING_PL_EN_PL2_NEXT	;IF SO, THEN CHECK THE NEXT SPRITE
	;IF THE ENEMY SPRITE IS NOT DEAD, WE NEED TO CHECK IF IT'S DIEING...
	CMP #1						;CHECK IF THE ENEMY SPRITE IS DIEING (STATE=1)
	BEQ RANGECHECKING_PL_EN_PL2_NEXT	;IF SO, THEN CHECK THE NEXT SPRITE, WE DONT NEED TO DO RANGE CHECKING ON A DIEING/EXPLODING ENEMY.
	;IF NOT, THEN IT'S ALIVE! LOAD THE X AND Y POSITION, Y POSITION FIRST.
	DEY
	DEY							;DECREMENT Y TWICE, NOW IT POINTS TO THE ENEMY SPRITE'S Y POSITION
	LDA (ZPADDR),Y				;LOAD THE ENEMY SPRITE'S Y POSITION
	CLC
	ADC #8						;ADD 8 TO THE Y POSITION OF THE SPRITE. SO IT TO THE CENTER OF THE VISISIBLE SPRITE. (SPRITE IS 16X16 IN SIZE)
	STA ZPADDR+11				;STORE THE MODIFIED Y POSITION TO ZPADDR+11
	;AND NOW THE X POSITION
	DEY							;DECREMENT Y ONCE MORE, NOW IT POINTS TO THE ENEMY SPRITE'S X POSITION
	LDA (ZPADDR),Y				;LOAD THE ENEMY SPRITE'S X POSITION
	CLC
	ADC #8						;ADD 8 TO THE X POSITION, SO IT'S IN THE MIDDLE OF THE VISIBLE SPRITE.
	STA ZPADDR+10				;STORE THE MODIFIED X POSITION TO ZPADDR+10
	INY
	INY
	INY							;INCREMENT Y THREE TIMES, SO IT POINTS BACK TO THE SPRITE STATE
	;NOW WE JUST NEED TO TELL THE RANGECHECK FUNCTION WHAT THE TWO SPRITES ARE, TO DO PROPER RANGE TESTING
	LDA #2
	STA ZPADDR+12				;STORE 2 IN ZPADDR+12, SO IT KNOWS THAT SPRITE 1 IS A PLAYER MISSILE (1=ENEMY, 2=PLAYER, 4=PLAYER MISSILE, 8=ENEMY LASER)
	LDA #1
	STA ZPADDR+13				;STORE 1 IN ZPADDR+13, SO IT KNOWS THAT SPRITE 2 IS A ENEMY (1=ENEMY, 2=PLAYER, 4=PLAYER MISSILE, 8=ENEMY LASER)
	PHX							;PUSH X TO THE STACK, THE COLLISIONCHECK SUB AND SUB-SUBS WILL USE IT TOO...
		JSR COLLISIONCHECK		;DO COLLISION CHECKING BASED ON THE VALUES STORED 
	PLX							;AND PULL X FROM THE STACK AGAIN, RESTORING THE VALUE.
	;THE RETURNED VALUE IS NOW IN A
	CMP #0						;CHECK IF WE HAVE NOT COLLIDED
	BEQ RANGECHECKING_PL_EN_PL2_NEXT	;IF NOT, THEN CHECK THE NEXT SPRITE...
	BNE RANGECHECKING_PL_EN_COLLISION	;WE HAVE HIT THE ENEMY SPRITE, HANDLE THE CHANGING OF THE SPRITE STATES!
	
RANGECHECKING_PL_EN_PL2_NEXT:	;CHECK THE NEXT SPRITE...
	INX							;INCREMENT THE SPRITE COUNTER
	CPX #20						;CHECK IF WE HAVE ALREADY CHECKED 20 SPRITES, AND ARE ABOUT TO CHECK THE 21TH...
	BEQ RANGECHECKING_PL_EN_PL2_DONE	;THEN WE'RE DONE CHECKING THE PLAYER MISSILE VS THE SECOND SPRITE DATA ARRAY
	TYA							;TRANSFER Y (OFFSET) TO A
	CLC
	ADC #6						;ADD 6
	TAY							;TRANSFER A BACK TO Y, THE OFFSET IS NOW INCREASED BY 6, POINTING TO THE NEXT SPRITE'S STATE...
	BRA RANGECHECKING_PL_EN_PL2	;BRANCH BACK TO RANGECHECKING_PL2, TO CHECK THE NEXT SPRITE.

RANGECHECKING_PL_EN_PL2_DONE:			;DONE CHECKING THE PLAYER MISSILE'S RANGE VS THE SPRITES IN SPRITE DATA ARRAY 2
	BRA RANGECHECKING_PL_EN_DONEP		;TEMPORARY... REMOVE LATER
	
RANGECHECKING_PL_EN_COLLISION:		;IF WE GET OVER HERE, THEN WE HAVE A COLLISION! WE HAVE TO LOSE A LIVE AND LOAD THE LEVEL AGAIN.
	;---------------------------------------------
	;FIRST WE UPDATE THE ENEMY SPRITE, AS ZPADDR AND ZPADDR+1 OFFSET BY Y IS STILL POINTING TO THE SPRITE STATE IN THE SPRITE DATA ARRAY (ARRAY 1 OR 2)
	LDA #1
	STA (ZPADDR),Y				;STORE THE SPRITE STATE OF 1 (DIEING) IN THE ENEMY SPRITE'S DATA ARRAY
	INY
	INY							;INCREMENT Y TWICE, TO IT POINTS TO THE 6TH BYTE, THE ANIMATION FRAME NUMBER.
	LDA #0					
	STA (ZPADDR),Y				;LOAD 0 INTO THE SPRITE'S ANIMATION FRAME NUMBER (SO IT CAN DO A FULL 4 FRAMES OF EXPLODING.)
	;NOW UPDATE THE PLAYER'S SPRITE STATE
	LDA #SPRITEARRAY3H			;LOAD THE HIGH BYTE FOR SPRITE DATA ARRAY 3
	STA ZPADDR+1				;STORE IN THE HIGH BYTE OF THE 16-BIT PAIR
	LDA #3
	STA ZPADDR					;STORE 3 IN THE LOW BYTE OF THE 16-BIT PAIR, POINTING TO THE SPRITE STATE (OF THE PLAYER SPRITE)
	LDA #1
	STA (ZPADDR)				;STORE THE STATE 1 (DIEING) IN THE PLAYER MISSILE DATA ARRAY.	
	LDA #5
	STA ZPADDR					;STORE 5 IN THE LOW BYTE, POINTING TO THE PLAYER SPRITE'S ANIMATION FRAME
	LDA #0
	STA (ZPADDR)				;STORE 0 IN THE PLAYER SPRITE'S ANIMATION FRAME, SO IT CAN DO A FULL 4 FRAME EXPLOSION ANIMATION.
	;AND WE'RE DONE, WE CAN EXIT THE LOOP NOW...
	
RANGECHECKING_PL_EN_DONEP:		;DONE RANGECHECKING PLAYER 
	RTS	
	
;---------------------------------------------
;FUNCTION - RANGECHECKING_MS_EN, THIS FUNCTION WILL DO RANGE CHECKING ON THE PLAYER MISSILE (AGAINST ENEMIES) AND ENEMY LASERS ON THE PLAYER'S POSITION
;---------------------------------------------
RANGECHECKING_MS_EN:
	;---------------------------------------------
	;FIRST WE'RE GOING TO DO RANGE CHECKING ON THE PLAYER MISSILE VS THE ACTIVE ENEMY SPRITES
	LDA #SPRITEARRAY3H			
	STA ZPADDR+1				;STORE THE HIGH BYTE OF THE 3RD SPRITE DATA ARRAY IN ZPADDR+1
	LDA #9
	STA ZPADDR					;STORE 09 IN THE LOW BYTE, MAKING IT POINT TO THE PLAYER MISSILE'S SPRITE STATE
	LDA (ZPADDR)				;LOAD THE PLAYER MISSILE'S SPRITE STATE IN A
	CMP #0						;CHECK IF THE PLAYER MISSILE IS DEAD/INACTIVE. 
	BEQ RANGECHECKING_MS_EN_DONEP_J	;IF IT IS DEAD, THEN WE DONT NEED TO DO ANY RANGE CHECKING ON THE PLAYER MISSILE... 
	;IF THE PLAYER MISSILE SPRITE IS NOT DEAD, THEN  GET PLAYER MISSILE'S X AND Y POSITION...
	LDA #6
	STA ZPADDR					;LOAD 06 IN THE LOW BYTE, POINTING TO THE PLAYER MISSILE'S X POSITION
	LDA (ZPADDR)				;LOAD THE PLAYER MISSILE'S X POSITION
	CLC
	ADC #4						;ADD 4 TO THE X POSITION, SO IT'S IN THE MIDDLE OF THE VISIBLE SPRITE (ON THE X AXIS)
	STA ZPADDR+8				;STORE THE MODIFIED PLAYER MISSILE'S X POSITION IN ZPADDR+8 FOR NOW
	;AND NOW THE Y POSITION
	LDA #7
	STA ZPADDR					;STORE 07 IN THE LOW BYTE, POINTING TO THE PLAYER MISSILE'S Y POSITION
	LDA (ZPADDR)				;LOAD THE PLAYER MISSILE'S Y POSITION
	STA ZPADDR+9				;AND STORE IT IN ZPADDR+9, WE DONT NEED TO MODIFY IT, AS IT'S ALREADY AT THE STOP OF THE VISIBLE SPRITE, WHERE WE WANT IT
	
	;---------------------------------------------
	;NOW WE HAVE TO RUN THROUG ALL THE ENEMY SPRITES, TO SEE IF ANY ARE WITHIN RANGE OF THE PLAYER'S MISSILE.
	;FIRST SPRITE DATA ARRAY 1 (ENEMY SPRITES 0-34)
	LDA #SPRITEARRAY1H
	STA ZPADDR+1				;LOAD THE HIGH BYTE WITH THE HIGH BYTE OF SPRITE DATA ARRAY 1
	STZ ZPADDR					;STORE 0 IN THE LOW BYTE
	LDX #0						;LOAD 0 IN X, IT WILL BE THE SPRITE COUNTER
	LDY #3						;LOAD 3 IN Y, IT WILL BE THE OFFSET (POINTING TO THE SPRITE'S STATE)
RANGECHECKING_MS_EN_PL1:
	LDA (ZPADDR),Y				;LOAD THE SPRITE'S STATE.
	CMP #0						;CHECK IF THE SPRITE IS DEAD (STATE=0)
	BEQ RANGECHECKING_MS_EN_PL1_NEXT	;IF SO, THEN CHECK THE NEXT SPRITE
	;IF THE ENEMY SPRITE IS NOT DEAD, WE NEED TO CHECK IF IT'S DIEING...
	CMP #1						;CHECK IF THE ENEMY SPRITE IS DIEING (STATE=1)
	BEQ RANGECHECKING_MS_EN_PL1_NEXT	;IF SO, THEN CHECK THE NEXT SPRITE, WE DONT NEED TO DO RANGE CHECKING ON A DIEING/EXPLODING ENEMY.
	;IF NOT, THEN IT'S ALIVE! LOAD THE X AND Y POSITION, Y POSITION FIRST.
	DEY
	DEY							;DECREMENT Y TWICE, NOW IT POINTS TO THE ENEMY SPRITE'S Y POSITION
	LDA (ZPADDR),Y				;LOAD THE ENEMY SPRITE'S Y POSITION
	CLC
	ADC #8						;ADD 8 TO THE Y POSITION OF THE SPRITE. SO IT TO THE CENTER OF THE VISISIBLE SPRITE. (SPRITE IS 16X16 IN SIZE)
	STA ZPADDR+11				;STORE THE MODIFIED Y POSITION TO ZPADDR+11
	;AND NOW THE X POSITION
	DEY							;DECREMENT Y ONCE MORE, NOW IT POINTS TO THE ENEMY SPRITE'S X POSITION
	LDA (ZPADDR),Y				;LOAD THE ENEMY SPRITE'S X POSITION
	CLC
	ADC #8						;ADD 8 TO THE X POSITION, SO IT'S IN THE MIDDLE OF THE VISIBLE SPRITE.
	STA ZPADDR+10				;STORE THE MODIFIED X POSITION TO ZPADDR+10
	INY
	INY
	INY							;INCREMENT Y THREE TIMES, SO IT POINTS BACK TO THE SPRITE STATE
	;NOW WE JUST NEED TO TELL THE RANGECHECK FUNCTION WHAT THE TWO SPRITES ARE, TO DO PROPER RANGE TESTING
	LDA #4
	STA ZPADDR+12				;STORE 4 IN ZPADDR+12, SO IT KNOWS THAT SPRITE 1 IS A PLAYER MISSILE (1=ENEMY, 2=PLAYER, 4=PLAYER MISSILE, 8=ENEMY LASER)
	LDA #1
	STA ZPADDR+13				;STORE 1 IN ZPADDR+13, SO IT KNOWS THAT SPRITE 2 IS A ENEMY (1=ENEMY, 2=PLAYER, 4=PLAYER MISSILE, 8=ENEMY LASER)
	PHX							;PUSH X TO THE STACK, THE COLLISIONCHECK SUB AND SUB-SUBS WILL USE IT TOO...
		JSR COLLISIONCHECK		;DO COLLISION CHECKING BASED ON THE VALUES STORED 
	PLX							;AND PULL X FROM THE STACK AGAIN, RESTORING THE VALUE.
	;THE RETURNED VALUE IS NOW IN A
	CMP #0						;CHECK IF WE HAVE NOT COLLIDED
	BEQ RANGECHECKING_MS_EN_PL1_NEXT	;IF NOT, THEN CHECK THE NEXT SPRITE...
	BNE RANGECHECKING_MS_EN_COLLISION	;WE HAVE HIT THE ENEMY SPRITE, HANDLE THE CHANGING OF THE SPRITE STATES!

RANGECHECKING_MS_EN_DONEP_J:			;INTERMEDIARY JUMP POINT, BRANCH DESTINATION OUT OF RANGE
	JMP RANGECHECKING_MS_EN_DONEP
	
RANGECHECKING_MS_EN_PL1_NEXT:			;CHECK THE NEXT SPRITE...
	INX							;INCREMENT THE SPRITE COUNTER
	CPX #35						;CHECK IF WE HAVE ALREADY CHECKED 35 SPRITES, AND ARE ABOUT TO CHECK THE 36TH...
	BEQ RANGECHECKING_MS_EN_PL1_DONE	;THEN WE'RE DONE CHECKING THE PLAYER MISSILE VS THE FIRST SPRITE DATA ARRAY
	TYA							;TRANSFER Y (OFFSET) TO A
	CLC
	ADC #6						;ADD 6
	TAY							;TRANSFER A BACK TO Y, THE OFFSET IS NOW INCREASED BY 6, POINTING TO THE NEXT SPRITE'S STATE...
	BRA RANGECHECKING_MS_EN_PL1		;BRANCH BACK TO RANGECHECKING_PL1, TO CHECK THE NEXT SPRITE.

	
RANGECHECKING_MS_EN_PL1_DONE:			;DONE CHECKING THE PLAYER MISSILE'S RANGE VS THE SPRITES IN SPRITE DATA ARRAY 1
	;---------------------------------------------
	;NOW SPRITE DATA ARRAY 2 (ENEMY SPRITES 35-54)
	LDA #SPRITEARRAY2H
	STA ZPADDR+1				;LOAD THE HIGH BYTE WITH THE HIGH BYTE OF SPRITE DATA ARRAY 1
	STZ ZPADDR					;STORE 0 IN THE LOW BYTE
	LDX #0						;LOAD 0 IN X, IT WILL BE THE SPRITE COUNTER
	LDY #3						;LOAD 3 IN Y, IT WILL BE THE OFFSET (POINTING TO THE SPRITE'S STATE)
RANGECHECKING_MS_EN_PL2:
	LDA (ZPADDR),Y				;LOAD THE SPRITE'S STATE.
	CMP #0						;CHECK IF THE SPRITE IS DEAD (STATE=0)
	BEQ RANGECHECKING_MS_EN_PL2_NEXT	;IF SO, THEN CHECK THE NEXT SPRITE
	;IF THE ENEMY SPRITE IS NOT DEAD, WE NEED TO CHECK IF IT'S DIEING...
	CMP #1						;CHECK IF THE ENEMY SPRITE IS DIEING (STATE=1)
	BEQ RANGECHECKING_MS_EN_PL2_NEXT	;IF SO, THEN CHECK THE NEXT SPRITE, WE DONT NEED TO DO RANGE CHECKING ON A DIEING/EXPLODING ENEMY.
	;IF NOT, THEN IT'S ALIVE! LOAD THE X AND Y POSITION, Y POSITION FIRST.
	DEY
	DEY							;DECREMENT Y TWICE, NOW IT POINTS TO THE ENEMY SPRITE'S Y POSITION
	LDA (ZPADDR),Y				;LOAD THE ENEMY SPRITE'S Y POSITION
	CLC
	ADC #8						;ADD 8 TO THE Y POSITION OF THE SPRITE. SO IT TO THE CENTER OF THE VISISIBLE SPRITE. (SPRITE IS 16X16 IN SIZE)
	STA ZPADDR+11				;STORE THE MODIFIED Y POSITION TO ZPADDR+11
	;AND NOW THE X POSITION
	DEY							;DECREMENT Y ONCE MORE, NOW IT POINTS TO THE ENEMY SPRITE'S X POSITION
	LDA (ZPADDR),Y				;LOAD THE ENEMY SPRITE'S X POSITION
	CLC
	ADC #8						;ADD 8 TO THE X POSITION, SO IT'S IN THE MIDDLE OF THE VISIBLE SPRITE.
	STA ZPADDR+10				;STORE THE MODIFIED X POSITION TO ZPADDR+10
	INY
	INY
	INY							;INCREMENT Y THREE TIMES, SO IT POINTS BACK TO THE SPRITE STATE
	;NOW WE JUST NEED TO TELL THE RANGECHECK FUNCTION WHAT THE TWO SPRITES ARE, TO DO PROPER RANGE TESTING
	LDA #4
	STA ZPADDR+12				;STORE 4 IN ZPADDR+12, SO IT KNOWS THAT SPRITE 1 IS A PLAYER MISSILE (1=ENEMY, 2=PLAYER, 4=PLAYER MISSILE, 8=ENEMY LASER)
	LDA #1
	STA ZPADDR+13				;STORE 1 IN ZPADDR+13, SO IT KNOWS THAT SPRITE 2 IS A ENEMY (1=ENEMY, 2=PLAYER, 4=PLAYER MISSILE, 8=ENEMY LASER)
	PHX							;PUSH X TO THE STACK, THE COLLISIONCHECK SUB AND SUB-SUBS WILL USE IT TOO...
		JSR COLLISIONCHECK		;DO COLLISION CHECKING BASED ON THE VALUES STORED 
	PLX							;AND PULL X FROM THE STACK AGAIN, RESTORING THE VALUE.
	;THE RETURNED VALUE IS NOW IN A
	CMP #0						;CHECK IF WE HAVE NOT COLLIDED
	BEQ RANGECHECKING_MS_EN_PL2_NEXT	;IF NOT, THEN CHECK THE NEXT SPRITE...
	BNE RANGECHECKING_MS_EN_COLLISION	;WE HAVE HIT THE ENEMY SPRITE, HANDLE THE CHANGING OF THE SPRITE STATES!
	
RANGECHECKING_MS_EN_PL2_NEXT:			;CHECK THE NEXT SPRITE...
	INX							;INCREMENT THE SPRITE COUNTER
	CPX #20						;CHECK IF WE HAVE ALREADY CHECKED 20 SPRITES, AND ARE ABOUT TO CHECK THE 21TH...
	BEQ RANGECHECKING_MS_EN_PL2_DONE	;THEN WE'RE DONE CHECKING THE PLAYER MISSILE VS THE SECOND SPRITE DATA ARRAY
	TYA							;TRANSFER Y (OFFSET) TO A
	CLC
	ADC #6						;ADD 6
	TAY							;TRANSFER A BACK TO Y, THE OFFSET IS NOW INCREASED BY 6, POINTING TO THE NEXT SPRITE'S STATE...
	BRA RANGECHECKING_MS_EN_PL2		;BRANCH BACK TO RANGECHECKING_PL2, TO CHECK THE NEXT SPRITE.

RANGECHECKING_MS_EN_PL2_DONE:			;DONE CHECKING THE PLAYER MISSILE'S RANGE VS THE SPRITES IN SPRITE DATA ARRAY 2
	
	BRA RANGECHECKING_MS_EN_DONEP		;TEMPORARY... REMOVE LATER
	
RANGECHECKING_MS_EN_COLLISION:		;IF WE GET OVER HERE, THEN WE HAVE A COLLISION! REMOVE THE PLAYER MISSILE, AND MARK THE ENEMY SPRITE AS DIEING!
	;---------------------------------------------
	;FIRST WE UPDATE THE SPRITE, AS ZPADDR AND ZPADDR+1 OFFSET BY Y IS STILL POINTING TO THE SPRITE STATE IN THE SPRITE DATA ARRAY (ARRAY 1 OR 2)
	LDA #1
	STA (ZPADDR),Y				;STORE THE SPRITE STATE OF 1 (DIEING) IN THE ENEMY SPRITE'S DATA ARRAY
	INY
	INY							;INCREMENT Y TWICE, TO IT POINTS TO THE 6TH BYTE, THE ANIMATION FRAME NUMBER.
	LDA #0					
	STA (ZPADDR),Y				;LOAD 0 INTO THE SPRITE'S ANIMATION FRAME NUMBER (SO IT CAN DO A FULL 4 FRAMES OF EXPLODING.)
	;NOW UPDATE THE MISSILE'S STATE
	LDA #SPRITEARRAY3H			;LOAD THE HIGH BYTE FOR SPRITE DATA ARRAY 3
	STA ZPADDR+1				;STORE IN THE HIGH BYTE OF THE 16-BIT PAIR
	LDA #9
	STA ZPADDR					;STORE 9 IN THE LOW BYTE OF THE 16-BIT PAIR
	LDA #0
	STA (ZPADDR)				;STORE THE STATE 0 (DEAD) IN THE PLAYER MISSILE DATA ARRAY.	
	;BUT THAT'S NOT ALL, WE NEED TO ACTUALLY DISABLE THE SPRITE ITSELF, AS CHANGING THE STATE TO DEAD IN THE DATA ARRAY JUST STOPS IT FROM UPDATING, BUT THE SPRITE WILL STILL BE ON SCREEN.
	LDA #4
	STA VAR_SPRITESTATUS		;SET 4 IN THE SPRITE STATUS VARIABLE, TO TELL THE SETSPRITE SUB TO DISABLE THE SPRITE
	LDA #56						;LOAD 56 INTO A, AS A IS USED AS THE SPRITE NUMBER. POINTING IT TO THE SPRITE NUMBER FOR THE PLAYER MISSILE FOR THE SETSPRITE.
	JSR SETSPRITE				;NOW LOAD THE SPRITE, WHICH THANKS TO VAR_SPRITESTATUS BEING 4, IT WILL DISABLE THE SPRITE IN VERA.
	;AND WE'RE DONE, NO MORE COLLISIONS TO CHECK FOR THE PLAYER MISSILE (AS IT'S NOW DISABLED)
	
RANGECHECKING_MS_EN_DONEP:			;DONE RANGECHECKING PLAYER MISSILE..
	;TODO HERE, RANGE CHECKING ENEMY LASERS AND ENEMY SPRITES RANGE VS THE PLAYER'S SHIP...
	RTS
	
;---------------------------------------------
;FUNCTION - COLLISIONCHECK, THIS FUNCTION DOES INDIVIDUAL RANGE CHECKING BETWEEN TWO X AND Y POSITIONS. GO TO RANGECHECKING SUB FOR THE CHECKING OF ALL SPRITES AGAINST EACH OTHER.
; X POS1 = ZPADDR+8, Y POS 1 = ZPADDR+9. X POS 2 = ZPADDR+10, Y POS 2 = ZPADDR+11. SPRITE 1 TYPE = ZPADDR+12, SPRITE 2 TYPE = ZPADDR+13 (1=ENEMY, 2=PLAYER, 4=PLAYER MISSILE, 8=ENEMY LASER)
;---------------------------------------------
COLLISIONCHECK:
	;FIRST WE CALCULATE THE RANGE BETWEEN XY POS 1 AND XY POS 2, USING THE PYTHAGOREAN THEOREM.
	LDA ZPADDR+8
	STA VAR_X1					;COPY SPRITE 1 X POSITION TO VAR_X1
	LDA ZPADDR+9
	STA VAR_Y1					;COPY SPRITE 1 Y POSITION TO VAR_Y1
	LDA ZPADDR+10
	STA VAR_X2					;COPY SPRITE 2 X POSITION TO VAR_X2
	LDA ZPADDR+11
	STA VAR_Y2					;COPY SPRITE 2 Y POSITION TO VAR_Y2
	JSR CALC_RANGE				;CALCULATE THE RANGE BETWEEN THE TWO XY POSITIONS USING THE PYTHAGOREAN THEOREM. A WILL CONTAIN THE RETURNED RANGE AFTERWARDS
	CMP #255					;WE CHECK IF THE RETURNED VALUE FROM CALC_RANGE IS 255, MEANING THE VALUE WAS TOO LARGE TO CALCULATE THE RANGE FOR...
	BEQ COLLISIONCHECK_NOCOL	;BRANCH TO COLLISIONCHECK_NOCOL TO RETURN THE COLLISION FAILED
	STA ZPADDR+15				;OTHERWISE WE STORE THE RANGE BETWEEN THE TWO XY POSITIONS TO ZPADDR+15 FOR NOW.
	;---------------------------------------------
	;OTHERWISE WE CAN CONTINUE CHECKING, WE NEED TO DO THE MATH ON WHAT THE MAX RANGE WILL BE.
	;FIRST, WE NEED TO CHECK WHAT KIND OF SPRITE TYPE SPRITE 1 IS.
	LDA ZPADDR+12				;LOAD THE SPRITE TYPE OF THE FIRST SPRITE
	CMP #1						;CHECK IF IT'S A ENEMY (TYPE=1)
	BEQ COLLISIONCHECK_1ENEMY	;IF SO, THEN BRANCH TO ADD THE OFFSETS FOR A ENEMY SPRITE
	CMP #2						;IF IT'S NOT, THEN CHECK IF IT'S THE PLAYER (SPRITE TYPE = 2)
	BEQ COLLISIONCHECK_1PLAYER	;IF IT IS, THEN BRANCH TO ADD THE OFFSET FOR THE PLAYER SPRITE
	CMP #4						;IF IT'S NOT, THEN CHECK IF IT'S A PLAYER MISSILE (SPRITE TYPE = 4)
	BEQ COLLISIONCHECK_1MISSILE	;IF IT IS, THEN BRANCH TO ADD THE OFFSET FOR THE PLAYER MISSILE SPRITE
	;OTHERWISE IT'S A ENEMY LASER (SPRITE TYPE = 8)
	LDA #2						;LOAD 2 INTO A, SO WE ONLY ADD A EXTRA RANGE OF 2 PIXELS FOR THE ENEMY LASER
	STA ZPADDR+14				;ADD THE RANGE TO ZPADDR+14
	BRA COLLISIONCHECK_2		;BRANCH TO COLLISIONCHECK_2 TO CHECK THE SECOND SPRITE SPRITE
COLLISIONCHECK_1ENEMY:			;IF SPRITE 1 IS A ENEMY SPRITE
	LDA #6						;LOAD 6 INTO A, WHICH IS THE RANGE OF THE CENTER OF THE ENEMY TO ITS OUTER VISIBLE PIXELS
	STA ZPADDR+14				;ADD THE RANGE TO ZPADDR+14
	BRA COLLISIONCHECK_2		;BRANCH TO COLLISIONCHECK_2 TO CHECK THE SECOND SPRITE SPRITE
COLLISIONCHECK_1PLAYER:			;IF SPRITE 1 IS A PLAYER SPRITE
	LDA #6						;LOAD 6 INTO A, WHICH IS THE RANGE OF THE CENTER OF THE PLAYER SPRITE TO ITS OUTER VISIBLE PIXELS
	STA ZPADDR+14				;ADD THE RANGE TO ZPADDR+14
	BRA COLLISIONCHECK_2		;BRANCH TO COLLISIONCHECK_2 TO CHECK THE SECOND SPRITE SPRITE
COLLISIONCHECK_1MISSILE:		;IF SPRITE 1 IS A PLAYER MISSILE SPRITE
	LDA #1						;LOAD 1 INTO A, AS THE MAXIMUM RANGE FROM THE TOP TIP OF THE MISSILE.
	STA ZPADDR+14				;ADD THE RANGE TO ZPADDR+14
	;---------------------------------------------
	;SO WE NEED TO CHECK WHAT KIND OF SPRITE TYPE SPRITE 2 IS AS WELL AND DO THE SAME!
COLLISIONCHECK_2:				;DO THE SAME FOR SPRITE 2
	LDA ZPADDR+13				;LOAD THE SPRITE TYPE OF THE FIRST SPRITE
	CMP #1						;CHECK IF IT'S A ENEMY (TYPE=1)
	BEQ COLLISIONCHECK_2ENEMY	;IF SO, THEN BRANCH TO ADD THE OFFSETS FOR A ENEMY SPRITE
	CMP #2						;IF IT'S NOT, THEN CHECK IF IT'S THE PLAYER (SPRITE TYPE = 2)
	BEQ COLLISIONCHECK_2PLAYER	;IF IT IS, THEN BRANCH TO ADD THE OFFSET FOR THE PLAYER SPRITE
	CMP #4						;IF IT'S NOT, THEN CHECK IF IT'S A PLAYER MISSILE (SPRITE TYPE = 4)
	BEQ COLLISIONCHECK_2MISSILE	;IF IT IS, THEN BRANCH TO ADD THE OFFSET FOR THE PLAYER MISSILE SPRITE
	;OTHERWISE IT'S A ENEMY LASER (SPRITE TYPE = 8)
	LDA ZPADDR+14				;LOAD THE MAXIMUM RANGE FROM THE FIRST SPRITE CHECK.
	CLC
	ADC #2						;ADD 2 INTO A, WHICH IS THE RANGE OF THE BOTTOM BIT OF THE ENEMY LASER SPRITE'S COLLISION BUBBLE
	BRA COLLISIONCHECK_CHECK	;BRANCH TO COLLISIONCHECK_CHECK TO CHECK THE SECOND SPRITE SPRITE
COLLISIONCHECK_2ENEMY:			;IF SPRITE 2 IS A ENEMY SPRITE
	LDA ZPADDR+14				;LOAD THE MAXIMUM RANGE FROM THE FIRST SPRITE CHECK.
	CLC
	ADC #6						;LOAD 6 INTO A, WHICH IS THE RANGE OF THE CENTER OF THE ENEMY TO ITS OUTER VISIBLE PIXELS
	BRA COLLISIONCHECK_CHECK	;BRANCH TO COLLISIONCHECK_CHECK TO CHECK THE SECOND SPRITE SPRITE
COLLISIONCHECK_2PLAYER:			;IF SPRITE 2 IS A PLAYER SPRITE
	LDA ZPADDR+14				;LOAD THE MAXIMUM RANGE FROM THE FIRST SPRITE CHECK.
	CLC
	ADC #6						;LOAD 6 INTO A, WHICH IS THE RANGE OF THE CENTER OF THE PLAYER SPRITE TO ITS OUTER VISIBLE PIXELS
	BRA COLLISIONCHECK_CHECK	;BRANCH TO COLLISIONCHECK_CHECK TO CHECK THE SECOND SPRITE SPRITE
COLLISIONCHECK_2MISSILE:		;IF SPRITE 2 IS A PLAYER MISSILE SPRITE
	LDA ZPADDR+14				;LOAD THE MAXIMUM RANGE FROM THE FIRST SPRITE CHECK.
	CLC
	ADC #1						;ADD 1 INTO A, WHICH IS THE MAXIMUM DETECTION BUBBLE AROUND THE MISSILE'S TIP (AT THE TOP)

COLLISIONCHECK_CHECK:	
	;---------------------------------------------
	;NOW THAT WE KNOW THE MAXIMUM RANGE, AND THE RANGE, LETS CHECK IF WE HAVE A COLLISION!
	CLC
	ADC #1						;ADD 1 TO THE MAXIMUM RANGE THAT'S IN A, SO WE CAN DO A COMARE AND THEN BRANCH IF LESS THEN...
	STA ZPADDR+14				;STORE THE MAXIMUM ALLOWED RANGE+1 IN ZPADDR+14 AGAIN.
	
	LDA ZPADDR+15				;LOAD THE RANGE BETWEEN THE TWO XY POSITIONS INTO A
	CMP ZPADDR+14				;COMPARE THE RANGE TO THE MAXIMUM ALLOWED RANGE+1
	BCC	COLLISIONCHECK_COL		;IF IT'S LESS THEN THE MAXIMUM ALLOWED RANGE, WE HAVE A COLLISION, BRANCH TO COLLISIONCHECK_COL TO SET THE VALUES TO LET THE PREVIOUS SUB KNOW.
	BCS COLLISIONCHECK_NOCOL	;OTHERWISE WE HAVE NO COLLISION... 
	
COLLISIONCHECK_COL:				;WE HAVE A COLLISION!!
	LDA #1
	BRA COLLISIONCHECK_DONE		;AND WE'RE DONE, RETURN TO PREVIOUS SUB
	
COLLISIONCHECK_NOCOL:			;COLLISION CHECK FAILED, OR NO COLLISION
	LDA #0						;SET 0 TO A, TO LET THE PREVIOUS SUB KNOW THE COLLISION CHECK RETURNED NO COLLISION
	
COLLISIONCHECK_DONE:
	RTS
	
;---------------------------------------------
;FUNCTION - CALC_RANGE, THIS FUNCTION CALCULATES THE RANGE BETWEEN TWO XY COORDINATES, USING THE PYTHAGOREAN THEOREM.
; USES VAR_X1 AND VAR_Y1 FOR THE FIRST XY COORDINATE, VAR_X2 AND VAR_Y2 FOR THE SECOND SET OF COORDINATES. RETURNS RANGE IN A
;---------------------------------------------
CALC_RANGE:
	;---------------------------------------------
	;FIRST WE NEED TO DETERMINE THE X DIFFERENCE BETWEEN X1 AN X2,
	LDA VAR_X1					;LOAD THE FIRST X POSITION
	CMP VAR_X2					;COMPARE X1 TO X2
	BCS	CALC_RANGE_X2			;IF X1 IS AS HIGH OR HIGHER THEN X2 THEN WE CAN SUBTRACT X2 FROM X1
	LDA VAR_X2
	SEC
	SBC VAR_X1					;SUBTRACT X1 FROM X2
	BRA CALC_RANGE_X_DONE		;DONE CALCULATING X DIFFERENCE
CALC_RANGE_X2:					;IF X1 IS BIGGER OR AS BIG, THEN SUBTRACT X2 FROM X1
	SEC
	SBC VAR_X2					;SUBTRACT X2 FROM X1
CALC_RANGE_X_DONE:
	STA VAR_X1					;STORE THE X DIFFERENCE IN VAR_X1 FOR NOW
	
	;NOW WE NEED TO CALCULATE THE Y DIFFERENCE
	LDA VAR_Y1					;LOAD THE FIRST Y POSITION
	CMP VAR_Y2					;COMPARE Y1 TO Y2
	BCS	CALC_RANGE_Y2			;IF Y1 IS AS HIGH OR HIGHER THEN Y2 THEN WE CAN SUBTRACT Y2 FROM Y1
	LDA VAR_Y2
	SEC
	SBC VAR_Y1					;SUBTRACT Y1 FROM Y2
	BRA CALC_RANGE_Y_DONE		;DONE CALCULATING Y DIFFERENCE
CALC_RANGE_Y2:					;IF Y2 IS BIGGER OR AS BIG, THEN SUBTRACT Y1 FROM Y2
	SEC
	SBC VAR_Y2					;SUBTRACT Y2 FROM Y1
CALC_RANGE_Y_DONE:
	STA VAR_Y1					;STORE THE Y DIFFERENCE IN VAR_Y1 FOR NOW
	
	;---------------------------------------------
	;NOW WE NEED TO APPLY THE PYTHAGOREAN THEOREM. HOWEVER, SINCE WE'RE WORKING WITH AN 8-BIT SYSTEM, THERE'S NO POINT IN SQUARING A VALUE MORE THEN 15. AS THE RESULT WILL BE 256 OR MORE, WHICH WE CAN'T HANDLE
	;HOWEVER, IT'S WORSE, IF WE ADD THE TWO 15^2'S TOGETHER, WE STILL OVERFLOW... SO THE MAX WE CAN HANDLE IN THIS SUB IS 11...
	;SO WE CHECK IF THE X AND Y DIFF IS LESS THEN 12 FIRST BEFORE WE DO SO, IF ANY OF THEM IS HIGHER, WE DON'T NEED TO CONTINUE WITH RANGECHECKING, AS THERE'S NO POINT
	CMP #12						;COMPARE THE VALUE IN A (WHICH IS STILL THE Y DIFFERENCE) WITH 12
	BCS	CALC_RANGE_FAIL			;IF THE Y DIFF IS 12 OR MORE, THEN THERE'S NO POINT TO CONTINUE, WE'RE ENDING AND FAILING THE RANGE CHECK.
	LDA VAR_X1					;IF Y DIFF IS LESS THEN 12, THEN LOAD THE X DIFFERENCE AND CHECK IT TOO
	CMP #12						;COMPARE THE X DIFFERENCE WITH 12
	BCS	CALC_RANGE_FAIL			;IF THE X DIFF IS 12 OR MORE, THEN THERE'S NO POINT TO CONTINUE, WE'RE ENDING AND FAILING THE RANGE CHECK.
	;---------------------------------------------
	;IF WE GET HERE, THE X AND Y DIFFERENCE IS LESS THEN 12 SO WE CAN CONTINUE SQUARING
	JSR SQUARE					;SQUARE THE VALUE IN A (WHICH IS THE X DIFFERENCE AT THIS POINT)
	STA VAR_X1					;STORE THE SQUARED VALUE (IN A) IN VAR_X1
	LDA VAR_Y1					;LOAD THE Y DIFFERENCE
	JSR SQUARE					;SQUARE THE VALUE IN A (WHICH IS THE Y DIFFERENCE)
	CLC
	ADC VAR_X1					;ADD THE QUARED X DIFFERENCE FROM VAR_X1 TO THE SQUARED Y DIFFERENCE THAT'S IN A
	JSR SQUAREROOT				;CALCULATE THE SQUARE ROOT OF THE VALUE IN A (WHICH CONTAINS (XDIFF^2 + YDIFF^2)
	;WE NOW HAVE OUR RANGE IN A, RETURN TO THE PREVIOUS SUBROUTINE!
	BRA	CALC_RANGE_END

CALC_RANGE_FAIL:				;RANGE CHECKING FAILED, THE NUMBERS WERE TO HIGH, RETURN A RANGE OF 255 TO LET THE PREVIOUS SUB KNOW IT'S FAILED
	LDA #255		
CALC_RANGE_END:
	RTS
	
;---------------------------------------------
;FUNCTION - SQUARE, TAKES THE VALUE IN A AND CALCULATE'S THE SQUARE OF THE NUMBER, RETURNS THE VALUE IN A
;---------------------------------------------
SQUARE:
	CMP #0						;CHECK IF THE VALUE TO SQUARE IS 0. IF IT IS, WE'RE NOT GOING TO DO ANYTHING!
	BEQ SQUARE_DONE
	STA VAR_MATH				;OTHERWISE, STORE THE VALUE TO SQUARE TO VAR_MATH FOR NOW.
	LDX #1						;LOAD X WITH 1, IT WILL BE LOOP COUNTER
SQUARE_LOOP:
	CPX VAR_MATH				;CHECK IF WE HAVE ALREADY LOOPED FOR THE AMOUNT TO BE SQUARED.
	BEQ SQUARE_DONE				;IF WE'VE DONE ENOUGH ADDITIONS, THEN END THE LOOP, SQUARING IS DONE!
	CLC
	ADC VAR_MATH				;ADD THE NUMBER TO SQUARE TO THE SUM
	INX
	BRA SQUARE_LOOP				;LOOP AGAIN.
	
SQUARE_DONE:					;DONE SQUARING, THE RESULT IS IN A
	RTS
	
;---------------------------------------------
;FUNCTION - SQUAREROOT, TAKES THE VALUE IN A AND CALCULATES THE SQUARE ROOT, NO REMAINDER, JUST THE WHOLE NUMBERS (SO ROUNDED DOWN)
;---------------------------------------------
SQUAREROOT:
	CMP #2						;CHECK IF THE VALUE TO CALC THE SQUARE ROOT OF IS 2
	BCC SQUAREROOT_END			;IF IT'S LESS THEN 2 (SO 0 OR 1), WE NEED TO STOP THE SUB, NOTHING TO DO...
	STA VAR_MATH				;STORE THE VALUE TO SQUARE TO VAR_MATH.
	LDX #1						;STORE 1 IN X, IT WILL BE THE LOOP COUNTER
	LDA #1
	STA VAR_MATH2				;STORE 1 TO VAR_MATH2, WE'LL USE IT TO CONTAIN THE (ODD) VALUE TO ADD
	LDA #1						;LOAD 1 INTO A, THE VALUE TO START ADDING TO
SQUAREROOT_LOOP:
	CMP VAR_MATH				;IF THE VALUE IN A (ADDED VALUES) IS EQUAL OR MORE THEN VAR_MATH, WE'RE DONE! SQUARE ROOT FOUND.
	BCS SQUAREROOT_DONE			;IF THE VALUE IS EQUAL OR MORE THEN VAR_MATH, WE'RE DONE! SQUARE ROOT FOUND.
	CLC
	ADC VAR_MATH2				;OTHERWISE WE ADD THE VALUE IN VAR_MATH2 TO A
	STA VAR_MATH3				;STORE THE ADDED VALUE TO VAR_MATH3 FOR NOW.
	;NOW WE INCREASE THE ODD NUMBER FOR THE NEXT LOOP
	LDA VAR_MATH2				;LOAD THE ODD VALUE TO SUBTRACT FROM VAR_MATH2
	CLC
	ADC #2						;ADD TWO TO THE ODD NUMBER 
	STA VAR_MATH2				;STORE THE ODD VALUE BACK TO VAR_MATH2
	INX							;INCREASE LOOP COUNTER
	LDA VAR_MATH3				;RELOAD THE ADDED VALUE TO A FOR THE NEXT LOOP
	BRA SQUAREROOT_LOOP			;LOOP AGAIN!
SQUAREROOT_DONE:				;DONE CALCULATING SQUARE ROOT, THE RESULT IS IN X (ROUNDED DOWN)
	DEX							;DECREMENT X BY ONE, THE RESULT IS ONE TOO FAR.
	TXA							;TRANSFER RESULT BACK TO A
	
SQUAREROOT_END:
	RTS
	
;---------------------------------------------
;FUNCTION - HANDLEDEATH, THIS FUNCTION HANDLES THE DEATH OF THE PLAYER, EIGHTER BY COLLIDING WITH A ENEMY AND DIE.
;---------------------------------------------
HANDLEDEATH:
	;FIRST WE'LL CHECK IF THE PLAYER IS DEAD
	;---------------------------------------------
	LDA #SPRITEARRAY3H
	STA ZPADDR+1				;LOAD THE HIGH BYTE OF SPRITE DATA ARRAY 3 IN THE HIGH BYTE (ZPADDR+1)
	LDA #3
	STA ZPADDR					;LOAD 3 IN THE LOW BYTE, POINTING TO THE PLAYER'S SPRITE STATE
	LDA (ZPADDR)				;LOAD THE PLAYER SPRITE'S STATE INTO A
	CMP #0						;CHECK IF THE STATE IS DEAD/INACTIVE
	BNE HANDLEDEATH_CHECKENEMY	;IF THE PLAYER IS NOT DEAD, THEN GO TO CHECK IF THE ENEMIES ARE AT THE BOTTOM ALREADY
	;IF THE PLAYER IS DEAD. WE NEED TO REDUCE THE LIVES, OR END THE GAME...
	JMP REDUCELIVES				;REDUCE LIVES IF POSSIBLE AND TRY LEVEL AGAIN
	
HANDLEDEATH_CHECKENEMY:	
	;HERE WE NEED TO CHECK IF ANY ACTIVE ENEMY IN SPRITE DATA ARRAY 1 OR 2 IS AT Y=220, 10 PIXELS BELOW THE PLAYER.
	;IF SO, THE PLAYER FAILED AND LOOSES A LIFE ASWEL.
	;---------------------------------------------
	;RUN THROUGH SPRITE DATA ARRAY 1 TO CHECK IF THE SPRITES ARE ACTIVE (0-34)
	LDA #SPRITEARRAY1H
	STA ZPADDR+8				;STORE THE HIGHT BYTE IN ZPADDR+8
	STZ ZPADDR+7				;STORE 0 IN ZPADDR+7, IT WILL BE THE LOW BYTE
	LDY #3						;SET Y TO 3, IT'LL BE THE OFFSET, POINTING TO BYTE 4 (SPRITE STATE) OF THE SPRITE DATA ARRAY
	LDX #0						;SET X TO ZERO, IT'LL BE THE LOOP COUNTER/SPRITE NUMBER
	
HANDLEDEATH_CHECKENEMY_L1:
	LDA (ZPADDR+7),Y			;LOAD BYTE 4 (SPRITE STATE), OF THE SPRITE DATA ARRAY
	CMP #0						;CHECK IF THE RESULT IS 0 (AKA THE SPRITE IS DEAD/INACTIVE)
	BEQ HANDLEDEATH_CHECKENEMY_L1_NEXT		;GET READY TO LOAD THE NEXT SPRITE'S DATA...
	
	;IF THE SPRITE IS ACTIVE
	DEY
	DEY							;DECREMENT Y TWICE, SO IT POINTS TO THE SPRITE'S Y POSITION
	LDA (ZPADDR+7),Y			;LOAD THE Y POSITION OF THE ENEMY SPRITE
	CMP #220					;CHECK IF IT'S AT Y=220,
	BNE HANDLEDEATH_CHECKENEMY_L1_NEXT_1	;IF NOT, THEN LETS CHECK THE NEXT SPRITE
	;IF IT IS AT Y=220, THEN THE PLAYER LOST. 
	JMP REDUCELIVES				;WE LOST A LIVES

HANDLEDEATH_CHECKENEMY_L1_NEXT_1:
	INY
	INY							;INCREMENT Y TWICE AGAIN, SO IT POINTS BACK TO THE SPRITE STATE.
	
HANDLEDEATH_CHECKENEMY_L1_NEXT:	;CHANGE VARIABLES TO LOAD THE NEXT SPRITE'S DATA.
	INX							;INCREMENT X, INCREASING THE LOOP COUNTER/SPRITE NUMBER
	
	CPX #35						;CHECK IF WE'VE ALREADY DONE 34 LOOPS (AND ARE ABOUT TO DO THE 35TH).
	BEQ HANDLEDEATH_CHECKENEMY_L1_DONE	;IF WE HAVE, THEN WE'RE DONE CHECKING, EXIT THE LOOP
	
	TYA							;TRANSFER Y TO A
	CLC
	ADC #6						;ADD 6 TO A, WE'RE GOING TO OFFSET Y BY 6
	TAY							;TRANSFER A BACK TO Y
	BRA HANDLEDEATH_CHECKENEMY_L1	;ALWAYS BRANCH TO HANDLEDEATH_CHECKENEMY_L1, TO LOAD THE NEXT SPRITE'S DATA

HANDLEDEATH_CHECKENEMY_L1_DONE:

	;---------------------------------------------
	;RUN THROUGH SPRITE DATA ARRAY 2 TO CHECK IF THE SPRITES ARE ACTIVE (35-54)
	LDA #SPRITEARRAY2H
	STA ZPADDR+8				;STORE THE HIGH BYTE IN ZPADDR+8
	STZ ZPADDR+7				;STORE 0 IN ZPADDR+7, IT WILL BE THE LOW BYTE
	LDY #3						;SET Y TO 3, IT'LL BE THE OFFSET, POINTING TO BYTE 4 (SPRITE STATE) OF THE SPRITE DATA ARRAY
	LDX #0						;SET X TO ZERO, IT'LL BE THE LOOP COUNTER/SPRITE NUMBER
	
HANDLEDEATH_CHECKENEMY_L2:
	LDA (ZPADDR+7),Y			;LOAD BYTE 4 (SPRITE STATE), OF THE SPRITE DATA ARRAY
	CMP #0						;CHECK IF THE RESULT IS 0 (AKA THE SPRITE IS DEAD/INACTIVE)
	BEQ HANDLEDEATH_CHECKENEMY_L2_NEXT		;GET READY TO LOAD THE NEXT SPRITE'S DATA...
	
	;IF THE SPRITE IS ACTIVE
	DEY
	DEY							;DECREMENT Y TWICE, SO IT POINTS TO THE SPRITE'S Y POSITION
	LDA (ZPADDR+7),Y			;LOAD THE Y POSITION OF THE ENEMY SPRITE
	CMP #220					;CHECK IF IT'S AT Y=220,
	BNE HANDLEDEATH_CHECKENEMY_L2_NEXT_1	;IF NOT, THEN LETS CHECK THE NEXT SPRITE
	;IF IT IS AT Y=220, THEN THE PLAYER LOST. 
	JMP REDUCELIVES				;WE LOST A LIVES

HANDLEDEATH_CHECKENEMY_L2_NEXT_1:
	INY
	INY							;INCREMENT Y TWICE AGAIN, SO IT POINTS BACK TO THE SPRITE STATE.
	
HANDLEDEATH_CHECKENEMY_L2_NEXT:	;CHANGE VARIABLES TO LOAD THE NEXT SPRITE'S DATA.
	INX							;INCREMENT X, INCREASING THE LOOP COUNTER/SPRITE NUMBER
	
	CPX #20						;CHECK IF WE'VE ALREADY DONE 20 LOOPS (AND ARE ABOUT TO DO THE 21TH).
	BEQ HANDLEDEATH_CHECKENEMY_L2_DONE	;IF WE HAVE, THEN WE'RE DONE CHECKING, EXIT THE LOOP
	
	TYA							;TRANSFER Y TO A
	CLC
	ADC #6						;ADD 6 TO A, WE'RE GOING TO OFFSET Y BY 6
	TAY							;TRANSFER A BACK TO Y
	BRA HANDLEDEATH_CHECKENEMY_L2	;ALWAYS BRANCH TO HANDLEDEATH_CHECKENEMY_L2, TO LOAD THE NEXT SPRITE'S DATA

HANDLEDEATH_CHECKENEMY_L2_DONE:
HANDLEDEATH_DONE:
	RTS
	
	
;---------------------------------------------
;FUNCTION - REDUCELIVES, THIS FUNCTION WILL REDUCE LIVES AND RESET THE GAME TO TRY AGAIN IF POSSIBLE.
;---------------------------------------------
REDUCELIVES:
	LDA VAR_LIVES				;LOAD THE AMOUNT OF LIVES WE HAVE LEFT
	CMP #0						;CHECK IF WE'RE ALREADY DOWN TO 0 LIVES
	BEQ GAMEOVER				;IF WE HAD NO LIVES LEFT, THEN IT'S GAME OVER
	;OTHERWISE WE DO HAVE LIVES LEFT, SUBTRACT ONE, AND LETS LOAD THE LEVEL AGAIN
	SEC 
	SBC #1						;SUBTRACT 1 FROM THE NUMBER OF LIVES LEFT
	STA VAR_LIVES				;STORE THE NEW AMOUNT OF LIVES IN VAR_LIVES
	;NOW WE HAVE TO SHOW THE AMOUNT OF LIVES LEFT
	JSR INITVARS				;INITIALIZE GAME VARIABLES, RESET EVERYTHING ON THE SCREEN
	LDA #<TITLETEXT				
	STA z_L
	LDA #>TITLETEXT
	STA z_H
	JSR PrintString				;PRINT THE PRESS RETURN TEXT ON THE SCREEN
	JSR SHOWLIVES				;SHOW THE AMOUNT OF LIVES LEFT ON THE SCREEN (AND WAIT FOR A KEYPRESS)
	
	;NOW THAT A KEYPRESS HAS BEEN GIVEN,
	LDA #<TITLECLEAR				
	STA z_L
	LDA #>TITLECLEAR	
	STA z_H
	JSR PrintString				;CLEAR THE PRESS RETURN TEXT FROM THE SCREEN
	JMP GAMEPREP				;JUMP ALL THE WAY BACK TO THE GAME PREP LABEL! TO RUN THE LEVEL AGAIN
	
;---------------------------------------------
;FUNCTION - GAMEOVER, THIS FUNCTION WILL ASK THE PLAYER TO PLAY AGAIN, RESET LIVES, LEVEL AND GO AGAIN
;---------------------------------------------	
GAMEOVER:
	JSR INITVARS				;INITIALIZE GAME VARIABLES, RESET EVERYTHING ON THE SCREEN
	LDA #<GAMEOVERTEXT				
	STA z_L
	LDA #>GAMEOVERTEXT
	STA z_H
	JSR PrintString				;PRINT THE GAME OVER TEXT ON SCREEN
	
GAMEOVER_WAIT:
	;CHECK IF THE N OR Y KEYS ARE PRESSED, AND WAIT UNTIL ONE OF THEM IS
	JSR GETIN					;READ KEYBOARD USING A KERNAL CALL; BE AWARE! THIS WILL RESET X
	CMP #$59					;CHECK IF THE Y KEY HAS BEEN PRESSED
	BEQ GAMEOVER_NEWGAME		;IF Y KEY IS PRESSED, THEN WE START A NEW GAME!
	CMP #$4E					;CHECK IF THE N KEY HAS BEEN PRESSED
	BEQ GAMEOVER_END		
	;IF NOT THE Y OR N KEY, LOOP BACK TO CHECK IF THE KEYS GET PRESSED
	BNE GAMEOVER_WAIT			;LOOP BACK TO CHECK KEYBOARD IF RETURN HASN'T BEEN PRESSED	
	
GAMEOVER_NEWGAME:				;START A NEW GAMELOOP
	LDA #<GAMEOVERCLEAR				
	STA z_L
	LDA #>GAMEOVERCLEAR
	STA z_H
	JSR PrintString				;CLEAR THE PRESS RETURN TEXT ON THE SCREEN
	
	LDA #2						
	STA VAR_LIVES				;STORE THE DEFAULT NUMBER OF LIVES IN VAR_LIVES AGAIN (3 LIVES IN TOTAL)
	STZ VAR_LEVEL				;RESET THE LEVEL NUMBER T0 0, STARTING OF AT LEVEL 0
	JSR SHOWLIVES				;SHOW THE AMOUNT OF LIVES LEFT ON SCREEN, AND WAIT FOR USER INPUT
	;NOW THAT USER INPUT HAS BEEN GIVEN, START THE GAME
	JMP GAMEPREP				;JUMP BACK TO DO GAMEPREPPING TO START PLAYING AGAIN!

GAMEOVER_END:					;GAME OVER, AND WE'RE NOT PLAYING AGAIN!
	JMP *						;INF LOOP THE GAME, CRASHING IT.
	
;---------------------------------------------
;FUNCTION - SHOWLIVES, THIS FUNCTIONS WILL SHOW THE AMOUNT OF LIVES LEFT ON THE SCREEN, ALONG WITH SOME TEXT
;---------------------------------------------	
SHOWLIVES:
	LDA #<LEVELTEXT				
	STA z_L
	LDA #>LEVELTEXT
	STA z_H
	JSR PrintString				;PRINT THE LEVEL TEXT ON SCREEN
	
	LDA #11
	STA CURSORY					;SET THE CURSOR'S Y POSITION TO 11
	LDA #21
	STA CURSORX					;SET THE CURSOR'S X POSITION TO 23
	LDA VAR_LEVEL				;LOAD THE CURRENT LEVEL
	CLC
	ADC #1						;ADD ONE TO THE LEVEL, SO WE START OF WITH LEVEL 1 INSTEAD OF LEVEL 0
	JSR printhex				;PRINT THE CURRENT LEVEL ON SCREEN

	LDA #<LIVESTEXT				
	STA z_L
	LDA #>LIVESTEXT
	STA z_H
	JSR PrintString				;PRINT THE LIVES LEFT TEXT ON SCREEN
	
	;NOW WE NEED TO SHOW SPACESHIPS ON SCREEN, UP TO THREE 
	LDA VAR_LIVES				;GET THE AMOUNT OF LIVES WE HAVE LEFT
	CMP #2						;CHECK IF WE STILL HAVE ALL THREE LIVES
	BNE SHOWLIVES_2LIVES		;IF NOT, CHECK IF WE HAVE 2 LIVES
	JSR SHOWLIVES_SHOWLIFE1		;SHOW LIFE 1 ON SCREEN
	JSR SHOWLIVES_SHOWLIFE2		;SHOW LIFE 2 ON SCREEN
	JSR SHOWLIVES_SHOWLIFE3		;SHOW LIFE 3 ON SCREEN
	BRA SHOWLIVES_WAIT
SHOWLIVES_2LIVES:
	CMP #1						;CHECK IF WE HAVE 2 LIVES LEFT
	BNE SHOWLIVES_1LIVES		;IF NOT, ONLY PRINT 1 LIFE
	JSR SHOWLIVES_SHOWLIFE1		;SHOW LIFE 1 ON SCREEN
	JSR SHOWLIVES_SHOWLIFE2		;SHOW LIFE 2 ON SCREEN
	BRA SHOWLIVES_WAIT
SHOWLIVES_1LIVES:
	;NO NEED TO CHECK, CANT BE LESS THEN 1 LIFE :)
	JSR SHOWLIVES_SHOWLIFE1		;SHOW LIFE 1 ON SCREEN
	BRA SHOWLIVES_WAIT
	
SHOWLIVES_SHOWLIFE1:			;PRINT LIFE 1 (THE LEFT MOST LIFE)
	LDA #100
	STA Z_IXL					;STORE THE X POSITION TO Z_IXL FOR THE SPRITE PRINTING.
	LDA #130
	STA Z_IYL					;STORE THE Y POSITION TO Z_IXL FOR THE SPRITE PRINTING.
	LDA #4
	STA Z_L						;STORE THE VERA VRAM ADDRESS TO LOAD TO Z_L (LOW BYTE OF Z_HL)
	LDA #$08
	STA Z_H						;LOAD $08 AND STORE IT TO THE HIGH BYTE OF Z_HL
	LDA #62						;LOAD 62 IN A 1, FOR HARDWARE SPRITE NUMBER
	PHA
		PHX
			PHY
				JSR SETSPRITE	;ACTUALLY DRAW THE SPRITE ON SCREEN.
			PLY					;NOW RETURN THE Y, X AND A REGISTER BACKUPS FROM THE STACK
		PLX
	PLA
	RTS
SHOWLIVES_SHOWLIFE2:			;PRINT LIFE 2 (THE MIDDLE LIFE)
	LDA #120
	STA Z_IXL					;STORE THE X POSITION TO Z_IXL FOR THE SPRITE PRINTING.
	LDA #130
	STA Z_IYL					;STORE THE Y POSITION TO Z_IXL FOR THE SPRITE PRINTING.
	LDA #4
	STA Z_L						;STORE THE VERA VRAM ADDRESS TO LOAD TO Z_L (LOW BYTE OF Z_HL)
	LDA #$08
	STA Z_H						;LOAD $08 AND STORE IT TO THE HIGH BYTE OF Z_HL
	LDA #63						;LOAD 62 IN A 1, FOR HARDWARE SPRITE NUMBER
	PHA
		PHX
			PHY
				JSR SETSPRITE	;ACTUALLY DRAW THE SPRITE ON SCREEN.
			PLY					;NOW RETURN THE Y, X AND A REGISTER BACKUPS FROM THE STACK
		PLX
	PLA
	RTS
SHOWLIVES_SHOWLIFE3:			;PRINT LIFE 3 (THE RIGHT MOST LIFE)
	LDA #140
	STA Z_IXL					;STORE THE X POSITION TO Z_IXL FOR THE SPRITE PRINTING.
	LDA #130
	STA Z_IYL					;STORE THE Y POSITION TO Z_IXL FOR THE SPRITE PRINTING.
	LDA #4
	STA Z_L						;STORE THE VERA VRAM ADDRESS TO LOAD TO Z_L (LOW BYTE OF Z_HL)
	LDA #$08
	STA Z_H						;LOAD $08 AND STORE IT TO THE HIGH BYTE OF Z_HL
	LDA #64						;LOAD 62 IN A 1, FOR HARDWARE SPRITE NUMBER
	PHA
		PHX
			PHY
				JSR SETSPRITE	;ACTUALLY DRAW THE SPRITE ON SCREEN.
			PLY					;NOW RETURN THE Y, X AND A REGISTER BACKUPS FROM THE STACK
		PLX
	PLA
	RTS	
	
SHOWLIVES_WAIT:
	JSR GETIN					;READ KEYBOARD USING A KERNAL CALL; BE AWARE! THIS WILL RESET X
	CMP #$0D					;CHECK IF THE RETURN KEY HAS BEEN PRESSED
	BNE SHOWLIVES_WAIT			;LOOP BACK TO CHECK KEYBOARD IF RETURN HASN'T BEEN PRESSED	
	;OTHERWISE A KEY HAS BEEN PRESSED, SO LETS TRY AGAIN WITH THE NUMBER OF LIVES LEFT
	LDA #<LIVESCLEAR				
	STA z_L
	LDA #>LIVESCLEAR
	STA z_H
	JSR PrintString				;CLEAR THE LIVES LEFT TEXT FROM THE SCREEN
	
	LDA #<LEVELCLEAR				
	STA z_L
	LDA #>LEVELCLEAR
	STA z_H
	JSR PrintString				;CLEAR THE LEVEL TEXT FROM THE SCREEN
	
	;AND WE NEED TO DISABLE SPRITE 62-64, SO THE LIVES LEFT WONT STAY ON SCREEN
	LDA #4
	STA VAR_SPRITESTATUS		;SET 4 IN THE SPRITE STATUS VARIABLE, TO TELL THE SETSPRITE SUB TO DISABLE THE SPRITE
	LDA #62						;LOAD 62 INTO A, AS A IS USED AS THE SPRITE NUMBER
	JSR SETSPRITE				;NOW LOAD THE SPRITE, WHICH THANKS TO VAR_SPRITESTATUS BEING 4, IT WILL DISABLE THE SPRITE IN VERA.
	
	LDA #4
	STA VAR_SPRITESTATUS		;SET 4 IN THE SPRITE STATUS VARIABLE, TO TELL THE SETSPRITE SUB TO DISABLE THE SPRITE
	LDA #63						;LOAD 62 INTO A, AS A IS USED AS THE SPRITE NUMBER
	JSR SETSPRITE				;NOW LOAD THE SPRITE, WHICH THANKS TO VAR_SPRITESTATUS BEING 4, IT WILL DISABLE THE SPRITE IN VERA.
	
	LDA #4
	STA VAR_SPRITESTATUS		;SET 4 IN THE SPRITE STATUS VARIABLE, TO TELL THE SETSPRITE SUB TO DISABLE THE SPRITE
	LDA #64						;LOAD 62 INTO A, AS A IS USED AS THE SPRITE NUMBER
	JSR SETSPRITE				;NOW LOAD THE SPRITE, WHICH THANKS TO VAR_SPRITESTATUS BEING 4, IT WILL DISABLE THE SPRITE IN VERA.
	
	RTS
	
;---------------------------------------------
;FUNCTION - CHECKENEMIESLEFT, CHECK HOW MANY ACTIVE ENEMIES ARE LEFT ON THE SCREEN
;---------------------------------------------	
CHECKENEMIESLEFT:
	STZ VAR_ENEMIES				;ZERO OUT THE NUMBER OF ENEMIES LEFT, WE'LL FILL THE LIST UP NEXT.
	
	;---------------------------------------------
	;RUN THROUGH SPRITE DATA ARRAY 1 TO CHECK IF THE SPRITES ARE ACTIVE (0-34)
	LDA #SPRITEARRAY1H
	STA ZPADDR+8				;STORE THE HIGHT BYTE IN ZPADDR+8
	STZ ZPADDR+7				;STORE 0 IN ZPADDR+7, IT WILL BE THE LOW BYTE
	LDY #3						;SET Y TO 3, IT'LL BE THE OFFSET, POINTING TO BYTE 4 (SPRITE STATE) OF THE SPRITE DATA ARRAY
	LDX #0						;SET X TO ZERO, IT'LL BE THE LOOP COUNTER/SPRITE NUMBER
	
CHECKENEMIESLEFT_L1:
	LDA (ZPADDR+7),Y			;LOAD BYTE 4 (SPRITE STATE), OF THE SPRITE DATA ARRAY
	CMP #0						;CHECK IF THE RESULT IS 0 (AKA THE SPRITE IS DEAD/INACTIVE)
	BEQ CHECKENEMIESLEFT_L1_NEXT	;GET READY TO LOAD THE NEXT SPRITE'S DATA...
	;OTHERWISE THE SPRITE IS ACTIVE, INCREMENT THE COUNTER AND LOAD THE NEXT SPRITE
	LDA VAR_ENEMIES
	CLC
	ADC #1						;INCREMENT THE VAR_ENEMIES BY ONE
	STA VAR_ENEMIES				;AND STORE IT AGAIN
	
CHECKENEMIESLEFT_L1_NEXT:		;CHANGE VARIABLES TO LOAD THE NEXT SPRITE'S DATA.
	INX							;INCREMENT X, INCREASING THE LOOP COUNTER/SPRITE NUMBER
	
	CPX #35						;CHECK IF WE'VE ALREADY DONE 34 LOOPS (AND ARE ABOUT TO DO THE 35TH).
	BEQ CHECKENEMIESLEFT_L1_DONE	;IF WE HAVE, THEN WE'RE DONE CHECKING, EXIT THE LOOP
	
	TYA							;TRANSFER Y TO A
	CLC
	ADC #6						;ADD 6 TO A, WE'RE GOING TO OFFSET Y BY 6
	TAY							;TRANSFER A BACK TO Y
	BRA CHECKENEMIESLEFT_L1		;ALWAYS BRANCH TO CHECKENEMIESLEFT_L1, TO LOAD THE NEXT SPRITE'S DATA

CHECKENEMIESLEFT_L1_DONE:
	;---------------------------------------------
	;RUN THROUGH SPRITE DATA ARRAY 2 TO CHECK IF THE SPRITES ARE ACTIVE (35-54)
	LDA #SPRITEARRAY2H
	STA ZPADDR+8				;STORE THE HIGHT BYTE IN ZPADDR+8
	STZ ZPADDR+7				;STORE 0 IN ZPADDR+7, IT WILL BE THE LOW BYTE
	LDY #3						;SET Y TO 3, IT'LL BE THE OFFSET, POINTING TO BYTE 4 (SPRITE STATE) OF THE SPRITE DATA ARRAY
	LDX #0						;SET X TO ZERO, IT'LL BE THE LOOP COUNTER/SPRITE NUMBER
	
CHECKENEMIESLEFT_L2:
	LDA (ZPADDR+7),Y			;LOAD BYTE 4 (SPRITE STATE), OF THE SPRITE DATA ARRAY
	CMP #0						;CHECK IF THE RESULT IS 0 (AKA THE SPRITE IS DEAD/INACTIVE)
	BEQ CHECKENEMIESLEFT_L2_NEXT	;GET READY TO LOAD THE NEXT SPRITE'S DATA...
	;OTHERWISE THE SPRITE IS ACTIVE, INCREMENT THE COUNTER AND LOAD THE NEXT SPRITE
	LDA VAR_ENEMIES
	CLC
	ADC #1						;INCREMENT THE VAR_ENEMIES BY ONE
	STA VAR_ENEMIES				;AND STORE IT AGAIN
	
CHECKENEMIESLEFT_L2_NEXT:		;CHANGE VARIABLES TO LOAD THE NEXT SPRITE'S DATA.
	INX							;INCREMENT X, INCREASING THE LOOP COUNTER/SPRITE NUMBER
	
	CPX #20						;CHECK IF WE'VE ALREADY DONE 19 LOOPS (AND ARE ABOUT TO DO THE 20TH).
	BEQ CHECKENEMIESLEFT_L2_DONE	;IF WE HAVE, THEN WE'RE DONE CHECKING, EXIT THE LOOP
	
	TYA							;TRANSFER Y TO A
	CLC
	ADC #6						;ADD 6 TO A, WE'RE GOING TO OFFSET Y BY 6
	TAY							;TRANSFER A BACK TO Y
	BRA CHECKENEMIESLEFT_L2		;ALWAYS BRANCH TO CHECKENEMIESLEFT_L2, TO LOAD THE NEXT SPRITE'S DATA

CHECKENEMIESLEFT_L2_DONE:
	;WE NOW HAVE THE AMOUNT OF ACTIVE ENEMIES LEFT IN VAR_ENEMIES
	RTS
	
;---------------------------------------------
;FUNCTION - CHECKLVLCOMPLETE, CHECK IF THE LEVEL IS COMPLETED, IF SO, WE NEED TO MOVE ON TO THE NEXT LEVEL
;---------------------------------------------		
CHECKLVLCOMPLETE:
	LDA VAR_ENEMIES				;LOAD THE AMOUNT OF ENEMIES LEFT.
	CMP #0						;CHECK IF THERE ARE NO ENEMIES LEFT!
	BEQ CHECKLVLCOMPLETE_NEXTLVL
	RTS							;IF THE NUMBER OF ENEMIES IS NOT ZERO, THEN RETURN TO THE MAIN LOOP

CHECKLVLCOMPLETE_NEXTLVL:		;LOAD UP THE NEXT LEVEL
	JSR INITVARS				;RESET ALL VARIABLES AND CLEAR OUT SPRITES.
	LDA VAR_LEVEL				;LOAD THE CURRENT LEVEL 
	CLC
	ADC #1						;ADD ONE TO THE CURRENT LEVEL
	STA VAR_LEVEL				;STORE THE NEW LEVEL TO PLAY IN VAR_LEVEL
	JSR SHOWLIVES				;SHOW THE AMOUNT OF LIVES LEFT ON SCREEN, AND WAIT FOR USER INPUT
	;NOW THAT USER INPUT HAS BEEN GIVEN, START THE GAME
	JMP GAMEPREP				;JUMP BACK TO DO GAMEPREPPING TO START PLAYING AGAIN!

;---------------------------------------------
;FUNCTION - RANDOM NUMBER GENERATOR, SETS THE RND SEED TO A NUMBER OF 1 TO MAX IN ACCUMULATOR
;RETURNS THE RANDOM NUMBER IN THE ACCUMULATOR
;---------------------------------------------
;NORMALLY SKIP OVER THIS FUNCTION
RAND
	PHX				
	PHY						;STORE X AND Y TO STACK AS BACKUP
		CLC					;CLEAR THE CARRY BEFORE ADD COMMAND BELOW
		ADC #1				;ADD 1 TO THE VALUE ACCUMULATOR (WE NEEDED THE CLC FOR THIS)
		STA SEED+1			;TEMP STORE MAX NUMBER WE WANT TO GENERATE IN SEED+1
		
RANDA
		JSR RERND 			;GENERATE A RANDOM NUMBER, RESULT IS IN THE ACCUMULATOR
		CMP SEED+1			;COMPARE THE RANDOM NUMBER TOO THE NUMBER WE WANTED (TO BE BELOW THE MAX MUMBER) IN SEED+1
		BCC RANDY			;IF IT'S AT OR BELOW THE NUMBER WE WANTED NOW, DONE!
		JMP RANDA			;IF IT'S MORE THEN THE NUMBER WE WANTED, FIND ANOTHER NUMBER
RANDY
	PLY
	PLX						;RESTORE X AND Y FROM BACKUP
	RTS

;---------------------------------------------
;FUNCTION - NUMBER RE-RANDOMIZER
;---------------------------------------------
RERND
	;SIMPLE 8-BIT SEED RE-RANDOMIZER (ACTUALLY A LINEAR FEEDBACK SHIFT REGISTER)
	;(FROM HERE: https://codebase64.org/doku.php?id=base:small_fast_8-bit_prng )
	LDA SEED			;LOAD THE SEED INTO ACCUMULATOR
    BEQ doEor
	ASL					;SHIFT ALL BITS TO THE LEFT, MULTIPLY BY 2
	BEQ NOEOR			
    BCC NOEOR
doEor:
	EOR #$1D 			;IF THE CARRY WAS SET, DO A EOR/XOR WITH VALUE $1D
NOEOR:	
	STA SEED			;RESTORE THE SEED, NUMBER IS NOW REGENERATED
	
	RTS

;---------------------------------------------
;FUNCTION - RANDOM NUMBER GENERATOR INITIALIZER
;---------------------------------------------
;NORMALLY SKIP OVER THIS FUNCTION
RNDINIT
	;WE'RE GOING TO INITIALLY SEED THE RND USING THE CLOCK (SECONDS)
	JSR CLOCKAPI		;RUN THE COMMANDER X16 CLOCK API CALL. 
	LDA CLOCKSEC 		;LOAD THE CURRENT CLOCK SECONDS IN A
	STA SEED			;STORE THE SECONDS AS THE SEED.
	
RNDINITY
	RTS
	
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
		
SetVeraHML:		;Select Memory address %----AAAA YYYYYYYYY XXXXXXXX
				;Set Inc to            %AAAA----
						
		sta V_H	
		sty V_M
		stx V_L
	rts


AddVHML_YX		;add 24 bit %00000000 YYYYYYYYY XXXXXXXX to V_HML
		lda #0
AddVHML_AYX		;add 24 bit %AAAAAAAA YYYYYYYYY XXXXXXXX to V_HML
		pha
			clc
			txa
			adc V_L
			sta V_L
			tya
			adc V_M
			sta V_M
		pla
		adc V_H
		sta V_H
		rts	

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
		
;THIS FUNCTION IS RESPONSIBLE FOR PRINTING CHARACTERS ON THE SCREEN	
PrintChar:
	SEC						
	SBC #32					;SUBTRACT 32 FROM THE CHARACTER VALUE TO PRINT. NOTE; OUR CHARACHTER HAS NO SPACE
	STA Z_AS				;STORE IN Z_AS (BACKUP REGISTER)
	
	PHX						
	PHY						;BACKUP THE REGSITERS WE USE, BOTH X AND Y
	LDA Z_H
	PHA
	LDA Z_L					
	PHA						;LOAD THE Z_HL PAIR AND STORE BOTH IN THE STACK AS BACKUPS.
		LDA Z_AS			;RELOAD THE CHARACTER VALUE TO PRINT (SUBTRACED BY 32); 8 bytes per character in our font
		STZ Z_H				;SET Z_H TO 0				
		ASL					;SHIFT THE BITS CHARACTER VALUE TO THE LEFT (MULTIPLY * 2)
		ROL	Z_H				;STORE THE BITS THAT HAVE FALLEN OF THE ACCUMULTOR IN THE SHIFT IN Z_H (AND SHIFT ALL THE BITS IN Z_H TO THE LEFT)
		ASL					;SHIFT THE BITS IN THE ACCUMULATOR TO THE LEFT AGAIN (MULTIPLY * 4) 
		ROL	Z_H				;STORE THE BITS THAT HAVE FALLEN OF THE ACCUMULTOR IN THE SHIFT IN Z_H (AND SHIFT ALL THE BITS IN Z_H TO THE LEFT)
		ASL 				;SHIFT THE BITS IN THE ACCUMULATOR TO THE LEFT AGAIN (MULTIPLY * 8) 
		ROL	Z_H				;STORE THE BITS THAT HAVE FALLEN OF THE ACCUMULTOR IN THE SHIFT IN Z_H (AND SHIFT ALL THE BITS IN Z_H TO THE LEFT)
		
		ADC #<BitmapFont	;ADD THE START RAM ADDRESS (LOW BYTE) OF THE BITMAPFONT TO THE ACCUMULATOR
		STA Z_L				;STORE THE ACCUMULATOR VALUE IN Z_L (FOR USE AS THE LOW BYTE)
		LDA Z_H				;NOW LOAD THE Z_H VALUE IN THE ACCUMULATOR
		ADC #>BitmapFont	;ADD THE START RAM ADDRESS (HIGH BYTE) OF THE BITMAPFONT TO THE ACCUMULATOR
		STA Z_H				;STORE THE ACCUMULATOR IN Z_H	(FOR USE AS THE HIGH BYTE)
		
		;AT THIS POINT Z_HL CONTAINS THE START ADDRESS OF THE BITMAPFONT, OFFSET BY THE CHARACTER THAT HAS TO BE LOADED.

		;NOW WE NEED TO SET THE VERA VRAM (SCREEN) ADDRESS TO PUT THE PIXELS IN. (24-BITS ADDRESS)
		;FOR THE 24 BITS WE'LL USE THE Z_D, Z_E AND A REGISTERS
		STZ Z_D				;FIRST EMPTY THE Z_D REGISTER. THIS WILL BE THE HIGH BYTE OF VERA VRAM
			
		LDA CursorY	 		;LOAD THE CURRENT CURSOR Y LOCATION TO A. CursorY* 160*8 = Cursor Y * %00000000 00000101 00000000
		STA Z_E				;STORE THE Y POSITION IN Z_E, WHICH WILL BE THE THE MIDDLE BYTE OF VERA VRAM
		
		ASL					;BITSHIFT A (CURSOR Y) TO THE LEFT (MULTIPLY * 2)
		ROL Z_D				;STORE THE BITS THAT HAVE FALLEN OF THE ACCUMULTOR IN THE SHIFT IN Z_D (AND SHIFT ALL THE BITS IN Z_D TO THE LEFT)
		ASL					;BITSHIFT A (CURSOR Y) TO THE LEFT (MULTIPLY * 4)
		ROL Z_D				;STORE THE BITS THAT HAVE FALLEN OF THE ACCUMULTOR IN THE SHIFT IN Z_D (AND SHIFT ALL THE BITS IN Z_D TO THE LEFT)
		
		CLC
		ADC Z_E				;ADD THE VALUE IN Z_E (CURSOR Y POSITION) TO THE ACCUMULATOR  
		
		TAY					;TRANSFER THE ACCUMULATOR VALUE TO Y
		
		LDA Z_D				;LOAD Z_D (THE CARRY BITS THAT HAVE FALLEN OFF BEFORE) IN THE ACCUMULATOR
		ADC #0				;ADD 0 TO THE ACCUMULAR;  Add carry to H byte
		STA Z_D				;STORE THE CARRY FROM THE BITSHIFTS TO Z_D (HIGH BYTE OF VERA VRAM ADDRESS)
		
		STZ Z_E				;EMPTY THE Z_E REGISTER
		LDA CursorX			;LOAD THE CURSOR X POSITION IN A; CursorX* 4 = Cursor Y * %00000000 00000000 00000100
		
		ASL					;BITSHIFT A (CURSOR X) TO THE LEFT (MULTIPLY * 2)
		ROL Z_E				;STORE THE BITS THAT HAVE FALLEN OF THE ACCUMULTOR IN THE SHIFT IN Z_E (AND SHIFT ALL THE BITS IN Z_E TO THE LEFT)
		ASL					;BITSHIFT A (CURSOR X) TO THE LEFT (MULTIPLY * 4)
		ROL Z_E				;STORE THE BITS THAT HAVE FALLEN OF THE ACCUMULTOR IN THE SHIFT IN Z_E (AND SHIFT ALL THE BITS IN Z_E TO THE LEFT)
		
		TAX					;TRANSFER A TO X ; X contains LOW byte 
		
		tya 				;Add carry to M byte
		adc z_e				
		tay					;Y contains M byte
		
		lda z_d				;Add carry to H byte & set AutoInc
		adc #$10			;A contains H byte  
		
		jsr SetVeraHML		;Set Screen Address for the location where the character has to be printed on screen.
		
		
		ldy #0				;Line in Font Char
CharLineAgain:
		phy
			lda (z_hl),y
			sta z_as
			ldx #4				;4 Bytes (2xpixels) Per Line; IN THE COMMANDS BELOW, TWO PIXELS WILL BE LOADED INTO A AT A TIME!
CharPixelAgain:				
			LDA #0				;EMPTY THE ACCUMULATOR
			ROL Z_AS			;Pop a bit off our font byte (12345678 -> 23456780 AND CARRY CONTAINS 1)
			ROL					;BITSHIFT A LEFT, INSERT THE BIT POPPED OF THE Z_AS ABOVE (00000000 -> 00000001)
			ROL					;BITSHIFT A LEFT AGAIN (00000001 -> 00000010)
			ROL					;BITSHIFT A LEFT AGAIN (00000010 -> 00000100)
			ROL					;BITSHIFT A LEFT AGAIN (00000100 -> 00001000)
			ROL Z_AS			;Pop a bit off our font byte (23456780 -> 34567800 AND CARRY CONTAINS 2)
			ROL					;BITSHIFT A LEFT AND INSERT THE BIT POPPED OF Z_AS ABOVE (00001000 -> 00010001)
	
			ASL					;BITSHIFT ALL THE BITS ONE TO THE LEFT, SO WE DON'T PRINT IN PALLET COLOR #1 BUT PALLET COLOR #2 (00010001 -> 00100010)
			
			STA V_D1			;SEND BITS (TWO PIXELS) TO SCREEN RAM
			
			dex 
			bne CharPixelAgain	;Next Pixel
			
			ldy #>(160-4)
			ldx #<(160-4)
			jsr AddVHML_YX			;Move down screen line (160 byte)
		ply
		iny							;Move down a line in our font
		tya
		cmp #8
		bne CharLineAgain			;8 lines per character
		
		inc CursorX
		lda CursorX
		cmp #40						;40 Characters per screen line
		bne NoNewLine
		jsr NewLine
NoNewLine:	
	pla								;Restore the registers
	sta z_l
	pla
	sta z_h	
	ply
	plx
	rts

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;FUNCTION TO COPY RAM MEMORY TO VERA VRAM.

VLDIR:					;Copy z_BC bytes from z_HL to V_D1 
						;(address must be already selected)
        lda (z_HL)
        sta V_D1		;Copy byte to V_D1
		INC z_L
		BNE	VLDIR_SkipInc1	;JUMP TO VLDIR_SkipInc1, AS LONG AS Z_L IS NOT 0
		INC	z_H
VLDIR_SkipInc1:
		DEC z_C			;Repeat until z_BC =0
		BNE VLDIR
		LDA z_B
		BEQ	VLDIR_Done
		DEC z_B
		jmp VLDIR
VLDIR_Done:
		rts
		
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;Use HardSprite A ... position to z_IX,z_IY... 
;				Show data at VRAM z_HL >>5
	
;USED TO ACTUALLY PRINT/SHOW SPRITES ON THE SCREEN, 
SetSprite:	
	
	;MULTIPLY SPRITE NUMBER BY 8 (BECAUSE EACH SPRITE IN VERA HAS 8 BYTES WORTH OF ATTRIBUTE DATA), AND ADD $FC
	sta z_as				;STORE SPRITE NUMBER IN Z_AS
	lda #0					;EMPTY A
	asl z_as				;x2 - SHIFT THE BITS IN THE SPRITE NUMBER TO THE LEFT
	rol						;SHIFT THE BIT IN A TO THE LEFT, STORE ANY DROPPED BIT ON THE RIGHT BIT OF A
	asl z_as				;x4 - SHIFT THE BITS IN THE SPRITE NUMBER TO THE LEFT AGAIN
	rol						;SHIFT THE BIT IN A TO THE LEFT, STORE ANY DROPPED BIT ON THE RIGHT BIT OF A
	asl z_as				;x8 - SHIFT THE BIT IN THE SPRITE NUMBER TO THE LEFT FOR A 3RD TIME
	rol						;SHIFT THE BIT IN A TO THE LEFT, STORE ANY DROPPED BIT ON THE RIGHT BIT OF A
	
	clc						;CLEAR THE CARRY FLAG
	adc #$FC				;Sprite settings base $1FC00 - ADD HEX FC (DEC 252) TO THE VALUE IN A (MID BYTE)
	tay						;TRANSFER A TO Y, FOR THE MIDDLE BYTE OF THE VERA ADDRESS
	ldx z_as				;TRANSFER Z_AS TO X, FOR THE LOW BYTE OF THE VERA ADDRESS
	lda #$11				;$1FC00 with Autoinc ($11FC00), LOAD $11 IN A, FOR THE TOP BYTE OF THE VERA ADDRESS 
	
	jsr SetVeraHML			;Set VERA to point to Sprite Nr Settings = $1F5000 for Sprite 0
	
	;NOW LETS SET THE ATTRIBUTE DATA OF THE SPRITE
	lda z_l					;128/4>>5 = %100 per sprite
	sta v_d1				;LLLLLLLL - Low BYTE address bits - $68
	
	lda z_h					; Addr: $10000>>5 = $800 ; Addr: $1A000>>5 = $D00
	sta v_d1				;C---HHHH  C=colordepth 
							;H=High address bits - $08
	
	;FOR THE X POSITION I'VE LIMITED THE MOVEMENT OF SPRITES TO 0-255 (1 BYTE), BUT THE SCREEN IS 320 PIXELS WIDE. HOWEVER THE SPRITES ARE 16 PIXELS WIDE.
	;SO WE NEED TO ADD DEC 24 TO THE X POSITION, SO ALL SPRITES STAY CENTERED (24-256-24 + 16 PIXELS FOR A SPRITE = 320 PIXELS)
	STZ Z_IXH				;ZERO OUT Z_IX'S HIGH BYTE
	LDA Z_IXL				;LOAD THE LOW BYTE OF Z_IX INTO A	
	CLC						
	ADC #24					;ADD 32 TO THE X POSITION
	STA V_D1				;TRANSFER SPRITE PARAMETER TO VERA, XXXXXXXX - X Lbits
	
	LDA Z_IXH				;NOW LOAD THE HIGH BYTE OF Z_IX INTO A
	ROL			 			;ROTATE IN THE CARRY FROM THE ADC ABOVE (EVEN IF WE DIDN'T OVERFLOW)		
	STA V_D1				;TRANSFER SPRITE PARAMETER TO VERA, ------XX - X Hbits
	STZ Z_IXH				;RESET Z_IX HIGH BYTE BACK TO 0, TO PREVENT ANY X COORDINATE PROBLEMS LATER ON
	
	lda z_iyl
	sta v_d1				;YYYYYYYY - Y Lbits
	
	lda z_iyh
	sta v_d1				;------YY -  Y Hbits
	
	LDX VAR_SPRITESTATUS	;LOAD VAR_SPRITESTATUS. IT SHOULD CONTAIN THE SPRITE SIZE, 0=8X8, 1=16X16, 2=32X32, 3=64X64, 4=SPRITE DISABLED
	CPX #4
	BNE ENABLESPRITE
	;HERE WE'RE GOING TO DISABLE THE SPRITE, THEN SET THE VAR_SPRITESTATUS BACK 1 SO IT WILL PRINT A 16X16 SPRITE NEXT TIME, UNLESS SPECIFIED OTHERWISE
		; CCCCZZVH Z=Zdepth C=collision V=Vdepth H=Hdepth
	lda #%00000000	;Z=0 (MEANING DISABLE SPRITE)
	sta v_d1
	LDA #1
	STA VAR_SPRITESTATUS	;RESET VAR_SPRITESTATUS BACK TO 1, SO IT WILL PRINT A 16X16 SPRITE, 
	BRA CHECKSPRITE8X8		;JUMP TO CHECKSPRITE16X16, TO CHECK FOR THE SIZE OF THE SPRITE TO BE PRINTED
	
ENABLESPRITE:
		; CCCCZZVH Z=Zdepth C=collision V=Vdepth H=Hdepth
	LDA #%00001100	;Z=3 (MEANING IN FRONT OF LAYER 0 AND 1, ALWAYS ON TOP)
	STA v_d1	
	
CHECKSPRITE8X8:
	CPX #0
	BNE CHECKSPRITE16X16	;CHECK IF THE SPRITE IS 0, IF NOT THEN HOP TO CHECKSPRITE32X32
		; HHWWPPPP - H=Height W=Width P=Palette 
	LDA #%00000000			;SET SPRITE SIZE 8X8 (HW 0123 = 8/16/32/64 px)
	BRA DONECHECKSPRITESIZE	;JUMP TO DONECHECKSPRITESIZE
	
CHECKSPRITE16X16:
	CPX #1
	BNE CHECKSPRITE32X32	;CHECK IF THE SPRITE IS 1, IF NOT THEN HOP TO CHECKSPRITE64X64
	    ; HHWWPPPP - H=Height W=Width P=Palette 
	LDA #%01010000			;SET SPRITE SIZE 32X32 HW 0123 = 8/16/32/64 px)
	BRA DONECHECKSPRITESIZE	;JUMP TO DONECHECKSPRITESIZE
	
CHECKSPRITE32X32:
	CPX #2
	BNE CHECKSPRITE64X64	;CHECK IF THE SPRITE IS 1, IF NOT THEN HOP TO CHECKSPRITE64X64
	    ; HHWWPPPP - H=Height W=Width P=Palette 
	LDA #%10100000			;SET SPRITE SIZE 32X32 HW 0123 = 8/16/32/64 px)
	BRA DONECHECKSPRITESIZE	;JUMP TO DONECHECKSPRITESIZE
	
CHECKSPRITE64X64:
		; HHWWPPPP - H=Height W=Width P=Palette 
	LDA #%11110000			;SET SPRITE SIZE 64X64 (HW 0123 = 8/16/32/64 px)

DONECHECKSPRITESIZE:
	STA v_d1		;SEND SPRITE HIGHT, WIDTH AND PALLETE TO V_DL
	
	RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

NewLine:		
	stz CursorX				;Zero X
	inc CursorY				;Increase Y
	rts