z_Regs 				EQU $C0			;We need some Zeropage space for our work
	include "\srcAll\BasicMacros.asm"
SPpage 				EQU $0100

;VERA CONTROL REGISTERS
V_L  				EQU $9f20		;VERA_ADDR_HI  IIIIHHHH	20 Bit address H  I=Increment
V_M  				EQU $9f21		;VERA_ADDR_MID MMMMMMMM	20 Bit address M
V_H  				EQU $9f22		;VERA_ADDR_LO  LLLLLLLL	20 Bit address L
V_D1 				EQU $9f23		;VERA_DATA1    DDDDDDDD	Data Port 1
VERA_CTRL 			EQU $9F25 		;$9F25 R------A   R=Reset A=Address (port 1 / 2)

;NEW VERA CONTROL REGISTERS (X16 EMU R37)
DC_VIDEO			EQU $9F29		;VERA DC_VIDEO REGISTER (BITS=FS10-CMM	F=CURRENT FIELD, S=SPRITES ENABLE, 1=LAYER1 ENABLE, 0=LAYER0 ENABLE, C=CHROMA DISABLE, MM=OUTPUT MODE (01=VGA) )
DC_HSCALE			EQU $9F2A
DC_VSCALE			EQU $9F2B
L0_CONFIG			EQU $9F2D		;LAYER 0 CONFIG (BITS=HHWWTBCC		H=MAP HEIGHT W=MAP WIDTH T=T256C B=BITMAP MODE (1=BITMAP MODE, 0=TILE MODE), C=COLOR DEPTH (0=1BPP, 1=2BPP, 2=4BPP, 3=8BPP))
L0_MAPBASE			EQU $9F2E
L0_TILEBASE			EQU $9F2F
L0_HSCROLL_L		EQU $9F30
L0_HSCROLL_H		EQU $9F31
L0_VSCROLL_L		EQU $9F32
L0_VSCROLL_H		EQU $9F33

;VARIABLES - FOR (TEMP) DATA STORAGE
ZPADDR				EQU	$30			;POINTER TO SET THE START BYTE LOCATION FOR THE 2 BYTE MEM ADDRESS (WHICH WILL STORE THE STRINGS MEMORY START AT $0030 AND $0031)
CHRBUF				EQU	$10 		;$10.$2F - USED TO STORE PLAYER TYPED TEXT (MAX 30 CHARACHTERS, MUST END WITH VALUE 00 AFTER LAST LETTER). $2F CONTAINS AMOUNT OF LETTERS!
CHRBUFC				EQU	$2F
SEED				EQU	$7E			;ZERO PAGE $7E - USED FOR RANDOM NUMBER GENERATOR SEED ($7F ALSO RESERVED FOR TEMP DATA)
CursorX 			EQU $45
CursorY				EQU $46
VAR_SPRITESTATUS	EQU $47			;USED TO LET THE SETSPRITE SUB KNOW TO DRAW 8X8, 16X16, 32X32 OR 64X64 SPRITES TO THE SCREEN. 0=8X8, 1=16X16, 2=32X32, 3=64X64, 4=SPRITE DISABLED
VAR_MOVE1			EQU $48			;MOVEMENT DIRECTION OF THE ENEMY SPRITES, 0=RIGHT, 1=DOWN (PREVIOUS WAS RIGHT), 2=LEFT, 3=DOWN (PREVIOUS WAS LEFT)
VAR_MOVE2			EQU $49			;NUMBER OF PIXELS MOVED SINCE LAST MOVEMENT DIRECTION CHANGE
VAR_PLAYERACCEL		EQU $4D			;PLAYER ACCELERATION OVER THE SCREEN (-2 TO +2, REPRESENTED AS 0,1,2,3,4. 0 IS -2 AND 4 IS +2)

VAR_MATH			EQU $4E			;TEMP MATH VARIABLE FOR THE SQUARE AND SQUARE ROOT SUBROUTINES.
VAR_MATH2			EQU $4F			;TEMP MATH VARIABLE #2 FOR THE SQUAREROOT SUBROUTINE.
VAR_MATH3			EQU $50			;TEMP MATH VARIABLE #3 FOR THE SQUAREROOT SUBROUTINE.
VAR_X1				EQU $51
VAR_Y1				EQU $52			;USED TO STORE XY COORDINATES OF A SPRITE TO DO RANGECHECKING ON
VAR_X2				EQU $53
VAR_Y2				EQU $54			;USED TO STORE XY COORDINATES OF A SPRITE TO DO RANGECHECKING ON
VAR_LIVES			EQU $56			;KEEPS TRACK OF THE NUMBER OF LIVES THE PLAYER HAS.
VAR_LEVEL			EQU $57			;KEEPS TRACK WHICH LEVEL TO PLAY AT, DETERMINES NUMBER OF ENEMIES, TYPE OF ENEMIES AND DIFFICULTY
VAR_ENEMIES			EQU	$58			;VARIABLE TO KEEP TRACK OF THE NUMBER OF ENEMIES LEFT.

VAR_JIFFYC			EQU $4A			;JIFFY COUNTER (0-59), STORES THE LAST JIFFY AT THE END OF THE 2/3 JIFFY PAUSE ROUTINE, TO LIMIT THE GAME TO 24 FPS. A JIFFY IS 1/60TH OF A SECOND.
VAR_JIFFYS			EQU $4B			;SHORT PAUSE JIFFY FLAG, SET TO 1 WHEN A 2 JIFFY PAUSE WAS DONE, AND SET TO 0 WHEN A 3 JIFFY PAUSE WAS DONE IN THE WAITGAMETICK SUBROUTINES
VAR_GAMETICK		EQU $4C			;COUNTER FOR THE CURRENT GAME TICK (0-23), FOR 24 FPS.

SPRITEARRAY1H 		EQU $04			;HIGH BYTE OF THE SPRITE DATA ARRAY 1 (ENEMY SPRITES 0-34)
SPRITEARRAY2H 		EQU $05			;HIGH BYTE OF THE SPRITE DATA ARRAY 2 (ENEMY SPRITES 35-54)
SPRITEARRAY3H 		EQU $06			;HIGH BYTE OF THE SPRITE DATA ARRAY 3 (PLAYER SPRITE, PLAYER MISSILE AND 5X ENEMY LASER)

;LABELS FOR KERNAL FUNCTIONS, HEX VALUES AND MEMORY ADDRESSES.
KENTER				EQU	$0D			;ENTER KEY ON KEYBOARD IS #0D
CR					EQU	$0D			;CR $0D IS CARRIAGE RETURN
GETIN				EQU $FFE4		;GETIN reads a single byte from default input
JOYSTICK_SCAN		EQU $FF53		;COMMANDER X16 KERNALL CALL, JOYSTICK SCAN, SCAN'S FOR THE PRESENCE OF JOYSTICKS
JOYSTICK_GET		EQU $FF56		;COMMANDER X16 KERNALL CALL, JOYSTICK_GET. GETS THE BUTTONS PRESSED ON THE JOYSTICK (THE JOYSTICK SET IN A WHEN CALLING THE CALL)
									;returns #0 if no key is pressed = non-blocking
CLOCKAPI			EQU $FF50		;COMMANDER X16 KERNAL_API CALL CLOCK_GET_DATE_TIME, RESULT: YEAR=$02,MONTH=$03,DAY=$04,HOURS=$05,MINUTES=$06,SECONDS=$07,JIFFIES=$08.
CLOCKSEC			EQU $07
CLOCKJIFFY			EQU $08

;---------------------------------------------
;COMMANDER X16 - MEMORY MAP SUMMERIZED
;---------------------------------------------
;$0000 - $000F 		!COMMANDER X16 VARIABLES USED BY THE NEW API CALLS. TRY NOT TO USE!
;$0010 - $002F		CHARACTER BUFFER FOR USER TYPED TEXT (30 CHAR MAX)
;$0030 - $0044 		TEMP DATA USED BY VARIOUS FUNCTIONS AND SUBROUTINES, NO FIXED ASSIGNMENT, 20 BYTES. (ZPADDR)
;$0045 - $0056		VARIABLES 
;$0057 - $007D		FREE
;$007E - $007F		RANDOMIZER SEED VARIABLES

;$0080 - $00FF		!COMMANDER X16 KERNAL AND BASIC ZERO PAGE VARIABLES, DO NOT USE! CAN BE USED IN ASM THAT ONLY USES KERNAL, NO BASIC OR FLOATING POINT.
;* $00C0 - $00D0	USED BY CHIBIAKUMAS Z_REGS VARIABLES (BASICFUNCTIONS.ASM) THAT ARE USED AS A BASE FOR THIS PROGRAM (17 BYTES)
;* $00DA - $00DF	USED MY MON1HEX ROUTINE

;$0100 - $01FF		!6502 CPU STACK VALUES, DO NOT USE!
;$0200 - $03FF		!COMMANDER X16 KERNAL AND BASIC VARIABLES AND VECTORS, DO NOT USE!
;$0400 - $04FF		ENEMY SPRITE DATA ARRAY	1 (SPRITES 0-34), 35X6 BYTES = 210 BYTES USED OF 256 BYTES (THE REST ARE RESERVED JUST IN CASE)
;$0500 - $05FF		ENEMY SPRITE DATA ARRAY	2 (SPRITES 35-54), 20X6 BYTES = 120 BYTES USED	OF 256 BYTES (THE REST ARE RESERVED JUST IN CASE)
;$0600 - $0605		PLAYER SPRITE DATA (SPRITE 55), 6 BYTES
;$0606 - $060B		PLAYER MISSLE SPRITE DATA (SPRITE 56), 6 BYTES
;$060C - $0629		ENEMY LASER SPRITE DATA (SPRITE 60-64) 5X6 BYTES = 30 BYTES
;$062A - $07FF		FREE	
;$0801 - 			ASSEMBLED PROGRAM CODE

;$9F00 - $9FFF		!COMMANDER X16 I/O AREA (256 BYTES), DO NOT USE!
;$A000 - $BFFF		!COMMANDER X16 BANKED RAM (8KB WINDOW, 256 BANKS AVAILABLE, TOTAL 2 MB)
;$C000 - $FFFF		!COMMANDER X16 BANKED ROM (16KB WINDOW, 8 BANKS AVAILABLE, TOTAL 128 KB)
;* $FF62 - $FFF3	!COMMANDER X16, KERNAL API CALLS FOR COMMODORE 64/128 COMPATIBILITY.!


;---------------------------------------------
;COMMANDER X16 - HARDWARE SPRITE USAGE
;---------------------------------------------
;SPRITES 0  - 54	ENEMY SPRITES (55 MAX ON SCREEN ENEMIES)
;SPRITES 55 		PLAYER SPRITE
;SPRITE  56			PLAYER MISSLE SPRITE (ONLY 1 ON SCREEN AT A TIME)
;SPRITE  57 - 61	ENEMY LASER SPRITE (5 MAX ON SCREEN AT A TIME)
;SPRITE  62 - 64	PLAYER SPRITE'S USED TO SHOW AMOUNT OF LIVES LEFT
;SPRITE  65 - 66	TITLE SCREEN SPRITES

;---------------------------------------------
;COMMANDER X16 - SPRITE DATA ARRAY
;---------------------------------------------
; BYTE 1 = X POSITION OF SPRITE (0-255)
; BYTE 2 = Y POSITION OF SPRITE (0-255)
; BYTE 3 = SPRITE TYPE (0=PLAYER, 1=PLAYER MISSLE, 2=ENEMY LASER, 3=ENEMY 1, 4=ENEMY 2.....)
; BYTE 4 = SPRITE STATE (000000SS) - S = STATE (2=ALIVE, 1=DIEING, 00=DEAD),
; BYTE 5 = VERA VRAM START ADDRESS OF SPRITE
; BYTE 6 = ANIMATION FRAME NUMBER (000000FF) - F = ANIMATION FRAME NUMBER TO SHOW (0-4)


	macro SetVeraAddr,val		;SetVeraHML $123456
		lda #(\val>>16 & $FF)		;$12
		sta V_H	
		
		lda #(\val>>8 & $FF)		;$34
		sta V_M
		
		lda #(\val & $FF)			;$56
		sta V_L
	endm


*=$0801
        BYTE $0E,$08,$0A,$00,$9E,$20,$28,$32,$30,$36,$34,$29,$00,$00,$00  ;BASIC HEADER, START PROGRAM AT $0810
*=$0810
	
	;FIRST THING FIRST, RESET THE STACK POINTER
	LDX #$FF				;LOAD 255 IN THE X REGISTER
	TXS						;TRANSFER THE VALUE OF X TO THE STACK (RESETTING THE STACK POINTER TO $FF)
	
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	;Screen Setup

		 ;R-----DA			R=RESET, D=DCSEL, A=ADDRSEL
	LDA #%00000000
	STA VERA_CTRL
	
		 ;FS10-CMM		F=CURRENT FIELD, S=SPRITES ENABLE, 1=LAYER1 ENABLE, 0=LAYER0 ENABLE, C=CHROMA DISABLE, MM=OUTPUT MODE (01=VGA)
	LDA #%01010001
	STA DC_VIDEO

	LDA #64
	STA DC_HSCALE			;THIS EFFECTIVELY SETS THE SCREEN TO A HEIGHT OF HALF THE MAXIMUM OF 640 = 320 PIXELS
	LDA #64
	STA DC_VSCALE			;THIS EFFECTIVELY SETS THE SCREEN TO A WIDTH OF HALF THE MAXIMUM OF 480 = 240 PIXELS
	
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	;Palette Setup
	SetVeraAddr $11FA00	;$40200	Color 0 Green/Blue
	
	ldx #2					;2 bytes x 256 entries ($GB $-R )
	ldy #0
	lda #<palette
	sta z_l
	lda #>palette
	sta z_h
	
PaletteLoop:	
	lda (z_hl),y		
	sta V_D1		
	iny
	bne PaletteLoop
	inc z_h
	dex
	bne PaletteLoop
	
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	; VERA Layer 0 Setup
	
	     ;HHWWTBCC		H=MAP HEIGHT W=MAP WIDTH T=T256C B=BITMAP MODE (1=BITMAP MODE, 0=TILE MODE), C=COLOR DEPTH (0=1BPP, 1=2BPP, 2=4BPP, 3=8BPP)
	LDA #%00000110
	STA L0_CONFIG
	
	STZ L0_MAPBASE
	STZ L0_TILEBASE
	STZ L0_HSCROLL_L
	STZ L0_HSCROLL_H
	STZ L0_VSCROLL_L
	STZ L0_VSCROLL_H

		
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	;CLEAR THE SCREEN 
	;DRAW BLACK PIXELS ON THE WHOLE 320X240 PIXELS
	JSR CLEANSCREENPIXELS
	JSR RNDINIT					;INITILIZE THE RANDOM NUMBER GENERATOR
	
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
		
	;Copy Sprite BMP to Vram

	SetVeraAddr $110000	;Dest Addr for Sprite Bitmaps
	
	;LOADING OF 16X16 PIXEL SPRITES TO VRAM
	LoadPair z_hl,sprite161
	LoadPair z_bc,(sprite161_end-sprite161)
	jsr VLDIR				;Copy Sprite data to VRAM (16X16 PIXELS, FRAME 1)
	
	LoadPair z_hl,sprite162
	LoadPair z_bc,(sprite162_end-sprite162)
	jsr VLDIR				;Copy Sprite data to VRAM (16X16 PIXELS, FRAME 2)
	
	LoadPair z_hl,sprite163
	LoadPair z_bc,(sprite163_end-sprite163)
	jsr VLDIR				;Copy Sprite data to VRAM (16X16 PIXELS, FRAME 3)
	
	LoadPair z_hl,sprite164
	LoadPair z_bc,(sprite164_end-sprite164)
	jsr VLDIR				;Copy Sprite data to VRAM (16X16 PIXELS, FRAME 4)
	
	;LOADING OF 8X8 PIXEL SPRITES TO VRAM
	LoadPair z_hl,sprite81
	LoadPair z_bc,(sprite81_end-sprite81)
	jsr VLDIR				;Copy Sprite data to VRAM (8X8 PIXELS, FRAME 1)
	
	LoadPair z_hl,sprite82
	LoadPair z_bc,(sprite82_end-sprite82)
	jsr VLDIR				;Copy Sprite data to VRAM (8X8 PIXELS, FRAME 2)
	
	LoadPair z_hl,sprite83
	LoadPair z_bc,(sprite83_end-sprite83)
	jsr VLDIR				;Copy Sprite data to VRAM (8X8 PIXELS, FRAME 3)
	
	LoadPair z_hl,sprite84
	LoadPair z_bc,(sprite84_end-sprite84)
	jsr VLDIR				;Copy Sprite data to VRAM (8X8 PIXELS, FRAME 4)
	
	;AND FINALLY, LOADING OF THE TWO 32X32 SPRITES TO MAKE UP THE TITLE SCREEN
	LoadPair z_hl,sprite32
	LoadPair z_bc,(sprite32_end-sprite32)
	jsr VLDIR	
	
;---------------------------------------------
TITLESCREEN:
	;HERE WE'LL SHOW THE TITLE SCREEN AND ASK THE PLAYER TO PRESS A KEY!
	;LOAD THE TWO SPRITES TO SHOW ON THE SCREEN FOR THE TITLE BITMAP
	LDA #95
	STA Z_IXL					;STORE THE X POSITION TO Z_IXL FOR THE SPRITE PRINTING.
	LDA #50
	STA Z_IYL					;STORE THE Y POSITION TO Z_IXL FOR THE SPRITE PRINTING.
	LDA #88
	STA Z_L						;STORE THE VERA VRAM ADDRESS TO LOAD TO Z_L (LOW BYTE OF Z_HL)
	LDA #$08
	STA Z_H						;LOAD $08 AND STORE IT TO THE HIGH BYTE OF Z_HL
	LDA #2
	STA VAR_SPRITESTATUS		;STORE 2 IN THE SPRITE STATUS VARIABLE, TO LET SETSPRITE KNOW WE'RE PRINTING 32X32 SPRITES
	LDA #65						;LOAD HARDWARE SPRITE NUMBER
	JSR SETSPRITE				;ACTUALLY DRAW THE SPRITE ON SCREEN.
	
	LDA #127
	STA Z_IXL					;STORE THE X POSITION TO Z_IXL FOR THE SPRITE PRINTING.
	LDA #50
	STA Z_IYL					;STORE THE Y POSITION TO Z_IXL FOR THE SPRITE PRINTING.
	LDA #104
	STA Z_L						;STORE THE VERA VRAM ADDRESS TO LOAD TO Z_L (LOW BYTE OF Z_HL)
	LDA #$08
	STA Z_H						;LOAD $08 AND STORE IT TO THE HIGH BYTE OF Z_HL
	LDA #2
	STA VAR_SPRITESTATUS		;STORE 2 IN THE SPRITE STATUS VARIABLE, TO LET SETSPRITE KNOW WE'RE PRINTING 32X32 SPRITES
	LDA #66						;LOAD HARDWARE SPRITE NUMBER
	JSR SETSPRITE				;ACTUALLY DRAW THE SPRITE ON SCREEN.

	LDA #1
	STA VAR_SPRITESTATUS		;SET VAR_SPRITESTATUS BACK TO 1, SINCE WE'LL BE PRINTING 16X16 SPRITES MOSTLY FROM HERE ON OUT.
	
	LDA #<TITLETEXT				
	STA z_L
	LDA #>TITLETEXT
	STA z_H
	JSR PrintString				;PRINT THE TITLE SCREEN TEXT TO THE SCREEN
	LDA #2						
	STA VAR_LIVES				;STORE THE NUMBER OF LIVES IN VAR_LIVES (3 LIVES IN TOTAL)
	STZ VAR_LEVEL				;RESET THE LEVEL NUMBER T0 0, STARTING OF AT LEVEL 0
	JSR SHOWLIVES				;SHOW THE AMOUNT OF LIVES LEFT ON SCREEN, AND WAIT FOR USER INPUT
	;NOW THAT USER INPUT HAS BEEN GIVEN, START THE GAME
	JSR INITVARS				;INITIALIZE GAME VARIABLES, RESET EVERYTHING ON THE SCREEN
	
GAMEPREP:
	JSR LOADENEMYPATTERN		;LOAD THE SPRITE DATA FOR THE LEVEL
	
;---------------------------------------------	
MainLoop:						;THE MAIN GAMELOOP HERE
	
	;CHECK IF THE PLAYER PRESSED ANY KEYS AND DEAL WITH IT
	JSR HANDLEUSERINPUT
	
	;THIS HAS TO RUN\UPDATE 24 TIMES PER SECOND (EVERY GAME TICK)
	JSR CHECKENEMIESLEFT		;CHECK NOW MANY ENEMIES ARE LEFT AND PUT THE NUMBER IN VAR_ENEMIES
	JSR MOVEENEMIES				;MOVE THE ENEMIES ON THE SCREEN
	JSR MOVEWEAPONS				;MOVE THE PLAYER MISSILE AND ENEMY LASERS UP OR DOWN OF KILL THEM IF THEY MOVE OFF SCREEN
	
	;THIS HAS TO UPDATE 8 FRAMES PER SECOND (EVERY 3 GAME TICKS)
	LDA #<THIRDTICK
	STA ZPADDR					;STORE TO ZPADDR+0 FOR NOW
	LDA #>THIRDTICK				;LOAD THIRDFRAME LOOKUP TABLE HIGH BYTE
	STA ZPADDR+1				;STORE TO ZPADDR+1
	LDY #0						;LOAD 0 IN Y, AS A LOOP COUNTER
TICKCHECK_L3:	
	LDA (ZPADDR),Y				;LOAD THE VALUE FROM THE LOOKUP TABLE, OFFSET BY Y
	CMP VAR_GAMETICK			;COMPARE IT TO THE CURRENT GAMETICK
	BEQ TICKCHECK_L3_RUN		;RUN THE FUNCTION
	CMP #255					;IF IT DOESN'T MATCH, CHECK IF IT'S 255, IF SO WE NEED TO END THE LOOP
	BEQ TICKCHECK_L3_DONE		;IF IT IS 255, THEN THE LOOKUP TABLE IS OVER, END THE LOOP
	INY
	BRA TICKCHECK_L3			;OTHERWISE RUN THE LOOP AGAIN
TICKCHECK_L3_RUN:
	JSR UPDATEANIMATIONS		;UPDATE ANIMATION FRAMES (8 TIMES PER SECOND, OR EVERY 3 GAMETICKS)
TICKCHECK_L3_DONE:

	;THIS HAS TO RUN EVERY SECOND (EVERY 24 GAME TICKS)
	LDA VAR_GAMETICK
	CMP #23
	BNE TICKCHECK_L24_DONE		;IF IT'S NOT THE 24TH GAME TICK, THEN SKIP OVER IT
	JSR HANDLEENEMYFIRE			;CHECK IF THE ENEMY NEEDS TO FIRE A LASER BLAST AND DO SO!
TICKCHECK_L24_DONE:

	JSR DRAWSPRITES				;DRAW THE (ACTIVE) SPRITES TO THE SCREEN WITH THEIR CORRECT XY POSITIONS !!	
	
	JSR RANGECHECKING_MS_EN		;DO RANGE/COLLISION CHECKING OF THE PLAYER MISSILE VS THE ENEMY SPRITES, AND DEAL WITH THE CONSEQUENCES.
	JSR RANGECHECKING_PL_EN		;DO RANGE CHECKING ON THE PLAYER VS THE ENEMY SPRITES, AND LOSE A LIFE AND RETRY THE LEVEL IF IT DOES 
	JSR RANGECHECKING_PL_LA		;DO RANGE CHECKING ON THE PLAYER VS THE ENEMY LASERS.
	
	JSR CHECKLVLCOMPLETE		;CHECK IF THE LEVEL IS COMPLETE, AND IF SO, LOAD THE NEXT LEVEL!
	
	JSR HANDLEDEATH				;DEAL WITH A DEATH CONDITION (PLAYER KILLED, OR ENEMY'S AT THE Y=220 PIXELS)
		
	JSR WAITGAMETICK			;WAIT A BIT BEFORE DOING THE NEXT GAME LOOP. SLOWS THE GAME DOWN TO 24 FPS, AND UPDATES VAR_GAMETICK

	jmp MainLoop				;JUMP BACK TO DO THE NEXT GAME LOOP

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

	include "\Sources\x16-ReverseDirection-r37\External-Includes.asm"
